@@ -24,6 +24,7 @@
 #include "lib/queue.h"
 #include "lib/stream.h"
 #include "lib/zclient.h"
+#include "lib/printfrr.h"
 
 #include "lib/bfd.h"
 
@@ -55,10 +56,10 @@ static struct zclient *zclient;
 /*
  * Prototypes
  */
-static int _ptm_msg_address(struct stream *msg, struct sockaddr_any *sa);
+static int _ptm_msg_address(struct stream *msg, int family, const void *addr);
 
 static void _ptm_msg_read_address(struct stream *msg, struct sockaddr_any *sa);
-static int _ptm_msg_read(struct stream *msg, int command,
+static int _ptm_msg_read(struct stream *msg, int command, vrf_id_t vrf_id,
 			 struct bfd_peer_cfg *bpc, struct ptm_client **pc);
 
 static struct ptm_client *pc_lookup(uint32_t pid);
@@ -72,26 +73,26 @@ static struct ptm_client_notification *pcn_lookup(struct ptm_client *pc,
 static void pcn_free(struct ptm_client_notification *pcn);
 
 
-static void bfdd_dest_register(struct stream *msg);
-static void bfdd_dest_deregister(struct stream *msg);
+static void bfdd_dest_register(struct stream *msg, vrf_id_t vrf_id);
+static void bfdd_dest_deregister(struct stream *msg, vrf_id_t vrf_id);
 static void bfdd_client_register(struct stream *msg);
 static void bfdd_client_deregister(struct stream *msg);
 
 /*
  * Functions
  */
-#ifdef BFD_DEBUG
-static void debug_printbpc(const char *func, unsigned int line,
-			   struct bfd_peer_cfg *bpc);
-
-static void debug_printbpc(const char *func, unsigned int line,
-			   struct bfd_peer_cfg *bpc)
+static void debug_printbpc(const struct bfd_peer_cfg *bpc, const char *fmt, ...)
 {
-	char addr[3][128];
-	char timers[3][128];
+	char timers[3][128] = {};
+	char addr[3][128] = {};
+	char profile[128] = {};
+	char cbit_str[32];
+	char msgbuf[256];
+	va_list vl;
 
-	addr[0][0] = addr[1][0] = addr[2][0] = timers[0][0] = timers[1][0] =
-		timers[2][0] = 0;
+	/* Avoid debug calculations if it's disabled. */
+	if (bglobal.debug_zebra == false)
+		return;
 
 	snprintf(addr[0], sizeof(addr[0]), "peer:%s", satostr(&bpc->bpc_peer));
 	if (bpc->bpc_local.sa_sin.sin_family)
@@ -106,54 +107,86 @@ static void debug_printbpc(const char *func, unsigned int line,
 		snprintf(addr[2], sizeof(addr[2]), " vrf:%s", bpc->bpc_vrfname);
 
 	if (bpc->bpc_has_recvinterval)
-		snprintf(timers[0], sizeof(timers[0]), " rx:%lu",
-			 bpc->bpc_recvinterval);
+		snprintfrr(timers[0], sizeof(timers[0]), " rx:%" PRIu64,
+			   bpc->bpc_recvinterval);
 
 	if (bpc->bpc_has_txinterval)
-		snprintf(timers[1], sizeof(timers[1]), " tx:%lu",
-			 bpc->bpc_recvinterval);
+		snprintfrr(timers[1], sizeof(timers[1]), " tx:%" PRIu64,
+			   bpc->bpc_recvinterval);
 
 	if (bpc->bpc_has_detectmultiplier)
 		snprintf(timers[2], sizeof(timers[2]), " detect-multiplier:%d",
 			 bpc->bpc_detectmultiplier);
 
-	log_debug("%s:%d: %s %s%s%s%s%s%s", func, line,
-		  bpc->bpc_mhop ? "multi-hop" : "single-hop", addr[0], addr[1],
-		  addr[2], timers[0], timers[1], timers[2]);
+	snprintf(cbit_str, sizeof(cbit_str), " cbit:0x%02x", bpc->bpc_cbit);
+
+	if (bpc->bpc_has_profile)
+		snprintf(profile, sizeof(profile), " profile:%s",
+			 bpc->bpc_profile);
+
+	va_start(vl, fmt);
+	vsnprintf(msgbuf, sizeof(msgbuf), fmt, vl);
+	va_end(vl);
+
+	zlog_debug("%s [mhop:%s %s%s%s%s%s%s%s%s]", msgbuf,
+		   bpc->bpc_mhop ? "yes" : "no", addr[0], addr[1], addr[2],
+		   timers[0], timers[1], timers[2], cbit_str, profile);
 }
 
-#define DEBUG_PRINTBPC(bpc) debug_printbpc(__FILE__, __LINE__, (bpc))
-#else
-#define DEBUG_PRINTBPC(bpc)
-#endif /* BFD_DEBUG */
-
-static int _ptm_msg_address(struct stream *msg, struct sockaddr_any *sa)
+static void _ptm_bfd_session_del(struct bfd_session *bs, uint8_t diag)
 {
-	switch (sa->sa_sin.sin_family) {
+	if (bglobal.debug_peer_event)
+		zlog_debug("session-delete: %s", bs_to_string(bs));
+
+	/* Change state and notify peer. */
+	bs->ses_state = PTM_BFD_DOWN;
+	bs->local_diag = diag;
+	ptm_bfd_snd(bs, 0);
+
+	/* Session reached refcount == 0, lets delete it. */
+	if (bs->refcount == 0) {
+		/*
+		 * Sanity check: if there is a refcount bug, we can't delete
+		 * the session a user configured manually. Lets leave a
+		 * message here so we can catch the bug if it exists.
+		 */
+		if (CHECK_FLAG(bs->flags, BFD_SESS_FLAG_CONFIG)) {
+			zlog_err(
+				"ptm-del-session: [%s] session refcount is zero but it was configured by CLI",
+				bs_to_string(bs));
+		} else {
+			control_notify_config(BCM_NOTIFY_CONFIG_DELETE, bs);
+			bfd_session_free(bs);
+		}
+	}
+}
+
+static int _ptm_msg_address(struct stream *msg, int family, const void *addr)
+{
+	stream_putc(msg, family);
+
+	switch (family) {
 	case AF_INET:
-		stream_putc(msg, sa->sa_sin.sin_family);
-		stream_put_in_addr(msg, &sa->sa_sin.sin_addr);
+		stream_put(msg, addr, sizeof(struct in_addr));
 		stream_putc(msg, 32);
 		break;
 
 	case AF_INET6:
-		stream_putc(msg, sa->sa_sin6.sin6_family);
-		stream_put(msg, &sa->sa_sin6.sin6_addr,
-			   sizeof(sa->sa_sin6.sin6_addr));
+		stream_put(msg, addr, sizeof(struct in6_addr));
 		stream_putc(msg, 128);
 		break;
 
 	default:
-		return -1;
+		assert(0);
+		break;
 	}
 
 	return 0;
 }
 
-int ptm_bfd_notify(struct bfd_session *bs)
+int ptm_bfd_notify(struct bfd_session *bs, uint8_t notify_state)
 {
 	struct stream *msg;
-	struct sockaddr_any sac;
 
 	bs->stats.znotification++;
 
@@ -174,6 +207,7 @@ int ptm_bfd_notify(struct bfd_session *bs)
 	 *   - AF_INET6:
 	 *     - 16 bytes: ipv6
 	 *   - c: prefix length
+	 * - c: cbit
 	 *
 	 * Commands: ZEBRA_BFD_DEST_REPLAY
 	 *
@@ -183,27 +217,33 @@ int ptm_bfd_notify(struct bfd_session *bs)
 	stream_reset(msg);
 
 	/* TODO: VRF handling */
-	zclient_create_header(msg, ZEBRA_BFD_DEST_REPLAY, VRF_DEFAULT);
+	if (bs->vrf)
+		zclient_create_header(msg, ZEBRA_BFD_DEST_REPLAY, bs->vrf->vrf_id);
+	else
+		zclient_create_header(msg, ZEBRA_BFD_DEST_REPLAY, VRF_DEFAULT);
 
 	/* This header will be handled by `zebra_ptm.c`. */
 	stream_putl(msg, ZEBRA_INTERFACE_BFD_DEST_UPDATE);
 
 	/* NOTE: Interface is a shortcut to avoid comparing source address. */
-	stream_putl(msg, bs->ifindex);
+	if (bs->ifp != NULL)
+		stream_putl(msg, bs->ifp->ifindex);
+	else
+		stream_putl(msg, IFINDEX_INTERNAL);
 
 	/* BFD destination prefix information. */
-	if (BFD_CHECK_FLAG(bs->flags, BFD_SESS_FLAG_MH))
-		_ptm_msg_address(msg, &bs->mhop.peer);
-	else
-		_ptm_msg_address(msg, &bs->shop.peer);
+	_ptm_msg_address(msg, bs->key.family, &bs->key.peer);
 
 	/* BFD status */
-	switch (bs->ses_state) {
+	switch (notify_state) {
 	case PTM_BFD_UP:
 		stream_putl(msg, BFD_STATUS_UP);
 		break;
 
 	case PTM_BFD_ADM_DOWN:
+		stream_putl(msg, BFD_STATUS_ADMIN_DOWN);
+		break;
+
 	case PTM_BFD_DOWN:
 	case PTM_BFD_INIT:
 		stream_putl(msg, BFD_STATUS_DOWN);
@@ -215,34 +255,9 @@ int ptm_bfd_notify(struct bfd_session *bs)
 	}
 
 	/* BFD source prefix information. */
-	if (BFD_CHECK_FLAG(bs->flags, BFD_SESS_FLAG_MH)) {
-		_ptm_msg_address(msg, &bs->mhop.local);
-	} else {
-		if (bs->local_address.sa_sin.sin_family)
-			_ptm_msg_address(msg, &bs->local_address);
-		else if (bs->local_address.sa_sin.sin_family)
-			_ptm_msg_address(msg, &bs->local_ip);
-		else {
-			sac = bs->shop.peer;
-			switch (sac.sa_sin.sin_family) {
-			case AF_INET:
-				memset(&sac.sa_sin.sin_addr, 0,
-				       sizeof(sac.sa_sin.sin_family));
-				break;
-			case AF_INET6:
-				memset(&sac.sa_sin6.sin6_addr, 0,
-				       sizeof(sac.sa_sin6.sin6_family));
-				break;
+	_ptm_msg_address(msg, bs->key.family, &bs->key.local);
 
-			default:
-				assert(false);
-				break;
-			}
-
-			/* No local address found yet, so send zeroes. */
-			_ptm_msg_address(msg, &sac);
-		}
-	}
+	stream_putc(msg, bs->remote_cbit);
 
 	/* Write packet size. */
 	stream_putw_at(msg, 0, stream_get_endp(msg));
@@ -276,7 +291,7 @@ static void _ptm_msg_read_address(struct stream *msg, struct sockaddr_any *sa)
 		return;
 
 	default:
-		log_warning("ptm-read-address: invalid family: %d", family);
+		zlog_warn("ptm-read-address: invalid family: %d", family);
 		break;
 	}
 
@@ -284,12 +299,11 @@ stream_failure:
 	memset(sa, 0, sizeof(*sa));
 }
 
-static int _ptm_msg_read(struct stream *msg, int command,
+static int _ptm_msg_read(struct stream *msg, int command, vrf_id_t vrf_id,
 			 struct bfd_peer_cfg *bpc, struct ptm_client **pc)
 {
 	uint32_t pid;
-	uint8_t ttl __attribute__((unused));
-	uint8_t ifnamelen;
+	size_t ifnamelen;
 
 	/*
 	 * Register/Deregister/Update Message format:
@@ -318,6 +332,9 @@ static int _ptm_msg_read(struct stream *msg, int command,
 	 *       - 16 bytes: ipv6 address
 	 *     - c: ifname length
 	 *     - X bytes: interface name
+	 * - c: bfd_cbit
+	 * - c: profile name length.
+	 * - X bytes: profile name.
 	 *
 	 * q(64), l(32), w(16), c(8)
 	 */
@@ -330,10 +347,6 @@ static int _ptm_msg_read(struct stream *msg, int command,
 	STREAM_GETL(msg, pid);
 
 	*pc = pc_new(pid);
-	if (*pc == NULL) {
-		log_debug("ptm-read: failed to allocate memory");
-		return -1;
-	}
 
 	/* Register/update peer information. */
 	_ptm_msg_read_address(msg, &bpc->bpc_peer);
@@ -361,7 +374,18 @@ static int _ptm_msg_read(struct stream *msg, int command,
 	if (bpc->bpc_mhop) {
 		/* Read multihop source address and TTL. */
 		_ptm_msg_read_address(msg, &bpc->bpc_local);
-		STREAM_GETC(msg, ttl);
+		STREAM_GETC(msg, bpc->bpc_minimum_ttl);
+		if (bpc->bpc_minimum_ttl >= BFD_TTL_VAL
+		    || bpc->bpc_minimum_ttl == 0) {
+			zlog_warn("%s: received invalid TTL configuration %d",
+				  __func__, bpc->bpc_has_minimum_ttl);
+			bpc->bpc_minimum_ttl = BFD_DEF_MHOP_TTL;
+			bpc->bpc_has_minimum_ttl = false;
+		} else {
+			bpc->bpc_minimum_ttl =
+				(BFD_TTL_VAL + 1) - bpc->bpc_minimum_ttl;
+			bpc->bpc_has_minimum_ttl = true;
+		}
 	} else {
 		/* If target is IPv6, then we must obtain local address. */
 		if (bpc->bpc_ipv4 == false)
@@ -373,7 +397,7 @@ static int _ptm_msg_read(struct stream *msg, int command,
 		 */
 		STREAM_GETC(msg, ifnamelen);
 		if (ifnamelen >= sizeof(bpc->bpc_localif)) {
-			log_error("ptm-read: interface name is too big");
+			zlog_err("ptm-read: interface name is too big");
 			return -1;
 		}
 
@@ -381,29 +405,40 @@ static int _ptm_msg_read(struct stream *msg, int command,
 		if (bpc->bpc_has_localif) {
 			STREAM_GET(bpc->bpc_localif, msg, ifnamelen);
 			bpc->bpc_localif[ifnamelen] = 0;
-
-			/*
-			 * IPv6 link-local addresses must use scope id,
-			 * otherwise the session lookup will always fail
-			 * and we'll have multiple sessions showing up.
-			 *
-			 * This problem only happens with single hop
-			 * since it is not possible to have link-local
-			 * address for multi hop sessions.
-			 */
-			if (bpc->bpc_ipv4 == false
-			    && IN6_IS_ADDR_LINKLOCAL(
-				       &bpc->bpc_peer.sa_sin6.sin6_addr))
-				bpc->bpc_peer.sa_sin6.sin6_scope_id =
-					ptm_bfd_fetch_ifindex(bpc->bpc_localif);
 		}
 	}
+	if (vrf_id != VRF_DEFAULT) {
+		struct vrf *vrf;
+
+		vrf = vrf_lookup_by_id(vrf_id);
+		if (vrf) {
+			bpc->bpc_has_vrfname = true;
+			strlcpy(bpc->bpc_vrfname, vrf->name, sizeof(bpc->bpc_vrfname));
+		} else {
+			zlog_err("ptm-read: vrf id %u could not be identified",
+				 vrf_id);
+			return -1;
+		}
+	} else {
+		bpc->bpc_has_vrfname = true;
+		strlcpy(bpc->bpc_vrfname, VRF_DEFAULT_NAME, sizeof(bpc->bpc_vrfname));
+	}
+
+	STREAM_GETC(msg, bpc->bpc_cbit);
+
+	/* Handle profile names. */
+	STREAM_GETC(msg, ifnamelen);
+	bpc->bpc_has_profile = ifnamelen > 0;
+	if (bpc->bpc_has_profile) {
+		STREAM_GET(bpc->bpc_profile, msg, ifnamelen);
+		bpc->bpc_profile[ifnamelen] = 0;
+	}
 
 	/* Sanity check: peer and local address must match IP types. */
 	if (bpc->bpc_local.sa_sin.sin_family != 0
 	    && (bpc->bpc_local.sa_sin.sin_family
 		!= bpc->bpc_peer.sa_sin.sin_family)) {
-		log_warning("ptm-read: peer family doesn't match local type");
+		zlog_warn("ptm-read: peer family doesn't match local type");
 		return -1;
 	}
 
@@ -413,45 +448,53 @@ stream_failure:
 	return -1;
 }
 
-static void bfdd_dest_register(struct stream *msg)
+static void bfdd_dest_register(struct stream *msg, vrf_id_t vrf_id)
 {
 	struct ptm_client *pc;
-	struct ptm_client_notification *pcn;
 	struct bfd_session *bs;
 	struct bfd_peer_cfg bpc;
 
 	/* Read the client context and peer data. */
-	if (_ptm_msg_read(msg, ZEBRA_BFD_DEST_REGISTER, &bpc, &pc) == -1)
+	if (_ptm_msg_read(msg, ZEBRA_BFD_DEST_REGISTER, vrf_id, &bpc, &pc) == -1)
 		return;
 
-	DEBUG_PRINTBPC(&bpc);
+	debug_printbpc(&bpc, "ptm-add-dest: register peer");
 
 	/* Find or start new BFD session. */
 	bs = bs_peer_find(&bpc);
 	if (bs == NULL) {
 		bs = ptm_bfd_sess_new(&bpc);
 		if (bs == NULL) {
-			log_debug("ptm-add-dest: failed to create BFD session");
+			if (bglobal.debug_zebra)
+				zlog_debug(
+					"ptm-add-dest: failed to create BFD session");
 			return;
 		}
+
+		/* Protocol created peers are 'no shutdown' by default. */
+		bs->peer_profile.admin_shutdown = false;
 	} else {
-		/* Don't try to change echo/shutdown state. */
-		bpc.bpc_echo = BFD_CHECK_FLAG(bs->flags, BFD_SESS_FLAG_ECHO);
-		bpc.bpc_shutdown =
-			BFD_CHECK_FLAG(bs->flags, BFD_SESS_FLAG_SHUTDOWN);
+		/*
+		 * BFD session was already created, we are just updating the
+		 * current peer.
+		 *
+		 * `ptm-bfd` (or `HAVE_BFDD == 0`) is the only implementation
+		 * that allow users to set peer specific timers via protocol.
+		 * BFD daemon (this code) on the other hand only supports
+		 * changing peer configuration manually (through `peer` node)
+		 * or via profiles.
+		 */
+		if (bpc.bpc_has_profile)
+			bfd_profile_apply(bpc.bpc_profile, bs);
 	}
 
 	/* Create client peer notification register. */
-	pcn = pcn_new(pc, bs);
-	if (pcn == NULL) {
-		log_error("ptm-add-dest: failed to registrate notifications");
-		return;
-	}
+	pcn_new(pc, bs);
 
-	ptm_bfd_notify(bs);
+	ptm_bfd_notify(bs, bs->ses_state);
 }
 
-static void bfdd_dest_deregister(struct stream *msg)
+static void bfdd_dest_deregister(struct stream *msg, vrf_id_t vrf_id)
 {
 	struct ptm_client *pc;
 	struct ptm_client_notification *pcn;
@@ -459,21 +502,35 @@ static void bfdd_dest_deregister(struct stream *msg)
 	struct bfd_peer_cfg bpc;
 
 	/* Read the client context and peer data. */
-	if (_ptm_msg_read(msg, ZEBRA_BFD_DEST_DEREGISTER, &bpc, &pc) == -1)
+	if (_ptm_msg_read(msg, ZEBRA_BFD_DEST_DEREGISTER, vrf_id, &bpc, &pc) == -1)
 		return;
 
-	DEBUG_PRINTBPC(&bpc);
+	debug_printbpc(&bpc, "ptm-del-dest: deregister peer");
 
 	/* Find or start new BFD session. */
 	bs = bs_peer_find(&bpc);
 	if (bs == NULL) {
-		log_debug("ptm-del-dest: failed to find BFD session");
+		if (bglobal.debug_zebra)
+			zlog_debug("ptm-del-dest: failed to find BFD session");
 		return;
 	}
 
 	/* Unregister client peer notification. */
 	pcn = pcn_lookup(pc, bs);
-	pcn_free(pcn);
+	if (pcn != NULL) {
+		pcn_free(pcn);
+		return;
+	}
+
+	if (bglobal.debug_zebra)
+		zlog_debug("ptm-del-dest: failed to find BFD session");
+
+	/*
+	 * XXX: We either got a double deregistration or the daemon who
+	 * created this is no longer around. Lets try to delete it anyway
+	 * and the worst case is the refcount will detain us.
+	 */
+	_ptm_bfd_session_del(bs, BD_NEIGHBOR_DOWN);
 }
 
 /*
@@ -482,22 +539,17 @@ static void bfdd_dest_deregister(struct stream *msg)
  */
 static void bfdd_client_register(struct stream *msg)
 {
-	struct ptm_client *pc;
 	uint32_t pid;
 
 	/* Find or allocate process context data. */
 	STREAM_GETL(msg, pid);
 
-	pc = pc_new(pid);
-	if (pc == NULL) {
-		log_error("ptm-add-client: failed to register client: %u", pid);
-		return;
-	}
+	pc_new(pid);
 
 	return;
 
 stream_failure:
-	log_error("ptm-add-client: failed to register client");
+	zlog_err("ptm-add-client: failed to register client");
 }
 
 /*
@@ -514,21 +566,26 @@ static void bfdd_client_deregister(struct stream *msg)
 
 	pc = pc_lookup(pid);
 	if (pc == NULL) {
-		log_debug("ptm-del-client: failed to find client: %u", pid);
+		if (bglobal.debug_zebra)
+			zlog_debug("ptm-del-client: failed to find client: %u",
+				   pid);
 		return;
 	}
 
+	if (bglobal.debug_zebra)
+		zlog_debug("ptm-del-client: client pid %u", pid);
+
 	pc_free(pc);
 
 	return;
 
 stream_failure:
-	log_error("ptm-del-client: failed to deregister client");
+	zlog_err("ptm-del-client: failed to deregister client");
 }
 
-static int bfdd_replay(int cmd, struct zclient *zc, uint16_t len, vrf_id_t vid)
+static int bfdd_replay(ZAPI_CALLBACK_ARGS)
 {
-	struct stream *msg = zc->ibuf;
+	struct stream *msg = zclient->ibuf;
 	uint32_t rcmd;
 
 	STREAM_GETL(msg, rcmd);
@@ -536,10 +593,10 @@ static int bfdd_replay(int cmd, struct zclient *zc, uint16_t len, vrf_id_t vid)
 	switch (rcmd) {
 	case ZEBRA_BFD_DEST_REGISTER:
 	case ZEBRA_BFD_DEST_UPDATE:
-		bfdd_dest_register(msg);
+		bfdd_dest_register(msg, vrf_id);
 		break;
 	case ZEBRA_BFD_DEST_DEREGISTER:
-		bfdd_dest_deregister(msg);
+		bfdd_dest_deregister(msg, vrf_id);
 		break;
 	case ZEBRA_BFD_CLIENT_REGISTER:
 		bfdd_client_register(msg);
@@ -549,14 +606,15 @@ static int bfdd_replay(int cmd, struct zclient *zc, uint16_t len, vrf_id_t vid)
 		break;
 
 	default:
-		log_debug("ptm-replay: invalid message type %u", rcmd);
+		if (bglobal.debug_zebra)
+			zlog_debug("ptm-replay: invalid message type %u", rcmd);
 		return -1;
 	}
 
 	return 0;
 
 stream_failure:
-	log_error("ptm-replay: failed to find command");
+	zlog_err("ptm-replay: failed to find command");
 	return -1;
 }
 
@@ -576,12 +634,200 @@ static void bfdd_zebra_connected(struct zclient *zc)
 	stream_putl(msg, ZEBRA_BFD_DEST_REPLAY);
 	stream_putw_at(msg, 0, stream_get_endp(msg));
 
+	/* Ask for interfaces information. */
+	zclient_create_header(msg, ZEBRA_INTERFACE_ADD, VRF_DEFAULT);
+
+	/* Send requests. */
 	zclient_send_message(zclient);
 }
 
+static void bfdd_sessions_enable_interface(struct interface *ifp)
+{
+	struct bfd_session_observer *bso;
+	struct bfd_session *bs;
+	struct vrf *vrf;
+
+	vrf = vrf_lookup_by_id(ifp->vrf_id);
+	if (!vrf)
+		return;
+
+	TAILQ_FOREACH(bso, &bglobal.bg_obslist, bso_entry) {
+		bs = bso->bso_bs;
+		/* check vrf name */
+		if (bs->key.vrfname[0] &&
+		    strcmp(vrf->name, bs->key.vrfname))
+			continue;
+
+		/* If Interface matches vrfname, then bypass iface check */
+		if (vrf_is_backend_netns() || strcmp(ifp->name, vrf->name)) {
+			/* Interface name mismatch. */
+			if (strcmp(ifp->name, bs->key.ifname))
+				continue;
+		}
+
+		/* Skip enabled sessions. */
+		if (bs->sock != -1)
+			continue;
+
+		/* Try to enable it. */
+		bfd_session_enable(bs);
+	}
+}
+
+static void bfdd_sessions_disable_interface(struct interface *ifp)
+{
+	struct bfd_session_observer *bso;
+	struct bfd_session *bs;
+
+	TAILQ_FOREACH(bso, &bglobal.bg_obslist, bso_entry) {
+		bs = bso->bso_bs;
+
+		if (bs->ifp != ifp)
+			continue;
+
+		/* Skip disabled sessions. */
+		if (bs->sock == -1) {
+			bs->ifp = NULL;
+			continue;
+		}
+
+		bfd_session_disable(bs);
+
+	}
+}
+
+void bfdd_sessions_enable_vrf(struct vrf *vrf)
+{
+	struct bfd_session_observer *bso;
+	struct bfd_session *bs;
+
+	/* it may affect configs without interfaces */
+	TAILQ_FOREACH(bso, &bglobal.bg_obslist, bso_entry) {
+		bs = bso->bso_bs;
+		/* update name */
+		if (bs->vrf && bs->vrf == vrf) {
+			if (!strmatch(bs->key.vrfname, vrf->name))
+				bfd_session_update_vrf_name(bs, vrf);
+		}
+		if (bs->vrf)
+			continue;
+		if (bs->key.vrfname[0] &&
+		    strcmp(vrf->name, bs->key.vrfname))
+			continue;
+		/* need to update the vrf information on
+		 * bs so that callbacks are handled
+		 */
+		bs->vrf = vrf;
+		/* Skip enabled sessions. */
+		if (bs->sock != -1)
+			continue;
+		/* Try to enable it. */
+		bfd_session_enable(bs);
+	}
+}
+
+void bfdd_sessions_disable_vrf(struct vrf *vrf)
+{
+	struct bfd_session_observer *bso;
+	struct bfd_session *bs;
+
+	TAILQ_FOREACH(bso, &bglobal.bg_obslist, bso_entry) {
+		bs = bso->bso_bs;
+		if (bs->key.vrfname[0] &&
+		    strcmp(vrf->name, bs->key.vrfname))
+			continue;
+		/* Skip disabled sessions. */
+		if (bs->sock == -1)
+			continue;
+
+		bfd_session_disable(bs);
+	}
+}
+
+static int bfd_ifp_destroy(struct interface *ifp)
+{
+	if (bglobal.debug_zebra)
+		zlog_debug("zclient: delete interface %s", ifp->name);
+
+	bfdd_sessions_disable_interface(ifp);
+
+	return 0;
+}
+
+static int bfdd_interface_vrf_update(ZAPI_CALLBACK_ARGS)
+{
+	struct interface *ifp;
+	vrf_id_t nvrfid;
+
+	ifp = zebra_interface_vrf_update_read(zclient->ibuf, vrf_id, &nvrfid);
+	if (ifp == NULL)
+		return 0;
+
+	if_update_to_new_vrf(ifp, nvrfid);
+
+	return 0;
+}
+
+static void bfdd_sessions_enable_address(struct connected *ifc)
+{
+	struct bfd_session_observer *bso;
+	struct bfd_session *bs;
+	struct prefix prefix;
+
+	TAILQ_FOREACH(bso, &bglobal.bg_obslist, bso_entry) {
+		/* Skip enabled sessions. */
+		bs = bso->bso_bs;
+		if (bs->sock != -1)
+			continue;
+
+		/* Check address. */
+		prefix = bso->bso_addr;
+		prefix.prefixlen = ifc->address->prefixlen;
+		if (prefix_cmp(&prefix, ifc->address))
+			continue;
+
+		/* Try to enable it. */
+		bfd_session_enable(bs);
+	}
+}
+
+static int bfdd_interface_address_update(ZAPI_CALLBACK_ARGS)
+{
+	struct connected *ifc;
+	char buf[64];
+
+	ifc = zebra_interface_address_read(cmd, zclient->ibuf, vrf_id);
+	if (ifc == NULL)
+		return 0;
+
+	if (bglobal.debug_zebra)
+		zlog_debug("zclient: %s local address %s",
+			   cmd == ZEBRA_INTERFACE_ADDRESS_ADD ? "add"
+							      : "delete",
+			   prefix2str(ifc->address, buf, sizeof(buf)));
+
+	if (cmd == ZEBRA_INTERFACE_ADDRESS_ADD)
+		bfdd_sessions_enable_address(ifc);
+	else
+		connected_free(&ifc);
+
+	return 0;
+}
+
+static int bfd_ifp_create(struct interface *ifp)
+{
+	if (bglobal.debug_zebra)
+		zlog_debug("zclient: add interface %s", ifp->name);
+
+	bfdd_sessions_enable_interface(ifp);
+
+	return 0;
+}
+
 void bfdd_zclient_init(struct zebra_privs_t *bfdd_priv)
 {
-	zclient = zclient_new_notify(master, &zclient_options_default);
+	if_zapi_callbacks(bfd_ifp_create, NULL, NULL, bfd_ifp_destroy);
+	zclient = zclient_new(master, &zclient_options_default);
 	assert(zclient != NULL);
 	zclient_init(zclient, ZEBRA_ROUTE_BFD, 0, bfdd_priv);
 
@@ -594,6 +840,27 @@ void bfdd_zclient_init(struct zebra_privs_t *bfdd_priv)
 
 	/* Send replay request on zebra connect. */
 	zclient->zebra_connected = bfdd_zebra_connected;
+
+	/* Learn about interface VRF. */
+	zclient->interface_vrf_update = bfdd_interface_vrf_update;
+
+	/* Learn about new addresses being registered. */
+	zclient->interface_address_add = bfdd_interface_address_update;
+	zclient->interface_address_delete = bfdd_interface_address_update;
+}
+
+void bfdd_zclient_register(vrf_id_t vrf_id)
+{
+	if (!zclient || zclient->sock < 0)
+		return;
+	zclient_send_reg_requests(zclient, vrf_id);
+}
+
+void bfdd_zclient_unregister(vrf_id_t vrf_id)
+{
+	if (!zclient || zclient->sock < 0)
+		return;
+	zclient_send_dereg_requests(zclient, vrf_id);
 }
 
 void bfdd_zclient_stop(void)
@@ -633,8 +900,6 @@ static struct ptm_client *pc_new(uint32_t pid)
 
 	/* Allocate the client data and save it. */
 	pc = XCALLOC(MTYPE_BFDD_CONTROL, sizeof(*pc));
-	if (pc == NULL)
-		return NULL;
 
 	pc->pc_pid = pid;
 	TAILQ_INSERT_HEAD(&pcqueue, pc, pc_entry);
@@ -645,9 +910,6 @@ static void pc_free(struct ptm_client *pc)
 {
 	struct ptm_client_notification *pcn;
 
-	if (pc == NULL)
-		return;
-
 	TAILQ_REMOVE(&pcqueue, pc, pc_entry);
 
 	while (!TAILQ_EMPTY(&pc->pc_pcnqueue)) {
@@ -680,8 +942,6 @@ static struct ptm_client_notification *pcn_new(struct ptm_client *pc,
 
 	/* Save the client notification data. */
 	pcn = XCALLOC(MTYPE_BFDD_NOTIFICATION, sizeof(*pcn));
-	if (pcn == NULL)
-		return NULL;
 
 	TAILQ_INSERT_HEAD(&pc->pc_pcnqueue, pcn, pcn_entry);
 	pcn->pcn_pc = pc;
@@ -711,14 +971,19 @@ static void pcn_free(struct ptm_client_notification *pcn)
 	struct ptm_client *pc;
 	struct bfd_session *bs;
 
-	if (pcn == NULL)
-		return;
-
 	/* Handle session de-registration. */
 	bs = pcn->pcn_bs;
 	pcn->pcn_bs = NULL;
 	bs->refcount--;
 
+	/* Log modification to users. */
+	if (bglobal.debug_zebra)
+		zlog_debug("ptm-del-session: [%s] refcount=%" PRIu64,
+			   bs_to_string(bs), bs->refcount);
+
+	/* Set session down. */
+	_ptm_bfd_session_del(bs, BD_NEIGHBOR_DOWN);
+
 	/* Handle ptm_client deregistration. */
 	pc = pcn->pcn_pc;
 	pcn->pcn_pc = NULL;
