@@ -31,12 +31,14 @@ extern void bgp_bfd_register_peer(struct peer *peer);
 
 extern void bgp_bfd_deregister_peer(struct peer *peer);
 
+extern void bgp_bfd_reset_peer(struct peer *peer);
+
 extern void bgp_bfd_peer_config_write(struct vty *vty, struct peer *peer,
 				      char *addr);
 
-extern void bgp_bfd_show_info(struct vty *vty, struct peer *peer,
-			      uint8_t use_json, json_object *json_neigh);
+extern void bgp_bfd_show_info(struct vty *vty, struct peer *peer, bool use_json,
+			      json_object *json_neigh);
 
-extern int bgp_bfd_is_peer_multihop(struct peer *peer);
+extern bool bgp_bfd_is_peer_multihop(struct peer *peer);
 
 #endif /* _QUAGGA_BGP_BFD_H */
