@@ -26,6 +26,8 @@
 #include "queue.h"
 #include "filter.h"
 #include "stream.h"
+#include "jhash.h"
+#include "frrstr.h"
 
 #include "bgpd/bgpd.h"
 #include "bgpd/bgp_community.h"
@@ -35,6 +37,62 @@
 #include "bgpd/bgp_regex.h"
 #include "bgpd/bgp_clist.h"
 
+/* Calculate new sequential number. */
+static int64_t bgp_clist_new_seq_get(struct community_list *list)
+{
+	int64_t maxseq;
+	int64_t newseq;
+	struct community_entry *entry;
+
+	maxseq = 0;
+
+	for (entry = list->head; entry; entry = entry->next) {
+		if (maxseq < entry->seq)
+			maxseq = entry->seq;
+	}
+
+	newseq = ((maxseq / 5) * 5) + 5;
+
+	return (newseq > UINT_MAX) ? UINT_MAX : newseq;
+}
+
+/* Return community-list entry which has same seq number. */
+static struct community_entry *bgp_clist_seq_check(struct community_list *list,
+						   int64_t seq)
+{
+	struct community_entry *entry;
+
+	for (entry = list->head; entry; entry = entry->next)
+		if (entry->seq == seq)
+			return entry;
+	return NULL;
+}
+
+static uint32_t bgp_clist_hash_key_community_list(const void *data)
+{
+	struct community_list *cl = (struct community_list *) data;
+
+	if (cl->name_hash)
+		return cl->name_hash;
+
+	cl->name_hash = bgp_clist_hash_key(cl->name);
+	return cl->name_hash;
+}
+
+static bool bgp_clist_hash_cmp_community_list(const void *a1, const void *a2)
+{
+	const struct community_list *cl1 = a1;
+	const struct community_list *cl2 = a2;
+
+	if (cl1->name_hash != cl2->name_hash)
+		return false;
+
+	if (strcmp(cl1->name, cl2->name) == 0)
+		return true;
+
+	return false;
+}
+
 /* Lookup master structure for community-list or
    extcommunity-list.  */
 struct community_list_master *
@@ -65,7 +123,7 @@ static void community_entry_free(struct community_entry *entry)
 	switch (entry->style) {
 	case COMMUNITY_LIST_STANDARD:
 		if (entry->u.com)
-			community_free(entry->u.com);
+			community_free(&entry->u.com);
 		break;
 	case LARGE_COMMUNITY_LIST_STANDARD:
 		if (entry->u.lcom)
@@ -74,16 +132,14 @@ static void community_entry_free(struct community_entry *entry)
 	case EXTCOMMUNITY_LIST_STANDARD:
 		/* In case of standard extcommunity-list, configuration string
 		   is made by ecommunity_ecom2str().  */
-		if (entry->config)
-			XFREE(MTYPE_ECOMMUNITY_STR, entry->config);
+		XFREE(MTYPE_ECOMMUNITY_STR, entry->config);
 		if (entry->u.ecom)
 			ecommunity_free(&entry->u.ecom);
 		break;
 	case COMMUNITY_LIST_EXPANDED:
 	case EXTCOMMUNITY_LIST_EXPANDED:
 	case LARGE_COMMUNITY_LIST_EXPANDED:
-		if (entry->config)
-			XFREE(MTYPE_COMMUNITY_LIST_CONFIG, entry->config);
+		XFREE(MTYPE_COMMUNITY_LIST_CONFIG, entry->config);
 		if (entry->reg)
 			bgp_regex_free(entry->reg);
 	default:
@@ -101,8 +157,7 @@ static struct community_list *community_list_new(void)
 /* Free community-list.  */
 static void community_list_free(struct community_list *list)
 {
-	if (list->name)
-		XFREE(MTYPE_COMMUNITY_LIST_NAME, list->name);
+	XFREE(MTYPE_COMMUNITY_LIST_NAME, list->name);
 	XFREE(MTYPE_COMMUNITY_LIST, list);
 }
 
@@ -125,11 +180,15 @@ community_list_insert(struct community_list_handler *ch, const char *name,
 	/* Allocate new community_list and copy given name. */
 	new = community_list_new();
 	new->name = XSTRDUP(MTYPE_COMMUNITY_LIST_NAME, name);
+	new->name_hash = bgp_clist_hash_key_community_list(new);
+
+	/* Save for later */
+	hash_get(cm->hash, new, hash_alloc_intern);
 
 	/* If name is made by all digit character.  We treat it as
 	   number. */
 	for (number = 0, i = 0; i < strlen(name); i++) {
-		if (isdigit((int)name[i]))
+		if (isdigit((unsigned char)name[i]))
 			number = (number * 10) + (name[i] - '0');
 		else
 			break;
@@ -194,9 +253,11 @@ community_list_insert(struct community_list_handler *ch, const char *name,
 }
 
 struct community_list *community_list_lookup(struct community_list_handler *ch,
-					     const char *name, int master)
+					     const char *name,
+					     uint32_t name_hash,
+					     int master)
 {
-	struct community_list *list;
+	struct community_list lookup;
 	struct community_list_master *cm;
 
 	if (!name)
@@ -206,14 +267,9 @@ struct community_list *community_list_lookup(struct community_list_handler *ch,
 	if (!cm)
 		return NULL;
 
-	for (list = cm->num.head; list; list = list->next)
-		if (strcmp(list->name, name) == 0)
-			return list;
-	for (list = cm->str.head; list; list = list->next)
-		if (strcmp(list->name, name) == 0)
-			return list;
-
-	return NULL;
+	lookup.name = (char *)name;
+	lookup.name_hash = name_hash;
+	return hash_get(cm->hash, &lookup, NULL);
 }
 
 static struct community_list *
@@ -222,13 +278,14 @@ community_list_get(struct community_list_handler *ch, const char *name,
 {
 	struct community_list *list;
 
-	list = community_list_lookup(ch, name, master);
+	list = community_list_lookup(ch, name, 0, master);
 	if (!list)
 		list = community_list_insert(ch, name, master);
 	return list;
 }
 
-static void community_list_delete(struct community_list *list)
+static void community_list_delete(struct community_list_master *cm,
+				  struct community_list *list)
 {
 	struct community_list_list *clist;
 	struct community_entry *entry, *next;
@@ -250,30 +307,18 @@ static void community_list_delete(struct community_list *list)
 	else
 		clist->head = list->next;
 
+	hash_release(cm->hash, list);
 	community_list_free(list);
 }
 
-static int community_list_empty_p(struct community_list *list)
+static bool community_list_empty_p(struct community_list *list)
 {
-	return (list->head == NULL && list->tail == NULL) ? 1 : 0;
-}
-
-/* Add community-list entry to the list.  */
-static void community_list_entry_add(struct community_list *list,
-				     struct community_entry *entry)
-{
-	entry->next = NULL;
-	entry->prev = list->tail;
-
-	if (list->tail)
-		list->tail->next = entry;
-	else
-		list->head = entry;
-	list->tail = entry;
+	return list->head == NULL && list->tail == NULL;
 }
 
 /* Delete community-list entry from the list.  */
-static void community_list_entry_delete(struct community_list *list,
+static void community_list_entry_delete(struct community_list_master *cm,
+					struct community_list *list,
 					struct community_entry *entry)
 {
 	if (entry->next)
@@ -289,7 +334,84 @@ static void community_list_entry_delete(struct community_list *list,
 	community_entry_free(entry);
 
 	if (community_list_empty_p(list))
-		community_list_delete(list);
+		community_list_delete(cm, list);
+}
+
+/*
+ * Replace community-list entry in the list. Note that entry is the new one
+ * and replace is one one being replaced.
+ */
+static void community_list_entry_replace(struct community_list *list,
+					 struct community_entry *replace,
+					 struct community_entry *entry)
+{
+	if (replace->next) {
+		entry->next = replace->next;
+		replace->next->prev = entry;
+	} else {
+		entry->next = NULL;
+		list->tail = entry;
+	}
+
+	if (replace->prev) {
+		entry->prev = replace->prev;
+		replace->prev->next = entry;
+	} else {
+		entry->prev = NULL;
+		list->head = entry;
+	}
+
+	community_entry_free(replace);
+}
+
+/* Add community-list entry to the list.  */
+static void community_list_entry_add(struct community_list *list,
+				     struct community_entry *entry,
+				     struct community_list_handler *ch,
+				     int master)
+{
+	struct community_entry *replace;
+	struct community_entry *point;
+
+	/* Automatic assignment of seq no. */
+	if (entry->seq == COMMUNITY_SEQ_NUMBER_AUTO)
+		entry->seq = bgp_clist_new_seq_get(list);
+
+	if (list->tail && entry->seq > list->tail->seq)
+		point = NULL;
+	else {
+		replace = bgp_clist_seq_check(list, entry->seq);
+		if (replace) {
+			community_list_entry_replace(list, replace, entry);
+			return;
+		}
+
+		/* Check insert point. */
+		for (point = list->head; point; point = point->next)
+			if (point->seq >= entry->seq)
+				break;
+	}
+
+	/* In case of this is the first element of the list. */
+	entry->next = point;
+
+	if (point) {
+		if (point->prev)
+			point->prev->next = entry;
+		else
+			list->head = entry;
+
+		entry->prev = point->prev;
+		point->prev = entry;
+	} else {
+		if (list->tail)
+			list->tail->next = entry;
+		else
+			list->head = entry;
+
+		entry->prev = list->tail;
+		list->tail = entry;
+	}
 }
 
 /* Lookup community-list entry from the list.  */
@@ -401,7 +523,7 @@ static char *community_str_get(struct community *com, int i)
 
 /* Internal function to perform regular expression match for
  * a single community. */
-static int community_regexp_include(regex_t *reg, struct community *com, int i)
+static bool community_regexp_include(regex_t *reg, struct community *com, int i)
 {
 	char *str;
 	int rv;
@@ -418,16 +540,12 @@ static int community_regexp_include(regex_t *reg, struct community *com, int i)
 
 	XFREE(MTYPE_COMMUNITY_STR, str);
 
-	if (rv == 0)
-		return 1;
-
-	/* No match.  */
-	return 0;
+	return rv == 0;
 }
 
 /* Internal function to perform regular expression match for community
    attribute.  */
-static int community_regexp_match(struct community *com, regex_t *reg)
+static bool community_regexp_match(struct community *com, regex_t *reg)
 {
 	const char *str;
 
@@ -440,10 +558,10 @@ static int community_regexp_match(struct community *com, regex_t *reg)
 
 	/* Regular expression match.  */
 	if (regexec(reg, str, 0, NULL, 0) == 0)
-		return 1;
+		return true;
 
 	/* No match.  */
-	return 0;
+	return false;
 }
 
 static char *lcommunity_str_get(struct lcommunity *lcom, int i)
@@ -453,33 +571,29 @@ static char *lcommunity_str_get(struct lcommunity *lcom, int i)
 	uint32_t localdata1;
 	uint32_t localdata2;
 	char *str;
-	uint8_t *ptr;
-	char *pnt;
+	const uint8_t *ptr;
 
 	ptr = lcom->val + (i * LCOMMUNITY_SIZE);
 
 	memcpy(&lcomval, ptr, LCOMMUNITY_SIZE);
 
 	/* Allocate memory.  48 bytes taken off bgp_lcommunity.c */
-	str = pnt = XMALLOC(MTYPE_LCOMMUNITY_STR, 48);
-
 	ptr = (uint8_t *)lcomval.val;
 	ptr = ptr_get_be32(ptr, &globaladmin);
 	ptr = ptr_get_be32(ptr, &localdata1);
 	ptr = ptr_get_be32(ptr, &localdata2);
 	(void)ptr; /* consume value */
 
-	sprintf(pnt, "%u:%u:%u", globaladmin, localdata1, localdata2);
-	pnt += strlen(pnt);
-	*pnt = '\0';
+	str = XMALLOC(MTYPE_LCOMMUNITY_STR, 48);
+	snprintf(str, 48, "%u:%u:%u", globaladmin, localdata1, localdata2);
 
 	return str;
 }
 
 /* Internal function to perform regular expression match for
  * a single community. */
-static int lcommunity_regexp_include(regex_t *reg, struct lcommunity *lcom,
-				     int i)
+static bool lcommunity_regexp_include(regex_t *reg, struct lcommunity *lcom,
+				      int i)
 {
 	char *str;
 
@@ -493,15 +607,15 @@ static int lcommunity_regexp_include(regex_t *reg, struct lcommunity *lcom,
 	/* Regular expression match.  */
 	if (regexec(reg, str, 0, NULL, 0) == 0) {
 		XFREE(MTYPE_LCOMMUNITY_STR, str);
-		return 1;
+		return true;
 	}
 
 	XFREE(MTYPE_LCOMMUNITY_STR, str);
 	/* No match.  */
-	return 0;
+	return false;
 }
 
-static int lcommunity_regexp_match(struct lcommunity *com, regex_t *reg)
+static bool lcommunity_regexp_match(struct lcommunity *com, regex_t *reg)
 {
 	const char *str;
 
@@ -514,14 +628,14 @@ static int lcommunity_regexp_match(struct lcommunity *com, regex_t *reg)
 
 	/* Regular expression match.  */
 	if (regexec(reg, str, 0, NULL, 0) == 0)
-		return 1;
+		return true;
 
 	/* No match.  */
-	return 0;
+	return false;
 }
 
 
-static int ecommunity_regexp_match(struct ecommunity *ecom, regex_t *reg)
+static bool ecommunity_regexp_match(struct ecommunity *ecom, regex_t *reg)
 {
 	const char *str;
 
@@ -534,10 +648,10 @@ static int ecommunity_regexp_match(struct ecommunity *ecom, regex_t *reg)
 
 	/* Regular expression match.  */
 	if (regexec(reg, str, 0, NULL, 0) == 0)
-		return 1;
+		return true;
 
 	/* No match.  */
-	return 0;
+	return false;
 }
 
 #if 0
@@ -558,7 +672,7 @@ community_regexp_delete (struct community *com, regex_t * reg)
 	i = 0;
 	while (i < com->size)
 	{
-		memcpy (&comval, com_nthval (com, i), sizeof (uint32_t));
+		memcpy (&comval, com_nthval (com, i), sizeof(uint32_t));
 		comval = ntohl (comval);
 
 		switch (comval) {
@@ -622,99 +736,113 @@ community_regexp_delete (struct community *com, regex_t * reg)
 
 /* When given community attribute matches to the community-list return
    1 else return 0.  */
-int community_list_match(struct community *com, struct community_list *list)
+bool community_list_match(struct community *com, struct community_list *list)
 {
 	struct community_entry *entry;
 
 	for (entry = list->head; entry; entry = entry->next) {
 		if (entry->any)
-			return entry->direct == COMMUNITY_PERMIT ? 1 : 0;
+			return entry->direct == COMMUNITY_PERMIT;
 
 		if (entry->style == COMMUNITY_LIST_STANDARD) {
 			if (community_include(entry->u.com, COMMUNITY_INTERNET))
-				return entry->direct == COMMUNITY_PERMIT ? 1
-									 : 0;
+				return entry->direct == COMMUNITY_PERMIT;
 
 			if (community_match(com, entry->u.com))
-				return entry->direct == COMMUNITY_PERMIT ? 1
-									 : 0;
+				return entry->direct == COMMUNITY_PERMIT;
 		} else if (entry->style == COMMUNITY_LIST_EXPANDED) {
 			if (community_regexp_match(com, entry->reg))
-				return entry->direct == COMMUNITY_PERMIT ? 1
-									 : 0;
+				return entry->direct == COMMUNITY_PERMIT;
 		}
 	}
-	return 0;
+	return false;
 }
 
-int lcommunity_list_match(struct lcommunity *lcom, struct community_list *list)
+bool lcommunity_list_match(struct lcommunity *lcom, struct community_list *list)
 {
 	struct community_entry *entry;
 
 	for (entry = list->head; entry; entry = entry->next) {
 		if (entry->any)
-			return entry->direct == COMMUNITY_PERMIT ? 1 : 0;
+			return entry->direct == COMMUNITY_PERMIT;
 
 		if (entry->style == LARGE_COMMUNITY_LIST_STANDARD) {
 			if (lcommunity_match(lcom, entry->u.lcom))
-				return entry->direct == COMMUNITY_PERMIT ? 1
-									 : 0;
+				return entry->direct == COMMUNITY_PERMIT;
 		} else if (entry->style == LARGE_COMMUNITY_LIST_EXPANDED) {
 			if (lcommunity_regexp_match(lcom, entry->reg))
-				return entry->direct == COMMUNITY_PERMIT ? 1
-									 : 0;
+				return entry->direct == COMMUNITY_PERMIT;
 		}
 	}
-	return 0;
+	return false;
 }
 
-int ecommunity_list_match(struct ecommunity *ecom, struct community_list *list)
+
+/* Perform exact matching.  In case of expanded large-community-list, do
+ * same thing as lcommunity_list_match().
+ */
+bool lcommunity_list_exact_match(struct lcommunity *lcom,
+				 struct community_list *list)
 {
 	struct community_entry *entry;
 
 	for (entry = list->head; entry; entry = entry->next) {
 		if (entry->any)
-			return entry->direct == COMMUNITY_PERMIT ? 1 : 0;
+			return entry->direct == COMMUNITY_PERMIT;
+
+		if (entry->style == LARGE_COMMUNITY_LIST_STANDARD) {
+			if (lcommunity_cmp(lcom, entry->u.com))
+				return entry->direct == COMMUNITY_PERMIT;
+		} else if (entry->style == LARGE_COMMUNITY_LIST_EXPANDED) {
+			if (lcommunity_regexp_match(lcom, entry->reg))
+				return entry->direct == COMMUNITY_PERMIT;
+		}
+	}
+	return false;
+}
+
+bool ecommunity_list_match(struct ecommunity *ecom, struct community_list *list)
+{
+	struct community_entry *entry;
+
+	for (entry = list->head; entry; entry = entry->next) {
+		if (entry->any)
+			return entry->direct == COMMUNITY_PERMIT;
 
 		if (entry->style == EXTCOMMUNITY_LIST_STANDARD) {
 			if (ecommunity_match(ecom, entry->u.ecom))
-				return entry->direct == COMMUNITY_PERMIT ? 1
-									 : 0;
+				return entry->direct == COMMUNITY_PERMIT;
 		} else if (entry->style == EXTCOMMUNITY_LIST_EXPANDED) {
 			if (ecommunity_regexp_match(ecom, entry->reg))
-				return entry->direct == COMMUNITY_PERMIT ? 1
-									 : 0;
+				return entry->direct == COMMUNITY_PERMIT;
 		}
 	}
-	return 0;
+	return false;
 }
 
 /* Perform exact matching.  In case of expanded community-list, do
    same thing as community_list_match().  */
-int community_list_exact_match(struct community *com,
-			       struct community_list *list)
+bool community_list_exact_match(struct community *com,
+				struct community_list *list)
 {
 	struct community_entry *entry;
 
 	for (entry = list->head; entry; entry = entry->next) {
 		if (entry->any)
-			return entry->direct == COMMUNITY_PERMIT ? 1 : 0;
+			return entry->direct == COMMUNITY_PERMIT;
 
 		if (entry->style == COMMUNITY_LIST_STANDARD) {
 			if (community_include(entry->u.com, COMMUNITY_INTERNET))
-				return entry->direct == COMMUNITY_PERMIT ? 1
-									 : 0;
+				return entry->direct == COMMUNITY_PERMIT;
 
 			if (community_cmp(com, entry->u.com))
-				return entry->direct == COMMUNITY_PERMIT ? 1
-									 : 0;
+				return entry->direct == COMMUNITY_PERMIT;
 		} else if (entry->style == COMMUNITY_LIST_EXPANDED) {
 			if (community_regexp_match(com, entry->reg))
-				return entry->direct == COMMUNITY_PERMIT ? 1
-									 : 0;
+				return entry->direct == COMMUNITY_PERMIT;
 		}
 	}
-	return 0;
+	return false;
 }
 
 /* Delete all permitted communities in the list from com.  */
@@ -769,6 +897,7 @@ struct community *community_list_match_delete(struct community *com,
 	/* Delete all of the communities we flagged for deletion */
 	for (i = delete_index - 1; i >= 0; i--) {
 		val = community_val_get(com, com_index_to_delete[i]);
+		val = htonl(val);
 		community_del_val(com, &val);
 	}
 
@@ -777,8 +906,8 @@ struct community *community_list_match_delete(struct community *com,
 
 /* To avoid duplicated entry in the community-list, this function
    compares specified entry to existing entry.  */
-static int community_list_dup_check(struct community_list *list,
-				    struct community_entry *new)
+static bool community_list_dup_check(struct community_list *list,
+				     struct community_entry *new)
 {
 	struct community_entry *entry;
 
@@ -793,42 +922,46 @@ static int community_list_dup_check(struct community_list *list,
 			continue;
 
 		if (entry->any)
-			return 1;
+			return true;
 
 		switch (entry->style) {
 		case COMMUNITY_LIST_STANDARD:
 			if (community_cmp(entry->u.com, new->u.com))
-				return 1;
+				return true;
 			break;
 		case LARGE_COMMUNITY_LIST_STANDARD:
 			if (lcommunity_cmp(entry->u.lcom, new->u.lcom))
-				return 1;
+				return true;
 			break;
 		case EXTCOMMUNITY_LIST_STANDARD:
 			if (ecommunity_cmp(entry->u.ecom, new->u.ecom))
-				return 1;
+				return true;
 			break;
 		case COMMUNITY_LIST_EXPANDED:
 		case EXTCOMMUNITY_LIST_EXPANDED:
 		case LARGE_COMMUNITY_LIST_EXPANDED:
 			if (strcmp(entry->config, new->config) == 0)
-				return 1;
+				return true;
 			break;
 		default:
 			break;
 		}
 	}
-	return 0;
+	return false;
 }
 
 /* Set community-list.  */
 int community_list_set(struct community_list_handler *ch, const char *name,
-		       const char *str, int direct, int style)
+		       const char *str, const char *seq, int direct, int style)
 {
 	struct community_entry *entry = NULL;
 	struct community_list *list;
 	struct community *com = NULL;
 	regex_t *regex = NULL;
+	int64_t seqnum = COMMUNITY_SEQ_NUMBER_AUTO;
+
+	if (seq)
+		seqnum = (int64_t)atol(seq);
 
 	/* Get community list. */
 	list = community_list_get(ch, name, COMMUNITY_LIST_MASTER);
@@ -861,9 +994,10 @@ int community_list_set(struct community_list_handler *ch, const char *name,
 	entry = community_entry_new();
 	entry->direct = direct;
 	entry->style = style;
-	entry->any = (str ? 0 : 1);
+	entry->any = (str ? false : true);
 	entry->u.com = com;
 	entry->reg = regex;
+	entry->seq = seqnum;
 	entry->config =
 		(regex ? XSTRDUP(MTYPE_COMMUNITY_LIST_CONFIG, str) : NULL);
 
@@ -871,7 +1005,8 @@ int community_list_set(struct community_list_handler *ch, const char *name,
 	if (community_list_dup_check(list, entry))
 		community_entry_free(entry);
 	else {
-		community_list_entry_add(list, entry);
+		community_list_entry_add(list, entry, ch,
+					 COMMUNITY_LIST_MASTER);
 		route_map_notify_dependencies(name, RMAP_EVENT_CLIST_ADDED);
 	}
 
@@ -880,20 +1015,23 @@ int community_list_set(struct community_list_handler *ch, const char *name,
 
 /* Unset community-list */
 int community_list_unset(struct community_list_handler *ch, const char *name,
-			 const char *str, int direct, int style)
+			 const char *str, const char *seq, int direct,
+			 int style)
 {
+	struct community_list_master *cm = NULL;
 	struct community_entry *entry = NULL;
 	struct community_list *list;
 	struct community *com = NULL;
 
 	/* Lookup community list.  */
-	list = community_list_lookup(ch, name, COMMUNITY_LIST_MASTER);
+	list = community_list_lookup(ch, name, 0, COMMUNITY_LIST_MASTER);
 	if (list == NULL)
 		return COMMUNITY_LIST_ERR_CANT_FIND_LIST;
 
+	cm = community_list_master_lookup(ch, COMMUNITY_LIST_MASTER);
 	/* Delete all of entry belongs to this community-list.  */
 	if (!str) {
-		community_list_delete(list);
+		community_list_delete(cm, list);
 		route_map_notify_dependencies(name, RMAP_EVENT_CLIST_DELETED);
 		return 0;
 	}
@@ -903,14 +1041,14 @@ int community_list_unset(struct community_list_handler *ch, const char *name,
 
 	if (com) {
 		entry = community_list_entry_lookup(list, com, direct);
-		community_free(com);
+		community_free(&com);
 	} else
 		entry = community_list_entry_lookup(list, str, direct);
 
 	if (!entry)
 		return COMMUNITY_LIST_ERR_CANT_FIND_LIST;
 
-	community_list_entry_delete(list, entry);
+	community_list_entry_delete(cm, list, entry);
 	route_map_notify_dependencies(name, RMAP_EVENT_CLIST_DELETED);
 
 	return 0;
@@ -971,14 +1109,74 @@ struct lcommunity *lcommunity_list_match_delete(struct lcommunity *lcom,
 	return lcom;
 }
 
+/* Helper to check if every octet do not exceed UINT_MAX */
+bool lcommunity_list_valid(const char *community, int style)
+{
+	int octets;
+	char **splits, **communities;
+	char *endptr;
+	int num, num_communities;
+	regex_t *regres;
+	int invalid = 0;
+
+	frrstr_split(community, " ", &communities, &num_communities);
+
+	for (int j = 0; j < num_communities; j++) {
+		octets = 0;
+		frrstr_split(communities[j], ":", &splits, &num);
+
+		for (int i = 0; i < num; i++) {
+			if (strlen(splits[i]) == 0)
+				/* There is no digit to check */
+				invalid++;
+
+			if (style == LARGE_COMMUNITY_LIST_STANDARD) {
+				if (*splits[i] == '-')
+					/* Must not be negative */
+					invalid++;
+				else if (strtoul(splits[i], &endptr, 10)
+					 > UINT_MAX)
+					/* Larger than 4 octets */
+					invalid++;
+				else if (*endptr)
+					/* Not all characters were digits */
+					invalid++;
+			} else {
+				regres = bgp_regcomp(communities[j]);
+				if (!regres)
+					/* malformed regex */
+					invalid++;
+				else
+					bgp_regex_free(regres);
+			}
+
+			octets++;
+			XFREE(MTYPE_TMP, splits[i]);
+		}
+		XFREE(MTYPE_TMP, splits);
+
+		if (octets != 3)
+			invalid++;
+
+		XFREE(MTYPE_TMP, communities[j]);
+	}
+	XFREE(MTYPE_TMP, communities);
+
+	return (invalid > 0) ? false : true;
+}
+
 /* Set lcommunity-list.  */
 int lcommunity_list_set(struct community_list_handler *ch, const char *name,
-			const char *str, int direct, int style)
+			const char *str, const char *seq, int direct, int style)
 {
 	struct community_entry *entry = NULL;
 	struct community_list *list;
 	struct lcommunity *lcom = NULL;
 	regex_t *regex = NULL;
+	int64_t seqnum = COMMUNITY_SEQ_NUMBER_AUTO;
+
+	if (seq)
+		seqnum = (int64_t)atol(seq);
 
 	/* Get community list. */
 	list = community_list_get(ch, name, LARGE_COMMUNITY_LIST_MASTER);
@@ -999,6 +1197,9 @@ int lcommunity_list_set(struct community_list_handler *ch, const char *name,
 	}
 
 	if (str) {
+		if (!lcommunity_list_valid(str, style))
+			return COMMUNITY_LIST_ERR_MALFORMED_VAL;
+
 		if (style == LARGE_COMMUNITY_LIST_STANDARD)
 			lcom = lcommunity_str2com(str);
 		else
@@ -1011,17 +1212,21 @@ int lcommunity_list_set(struct community_list_handler *ch, const char *name,
 	entry = community_entry_new();
 	entry->direct = direct;
 	entry->style = style;
-	entry->any = (str ? 0 : 1);
+	entry->any = (str ? false : true);
 	entry->u.lcom = lcom;
 	entry->reg = regex;
+	entry->seq = seqnum;
 	entry->config =
 		(regex ? XSTRDUP(MTYPE_COMMUNITY_LIST_CONFIG, str) : NULL);
 
 	/* Do not put duplicated community entry.  */
 	if (community_list_dup_check(list, entry))
 		community_entry_free(entry);
-	else
-		community_list_entry_add(list, entry);
+	else {
+		community_list_entry_add(list, entry, ch,
+					 LARGE_COMMUNITY_LIST_MASTER);
+		route_map_notify_dependencies(name, RMAP_EVENT_LLIST_ADDED);
+	}
 
 	return 0;
 }
@@ -1029,21 +1234,25 @@ int lcommunity_list_set(struct community_list_handler *ch, const char *name,
 /* Unset community-list.  When str is NULL, delete all of
    community-list entry belongs to the specified name.  */
 int lcommunity_list_unset(struct community_list_handler *ch, const char *name,
-			  const char *str, int direct, int style)
+			  const char *str, const char *seq, int direct,
+			  int style)
 {
+	struct community_list_master *cm = NULL;
 	struct community_entry *entry = NULL;
 	struct community_list *list;
 	struct lcommunity *lcom = NULL;
 	regex_t *regex = NULL;
 
 	/* Lookup community list.  */
-	list = community_list_lookup(ch, name, LARGE_COMMUNITY_LIST_MASTER);
+	list = community_list_lookup(ch, name, 0, LARGE_COMMUNITY_LIST_MASTER);
 	if (list == NULL)
 		return COMMUNITY_LIST_ERR_CANT_FIND_LIST;
 
+	cm = community_list_master_lookup(ch, LARGE_COMMUNITY_LIST_MASTER);
 	/* Delete all of entry belongs to this community-list.  */
 	if (!str) {
-		community_list_delete(list);
+		community_list_delete(cm, list);
+		route_map_notify_dependencies(name, RMAP_EVENT_LLIST_DELETED);
 		return 0;
 	}
 
@@ -1068,19 +1277,25 @@ int lcommunity_list_unset(struct community_list_handler *ch, const char *name,
 	if (!entry)
 		return COMMUNITY_LIST_ERR_CANT_FIND_LIST;
 
-	community_list_entry_delete(list, entry);
+	community_list_entry_delete(cm, list, entry);
+	route_map_notify_dependencies(name, RMAP_EVENT_LLIST_DELETED);
 
 	return 0;
 }
 
 /* Set extcommunity-list.  */
 int extcommunity_list_set(struct community_list_handler *ch, const char *name,
-			  const char *str, int direct, int style)
+			  const char *str, const char *seq, int direct,
+			  int style)
 {
 	struct community_entry *entry = NULL;
 	struct community_list *list;
 	struct ecommunity *ecom = NULL;
 	regex_t *regex = NULL;
+	int64_t seqnum = COMMUNITY_SEQ_NUMBER_AUTO;
+
+	if (seq)
+		seqnum = (int64_t)atol(seq);
 
 	if (str == NULL)
 		return COMMUNITY_LIST_ERR_MALFORMED_VAL;
@@ -1118,7 +1333,7 @@ int extcommunity_list_set(struct community_list_handler *ch, const char *name,
 	entry = community_entry_new();
 	entry->direct = direct;
 	entry->style = style;
-	entry->any = 0;
+	entry->any = false;
 	if (ecom)
 		entry->config = ecommunity_ecom2str(
 			ecom, ECOMMUNITY_FORMAT_COMMUNITY_LIST, 0);
@@ -1127,12 +1342,14 @@ int extcommunity_list_set(struct community_list_handler *ch, const char *name,
 
 	entry->u.ecom = ecom;
 	entry->reg = regex;
+	entry->seq = seqnum;
 
 	/* Do not put duplicated community entry.  */
 	if (community_list_dup_check(list, entry))
 		community_entry_free(entry);
 	else {
-		community_list_entry_add(list, entry);
+		community_list_entry_add(list, entry, ch,
+					 EXTCOMMUNITY_LIST_MASTER);
 		route_map_notify_dependencies(name, RMAP_EVENT_ECLIST_ADDED);
 	}
 
@@ -1145,20 +1362,23 @@ int extcommunity_list_set(struct community_list_handler *ch, const char *name,
  * specified name.
  */
 int extcommunity_list_unset(struct community_list_handler *ch, const char *name,
-			    const char *str, int direct, int style)
+			    const char *str, const char *seq, int direct,
+			    int style)
 {
+	struct community_list_master *cm = NULL;
 	struct community_entry *entry = NULL;
 	struct community_list *list;
 	struct ecommunity *ecom = NULL;
 
 	/* Lookup extcommunity list.  */
-	list = community_list_lookup(ch, name, EXTCOMMUNITY_LIST_MASTER);
+	list = community_list_lookup(ch, name, 0, EXTCOMMUNITY_LIST_MASTER);
 	if (list == NULL)
 		return COMMUNITY_LIST_ERR_CANT_FIND_LIST;
 
+	cm = community_list_master_lookup(ch, EXTCOMMUNITY_LIST_MASTER);
 	/* Delete all of entry belongs to this extcommunity-list.  */
 	if (!str) {
-		community_list_delete(list);
+		community_list_delete(cm, list);
 		route_map_notify_dependencies(name, RMAP_EVENT_ECLIST_DELETED);
 		return 0;
 	}
@@ -1175,7 +1395,7 @@ int extcommunity_list_unset(struct community_list_handler *ch, const char *name,
 	if (!entry)
 		return COMMUNITY_LIST_ERR_CANT_FIND_LIST;
 
-	community_list_entry_delete(list, entry);
+	community_list_entry_delete(cm, list, entry);
 	route_map_notify_dependencies(name, RMAP_EVENT_ECLIST_DELETED);
 
 	return 0;
@@ -1187,6 +1407,22 @@ struct community_list_handler *community_list_init(void)
 	struct community_list_handler *ch;
 	ch = XCALLOC(MTYPE_COMMUNITY_LIST_HANDLER,
 		     sizeof(struct community_list_handler));
+
+	ch->community_list.hash =
+		hash_create_size(4, bgp_clist_hash_key_community_list,
+				 bgp_clist_hash_cmp_community_list,
+				 "Community List Number Quick Lookup");
+
+	ch->extcommunity_list.hash =
+		hash_create_size(4, bgp_clist_hash_key_community_list,
+				 bgp_clist_hash_cmp_community_list,
+				 "Extended Community List Quick Lookup");
+
+	ch->lcommunity_list.hash =
+		hash_create_size(4, bgp_clist_hash_key_community_list,
+				 bgp_clist_hash_cmp_community_list,
+				 "Large Community List Quick Lookup");
+
 	return ch;
 }
 
@@ -1198,21 +1434,24 @@ void community_list_terminate(struct community_list_handler *ch)
 
 	cm = &ch->community_list;
 	while ((list = cm->num.head) != NULL)
-		community_list_delete(list);
+		community_list_delete(cm, list);
 	while ((list = cm->str.head) != NULL)
-		community_list_delete(list);
+		community_list_delete(cm, list);
+	hash_free(cm->hash);
 
 	cm = &ch->lcommunity_list;
 	while ((list = cm->num.head) != NULL)
-		community_list_delete(list);
+		community_list_delete(cm, list);
 	while ((list = cm->str.head) != NULL)
-		community_list_delete(list);
+		community_list_delete(cm, list);
+	hash_free(cm->hash);
 
 	cm = &ch->extcommunity_list;
 	while ((list = cm->num.head) != NULL)
-		community_list_delete(list);
+		community_list_delete(cm, list);
 	while ((list = cm->str.head) != NULL)
-		community_list_delete(list);
+		community_list_delete(cm, list);
+	hash_free(cm->hash);
 
 	XFREE(MTYPE_COMMUNITY_LIST_HANDLER, ch);
 }
