@@ -1707,6 +1707,8 @@ void vrf_import_from_vrf(struct bgp *to_bgp, struct bgp *from_bgp,
 	if (!is_inst_match)
 		listnode_add(to_bgp->vpn_policy[afi].import_vrf,
 				     vname);
+	else
+		XFREE(MTYPE_TMP, vname);
 
 	/* Check if the source vrf already exports to any vrf,
 	 * first time export requires to setup auto derived RD/RT values.
@@ -1729,6 +1731,9 @@ void vrf_import_from_vrf(struct bgp *to_bgp, struct bgp *from_bgp,
 	if (!is_inst_match)
 		listnode_add(from_bgp->vpn_policy[afi].export_vrf,
 			     vname);
+	else
+		XFREE(MTYPE_TMP, vname);
+
 	/* Update import RT for current VRF using export RT of the VRF we're
 	 * importing from. First though, make sure "import_vrf" has that
 	 * set.
@@ -1759,19 +1764,26 @@ void vrf_import_from_vrf(struct bgp *to_bgp, struct bgp *from_bgp,
 
 	if (debug) {
 		const char *from_name;
+		char *ecom1, *ecom2;
 
 		from_name = from_bgp->name ? from_bgp->name :
 			VRF_DEFAULT_NAME;
-		zlog_debug("%s from %s to %s first_export %u import-rt %s export-rt %s",
-			   __func__, from_name, export_name, first_export,
-			   to_bgp->vpn_policy[afi].rtlist[idir] ?
-			   (ecommunity_ecom2str(to_bgp->vpn_policy[afi].
-						rtlist[idir],
-					ECOMMUNITY_FORMAT_ROUTE_MAP, 0)) : " ",
-			   to_bgp->vpn_policy[afi].rtlist[edir] ?
-			   (ecommunity_ecom2str(to_bgp->vpn_policy[afi].
-						rtlist[edir],
-					ECOMMUNITY_FORMAT_ROUTE_MAP, 0)) : " ");
+
+		ecom1 = ecommunity_ecom2str(
+			to_bgp->vpn_policy[afi].rtlist[idir],
+			ECOMMUNITY_FORMAT_ROUTE_MAP, 0);
+
+		ecom2 = ecommunity_ecom2str(
+			to_bgp->vpn_policy[afi].rtlist[edir],
+			ECOMMUNITY_FORMAT_ROUTE_MAP, 0);
+
+		zlog_debug(
+			"%s from %s to %s first_export %u import-rt %s export-rt %s",
+			__func__, from_name, export_name, first_export, ecom1,
+			ecom2);
+
+		ecommunity_strfree(&ecom1);
+		ecommunity_strfree(&ecom2);
 	}
 
 	/* Does "import_vrf" first need to export its routes or that
@@ -2581,7 +2593,7 @@ void vpn_leak_postchange_all(void)
  * also VRF Y should unimport its routes from VRF X table.
  * This will ensure VPN table is cleaned up appropriately.
  */
-int bgp_vpn_leak_unimport(struct bgp *from_bgp, struct vty *vty)
+int bgp_vpn_leak_unimport(struct bgp *from_bgp)
 {
 	struct bgp *to_bgp;
 	const char *tmp_name;
