@@ -24,8 +24,9 @@
 #include "lib/ferr.h"
 
 enum isis_log_refs {
-	ISIS_ERR_PACKET = ISIS_FERR_START,
-	ISIS_ERR_CONFIG,
+	EC_ISIS_PACKET = ISIS_FERR_START,
+	EC_ISIS_CONFIG,
+	EC_ISIS_SID_OVERFLOW,
 };
 
 extern void isis_error_init(void);
