@@ -20,6 +20,7 @@
 #include <zebra.h>
 
 #include "ldpd.h"
+#include "ldpe.h"
 #include "lde.h"
 #include "log.h"
 
@@ -31,7 +32,7 @@ static int		 lde_nbr_is_nexthop(struct fec_node *,
 static void		 fec_free(void *);
 static struct fec_node	*fec_add(struct fec *fec);
 static struct fec_nh	*fec_nh_add(struct fec_node *, int, union ldpd_addr *,
-			    ifindex_t, uint8_t);
+			    ifindex_t, uint8_t, unsigned short);
 static void		 fec_nh_del(struct fec_nh *);
 
 RB_GENERATE(fec_tree, fec, entry, fec_compare)
@@ -266,6 +267,9 @@ fec_add(struct fec *fec)
 	RB_INIT(lde_map_head, &fn->downstream);
 	LIST_INIT(&fn->nexthops);
 
+	if (fec->type == FEC_TYPE_PWID)
+		fn->pw_remote_status = PW_FORWARDING;
+
 	if (fec_insert(&ft, &fn->fec))
 		log_warnx("failed to add %s to ft tree",
 		    log_fec(&fn->fec));
@@ -275,7 +279,7 @@ fec_add(struct fec *fec)
 
 struct fec_nh *
 fec_nh_find(struct fec_node *fn, int af, union ldpd_addr *nexthop,
-    ifindex_t ifindex, uint8_t priority)
+    ifindex_t ifindex, uint8_t route_type, unsigned short route_instance)
 {
 	struct fec_nh	*fnh;
 
@@ -283,7 +287,8 @@ fec_nh_find(struct fec_node *fn, int af, union ldpd_addr *nexthop,
 		if (fnh->af == af &&
 		    ldp_addrcmp(af, &fnh->nexthop, nexthop) == 0 &&
 		    fnh->ifindex == ifindex &&
-		    fnh->priority == priority)
+		    fnh->route_type == route_type &&
+		    fnh->route_instance == route_instance)
 			return (fnh);
 
 	return (NULL);
@@ -291,7 +296,7 @@ fec_nh_find(struct fec_node *fn, int af, union ldpd_addr *nexthop,
 
 static struct fec_nh *
 fec_nh_add(struct fec_node *fn, int af, union ldpd_addr *nexthop,
-    ifindex_t ifindex, uint8_t priority)
+    ifindex_t ifindex, uint8_t route_type, unsigned short route_instance)
 {
 	struct fec_nh	*fnh;
 
@@ -303,7 +308,8 @@ fec_nh_add(struct fec_node *fn, int af, union ldpd_addr *nexthop,
 	fnh->nexthop = *nexthop;
 	fnh->ifindex = ifindex;
 	fnh->remote_label = NO_LABEL;
-	fnh->priority = priority;
+	fnh->route_type = route_type;
+	fnh->route_instance = route_instance;
 	LIST_INSERT_HEAD(&fn->nexthops, fnh, entry);
 
 	return (fnh);
@@ -318,10 +324,12 @@ fec_nh_del(struct fec_nh *fnh)
 
 void
 lde_kernel_insert(struct fec *fec, int af, union ldpd_addr *nexthop,
-    ifindex_t ifindex, uint8_t priority, int connected, void *data)
+    ifindex_t ifindex, uint8_t route_type, unsigned short route_instance,
+    int connected, void *data)
 {
 	struct fec_node		*fn;
 	struct fec_nh		*fnh;
+	struct iface		*iface;
 
 	fn = (struct fec_node *)fec_find(&ft, fec);
 	if (fn == NULL)
@@ -329,9 +337,22 @@ lde_kernel_insert(struct fec *fec, int af, union ldpd_addr *nexthop,
 	if (data)
 		fn->data = data;
 
-	fnh = fec_nh_find(fn, af, nexthop, ifindex, priority);
-	if (fnh == NULL)
-		fnh = fec_nh_add(fn, af, nexthop, ifindex, priority);
+	fnh = fec_nh_find(fn, af, nexthop, ifindex, route_type, route_instance);
+	if (fnh == NULL) {
+		fnh = fec_nh_add(fn, af, nexthop, ifindex, route_type,
+		    route_instance);
+		/*
+		 * Ordered Control: if not a connected route and not a route
+		 * learned over an interface not running LDP and not a PW
+		 * then mark to wait until we receive labelmap msg before
+		 * installing in kernel and sending to peer
+		 */
+		iface = if_lookup(ldeconf, ifindex);
+		if ((ldeconf->flags & F_LDPD_ORDERED_CONTROL) &&
+		    !connected && iface != NULL && fec->type != FEC_TYPE_PWID)
+			fnh->flags |= F_FEC_NH_DEFER;
+	}
+
 	fnh->flags |= F_FEC_NH_NEW;
 	if (connected)
 		fnh->flags |= F_FEC_NH_CONNECTED;
@@ -339,7 +360,7 @@ lde_kernel_insert(struct fec *fec, int af, union ldpd_addr *nexthop,
 
 void
 lde_kernel_remove(struct fec *fec, int af, union ldpd_addr *nexthop,
-    ifindex_t ifindex, uint8_t priority)
+    ifindex_t ifindex, uint8_t route_type, unsigned short route_instance)
 {
 	struct fec_node		*fn;
 	struct fec_nh		*fnh;
@@ -348,7 +369,7 @@ lde_kernel_remove(struct fec *fec, int af, union ldpd_addr *nexthop,
 	if (fn == NULL)
 		/* route lost */
 		return;
-	fnh = fec_nh_find(fn, af, nexthop, ifindex, priority);
+	fnh = fec_nh_find(fn, af, nexthop, ifindex, route_type, route_instance);
 	if (fnh == NULL)
 		/* route lost */
 		return;
@@ -370,15 +391,30 @@ lde_kernel_update(struct fec *fec)
 	struct fec_nh		*fnh, *safe;
 	struct lde_nbr		*ln;
 	struct lde_map		*me;
+	struct iface		*iface;
 
 	fn = (struct fec_node *)fec_find(&ft, fec);
 	if (fn == NULL)
 		return;
 
 	LIST_FOREACH_SAFE(fnh, &fn->nexthops, entry, safe) {
-		if (fnh->flags & F_FEC_NH_NEW)
+		if (fnh->flags & F_FEC_NH_NEW) {
 			fnh->flags &= ~F_FEC_NH_NEW;
-		else {
+			/*
+			 * if LDP configured on interface or a static route
+			 * clear flag else treat fec as a connected route
+			 */
+			if (ldeconf->flags & F_LDPD_ENABLED) {
+				iface = if_lookup(ldeconf,fnh->ifindex);
+				if (fnh->flags & F_FEC_NH_CONNECTED ||
+				    iface ||
+				    fnh->route_type == ZEBRA_ROUTE_STATIC)
+					fnh->flags &=~F_FEC_NH_NO_LDP;
+				else
+					fnh->flags |= F_FEC_NH_NO_LDP;
+			} else
+				fnh->flags |= F_FEC_NH_NO_LDP;
+		} else {
 			lde_send_delete_klabel(fn, fnh);
 			fec_nh_del(fnh);
 		}
@@ -406,6 +442,10 @@ lde_kernel_update(struct fec *fec)
 				lde_send_labelmapping(ln, fn, 1);
 	}
 
+	/* if no label created yet then don't try to program labeled route */
+	if (fn->local_label == NO_LABEL)
+		return;
+
 	LIST_FOREACH(fnh, &fn->nexthops, entry) {
 		lde_send_change_klabel(fn, fnh);
 
@@ -427,13 +467,13 @@ lde_kernel_update(struct fec *fec)
 			me = (struct lde_map *)fec_find(&ln->recv_map, &fn->fec);
 			if (me)
 				/* FEC.5 */
-				lde_check_mapping(&me->map, ln);
+				lde_check_mapping(&me->map, ln, 0);
 		}
 	}
 }
 
 void
-lde_check_mapping(struct map *map, struct lde_nbr *ln)
+lde_check_mapping(struct map *map, struct lde_nbr *ln, int rcvd_label_mapping)
 {
 	struct fec		 fec;
 	struct fec_node		*fn;
@@ -441,6 +481,7 @@ lde_check_mapping(struct map *map, struct lde_nbr *ln)
 	struct lde_req		*lre;
 	struct lde_map		*me;
 	struct l2vpn_pw		*pw;
+	bool			 send_map = false;
 
 	lde_map2fec(map, ln->id, &fec);
 
@@ -478,8 +519,12 @@ lde_check_mapping(struct map *map, struct lde_nbr *ln)
 		lde_req_del(ln, lre, 1);
 
 	/* RFC 4447 control word and status tlv negotiation */
-	if (map->type == MAP_TYPE_PWID && l2vpn_pw_negotiate(ln, fn, map))
+	if (map->type == MAP_TYPE_PWID && l2vpn_pw_negotiate(ln, fn, map)) {
+		if (rcvd_label_mapping && map->flags & F_MAP_PW_STATUS)
+			fn->pw_remote_status = map->pw_status;
+
 		return;
+	}
 
 	/*
 	 * LMp.3 - LMp.8: loop detection - unnecessary for frame-mode
@@ -521,8 +566,18 @@ lde_check_mapping(struct map *map, struct lde_nbr *ln)
 			if (!lde_address_find(ln, fnh->af, &fnh->nexthop))
 				continue;
 
+			/*
+			 * Ordered Control: labelmap msg received from
+			 * NH so clear flag and send labelmap msg to
+			 * peer
+			 */
+			if (ldeconf->flags & F_LDPD_ORDERED_CONTROL) {
+				send_map = true;
+				fnh->flags &= ~F_FEC_NH_DEFER;
+			}
 			fnh->remote_label = map->label;
-			lde_send_change_klabel(fn, fnh);
+			if (fn->local_label != NO_LABEL)
+				lde_send_change_klabel(fn, fnh);
 			break;
 		case FEC_TYPE_PWID:
 			pw = (struct l2vpn_pw *) fn->data;
@@ -532,8 +587,10 @@ lde_check_mapping(struct map *map, struct lde_nbr *ln)
 			pw->remote_group = map->fec.pwid.group_id;
 			if (map->flags & F_MAP_PW_IFMTU)
 				pw->remote_mtu = map->fec.pwid.ifmtu;
-			if (map->flags & F_MAP_PW_STATUS)
+			if (rcvd_label_mapping && map->flags & F_MAP_PW_STATUS) {
 				pw->remote_status = map->pw_status;
+				fn->pw_remote_status = map->pw_status;
+			}
 			else
 				pw->remote_status = PW_FORWARDING;
 			fnh->remote_label = map->label;
@@ -554,6 +611,15 @@ lde_check_mapping(struct map *map, struct lde_nbr *ln)
 	 * loop detection. LMp.28 - LMp.30 are unnecessary because we are
 	 * merging capable.
 	 */
+
+	/*
+	 * Ordered Control: just received a labelmap for this fec from NH so
+	 * need to send labelmap to all peers
+	 * LMp.20 - LMp21 Execute procedure to send Label Mapping
+	 */
+	if (send_map && fn->local_label != NO_LABEL)
+		RB_FOREACH(ln, nbr_tree, &lde_nbrs)
+			lde_send_labelmapping(ln, fn, 1);
 }
 
 void
@@ -753,6 +819,7 @@ lde_check_withdraw(struct map *map, struct lde_nbr *ln)
 	struct fec_nh		*fnh;
 	struct lde_map		*me;
 	struct l2vpn_pw		*pw;
+	struct lde_nbr		*lnbr;
 
 	/* wildcard label withdraw */
 	if (map->type == MAP_TYPE_WILDCARD ||
@@ -799,6 +866,31 @@ lde_check_withdraw(struct map *map, struct lde_nbr *ln)
 	if (me && (map->label == NO_LABEL || map->label == me->map.label))
 		/* LWd.4: remove record of previously received lbl mapping */
 		lde_map_del(ln, me, 0);
+	else
+		/* LWd.13 done */
+		return;
+
+	/* Ordered Control: additional withdraw steps */
+	if (ldeconf->flags & F_LDPD_ORDERED_CONTROL) {
+		/* LWd.8: for each neighbor other that src of withdraw msg */
+		RB_FOREACH(lnbr, nbr_tree, &lde_nbrs) {
+			if (ln->peerid == lnbr->peerid)
+				continue;
+
+			/* LWd.9: check if previously sent a label mapping */
+			me = (struct lde_map *)fec_find(&lnbr->sent_map,
+			    &fn->fec);
+
+			/*
+			 * LWd.10: does label sent to peer "map" to withdraw
+			 * label
+			 */
+			if (me && lde_nbr_is_nexthop(fn, lnbr))
+				/* LWd.11: send label withdraw */
+				lde_send_labelwithdraw(lnbr, fn, NULL, NULL);
+		}
+	}
+
 }
 
 void
@@ -809,6 +901,7 @@ lde_check_withdraw_wcard(struct map *map, struct lde_nbr *ln)
 	struct fec_nh	*fnh;
 	struct lde_map	*me;
 	struct l2vpn_pw	*pw;
+	struct lde_nbr  *lnbr;
 
 	/* LWd.2: send label release */
 	lde_send_labelrelease(ln, NULL, map, map->label);
@@ -855,6 +948,35 @@ lde_check_withdraw_wcard(struct map *map, struct lde_nbr *ln)
 			 * label mapping
 			 */
 			lde_map_del(ln, me, 0);
+		else
+			/* LWd.13 done */
+			continue;
+
+		/* Ordered Control: additional withdraw steps */
+		if (ldeconf->flags & F_LDPD_ORDERED_CONTROL) {
+			/*
+			 * LWd.8: for each neighbor other that src of
+			 *  withdraw msg
+			 */
+			RB_FOREACH(lnbr, nbr_tree, &lde_nbrs) {
+				if (ln->peerid == lnbr->peerid)
+					continue;
+
+				/* LWd.9: check if previously sent a label
+				 * mapping
+				 */
+				me = (struct lde_map *)fec_find(
+				    &lnbr->sent_map, &fn->fec);
+				/*
+				 * LWd.10: does label sent to peer "map" to
+				 *  withdraw label
+				 */
+				if (me && lde_nbr_is_nexthop(fn, lnbr))
+					/* LWd.11: send label withdraw */
+					lde_send_labelwithdraw(lnbr, fn, NULL,
+					    NULL);
+			}
+		}
 	}
 }
 
@@ -919,6 +1041,9 @@ lde_gc_timer(struct thread *thread)
 		    !RB_EMPTY(lde_map_head, &fn->upstream))
 			continue;
 
+		if (fn->local_label != NO_LABEL)
+			lde_free_label(fn->local_label);
+
 		fec_remove(&ft, &fn->fec);
 		free(fn);
 		count++;
