@@ -72,7 +72,7 @@ static int ospf6_abr_nexthops_belong_to_area(struct ospf6_route *route,
 	struct ospf6_interface *oi;
 
 	oi = ospf6_interface_lookup_by_ifindex(
-		ospf6_route_get_first_nh_index(route));
+		ospf6_route_get_first_nh_index(route), area->ospf6->vrf_id);
 	if (oi && oi->area && oi->area == area)
 		return 1;
 	else
@@ -171,8 +171,9 @@ int ospf6_abr_originate_summary_to_area(struct ospf6_route *route,
 	/* AS External routes are never considered */
 	if (route->path.type == OSPF6_PATH_TYPE_EXTERNAL1
 	    || route->path.type == OSPF6_PATH_TYPE_EXTERNAL2) {
-		if (is_debug)
-			zlog_debug("Path type is external, skip");
+#if 0
+		zlog_debug("Path type is external, skip");
+#endif
 		return 0;
 	}
 
@@ -231,8 +232,23 @@ int ospf6_abr_originate_summary_to_area(struct ospf6_route *route,
 							&route->prefix)), buf,
 						  sizeof(buf));
 					zlog_debug(
-						"%s: route %s with cost %u is not best, ignore."
-						, __PRETTY_FUNCTION__, buf,
+						"%s: route %s with cost %u is not best, ignore.",
+						__func__, buf,
+						route->path.cost);
+				}
+				return 0;
+			}
+		}
+
+		if (route->path.origin.type ==
+		    htons(OSPF6_LSTYPE_INTRA_PREFIX)) {
+			if (!CHECK_FLAG(route->flag, OSPF6_ROUTE_BEST)) {
+				if (is_debug) {
+					prefix2str(&route->prefix, buf,
+						   sizeof(buf));
+					zlog_debug(
+						"%s: intra-prefix route %s with cost %u is not best, ignore.",
+						__func__, buf,
 						route->path.cost);
 				}
 				return 0;
@@ -270,9 +286,10 @@ int ospf6_abr_originate_summary_to_area(struct ospf6_route *route,
 				ospf6_abr_delete_route(route, summary,
 						       summary_table, old);
 			}
-		} else if (old)
+		} else if (old) {
+			ospf6_route_remove(summary, summary_table);
 			ospf6_lsa_purge(old);
-
+		}
 		return 0;
 	}
 
@@ -746,11 +763,13 @@ void ospf6_abr_old_path_update(struct ospf6_route *old_route,
 		}
 
 		if (IS_OSPF6_DEBUG_ABR || IS_OSPF6_DEBUG_EXAMIN(INTER_PREFIX))
-			zlog_debug("%s: paths %u nh %u", __PRETTY_FUNCTION__,
-				   old_route->paths ?
-				   listcount(old_route->paths) : 0,
-				   old_route->nh_list ?
-				   listcount(old_route->nh_list) : 0);
+			zlog_debug("%s: paths %u nh %u", __func__,
+				   old_route->paths
+					   ? listcount(old_route->paths)
+					   : 0,
+				   old_route->nh_list
+					   ? listcount(old_route->nh_list)
+					   : 0);
 
 		if (table->hook_add)
 			(*table->hook_add)(old_route);
@@ -774,6 +793,10 @@ void ospf6_abr_old_route_remove(struct ospf6_lsa *lsa,
 				struct ospf6_route *old,
 				struct ospf6_route_table *table)
 {
+	if (IS_OSPF6_DEBUG_ABR)
+		zlog_debug("%s: route %pFX, paths %d", __func__, &old->prefix,
+			   listcount(old->paths));
+
 	if (listcount(old->paths) > 1) {
 		struct listnode *anode, *anext, *nnode, *rnode, *rnext;
 		struct ospf6_path *o_path;
@@ -783,13 +806,15 @@ void ospf6_abr_old_route_remove(struct ospf6_lsa *lsa,
 
 		for (ALL_LIST_ELEMENTS(old->paths, anode, anext, o_path)) {
 			if (o_path->origin.adv_router != lsa->header->adv_router
-			    && o_path->origin.id != lsa->header->id)
+			    || o_path->origin.id != lsa->header->id)
 				continue;
 			for (ALL_LIST_ELEMENTS_RO(o_path->nh_list, nnode, nh)) {
 				for (ALL_LIST_ELEMENTS(old->nh_list,
 							rnode, rnext, rnh)) {
 					if (!ospf6_nexthop_is_same(rnh, nh))
 						continue;
+					if (IS_OSPF6_DEBUG_ABR)
+						zlog_debug("deleted nexthop");
 					listnode_delete(old->nh_list, rnh);
 					ospf6_nexthop_delete(rnh);
 				}
@@ -806,9 +831,10 @@ void ospf6_abr_old_route_remove(struct ospf6_lsa *lsa,
 					prefix2str(&old->prefix, buf,
 						   sizeof(buf));
 					zlog_debug("%s: old %s updated nh %u",
-						   __PRETTY_FUNCTION__, buf,
-						   old->nh_list ?
-						   listcount(old->nh_list) : 0);
+						   __func__, buf,
+						   old->nh_list ? listcount(
+							   old->nh_list)
+								: 0);
 				}
 
 				if (table->hook_add)
@@ -855,15 +881,16 @@ void ospf6_abr_examin_summary(struct ospf6_lsa *lsa, struct ospf6_area *oa)
 	bool old_entry_updated = false;
 	struct ospf6_path *path, *o_path, *ecmp_path;
 	struct listnode *anode;
-	char adv_router[16];
+	bool add_route = false;
 
 	memset(&prefix, 0, sizeof(prefix));
 
 	if (lsa->header->type == htons(OSPF6_LSTYPE_INTER_PREFIX)) {
 		if (IS_OSPF6_DEBUG_EXAMIN(INTER_PREFIX)) {
 			is_debug++;
-			zlog_debug("%s: Examin %s in area %s",
-				   __PRETTY_FUNCTION__, lsa->name, oa->name);
+			zlog_debug("%s: LSA %s age %d in area %s", __func__,
+				   lsa->name, ospf6_lsa_age_current(lsa),
+				   oa->name);
 		}
 
 		prefix_lsa =
@@ -882,8 +909,9 @@ void ospf6_abr_examin_summary(struct ospf6_lsa *lsa, struct ospf6_area *oa)
 	} else if (lsa->header->type == htons(OSPF6_LSTYPE_INTER_ROUTER)) {
 		if (IS_OSPF6_DEBUG_EXAMIN(INTER_ROUTER)) {
 			is_debug++;
-			zlog_debug("%s: Examin %s in area %s",
-				   __PRETTY_FUNCTION__, lsa->name, oa->name);
+			zlog_debug("%s: LSA %s age %d in area %s", __func__,
+				   lsa->name, ospf6_lsa_age_current(lsa),
+				   oa->name);
 		}
 
 		router_lsa =
@@ -907,8 +935,12 @@ void ospf6_abr_examin_summary(struct ospf6_lsa *lsa, struct ospf6_area *oa)
 
 	/* Find existing route */
 	route = ospf6_route_lookup(&prefix, table);
-	if (route)
+	if (route) {
 		ospf6_route_lock(route);
+		if (is_debug)
+			zlog_debug("%s: route %pFX, paths %d", __func__,
+				   &prefix, listcount(route->paths));
+	}
 	while (route && ospf6_route_is_prefix(&prefix, route)) {
 		if (route->path.area_id == oa->area_id
 		    && route->path.origin.type == lsa->header->type
@@ -918,24 +950,18 @@ void ospf6_abr_examin_summary(struct ospf6_lsa *lsa, struct ospf6_area *oa)
 			 * old as the route.
 			 */
 			if (listcount(route->paths) > 1) {
-				struct listnode *anode;
-				struct ospf6_path *o_path;
-
 				for (ALL_LIST_ELEMENTS_RO(route->paths, anode,
 							  o_path)) {
-					inet_ntop(AF_INET,
-						  &o_path->origin.adv_router,
-						  adv_router,
-						  sizeof(adv_router));
 					if (o_path->origin.id == lsa->header->id
 					    && o_path->origin.adv_router ==
 					    lsa->header->adv_router) {
 						old = route;
 
 						if (is_debug)
-							zlog_debug("%s: old entry found in paths, adv_router %s",
-							__PRETTY_FUNCTION__,
-							adv_router);
+							zlog_debug(
+								"%s: old entry found in paths, adv_router %pI4",
+								__func__,
+								&o_path->origin.adv_router);
 
 						break;
 					}
@@ -960,13 +986,14 @@ void ospf6_abr_examin_summary(struct ospf6_lsa *lsa, struct ospf6_area *oa)
 	}
 	if (OSPF6_LSA_IS_MAXAGE(lsa)) {
 		if (is_debug)
-			zlog_debug("%s: LSA %s is MaxAge, ignore",
-				   __PRETTY_FUNCTION__, lsa->name);
+			zlog_debug("%s: LSA %s is MaxAge, ignore", __func__,
+				   lsa->name);
 		if (old)
 			ospf6_abr_old_route_remove(lsa, old, table);
 		return;
 	}
 
+
 	/* (2) if the LSA is self-originated, ignore */
 	if (lsa->header->adv_router == oa->ospf6->router_id) {
 		if (is_debug)
@@ -1007,6 +1034,7 @@ void ospf6_abr_examin_summary(struct ospf6_lsa *lsa, struct ospf6_area *oa)
 
 	if (lsa->header->type == htons(OSPF6_LSTYPE_INTER_ROUTER)) {
 		/* To pass test suites */
+		assert(router_lsa);
 		if (!OSPF6_OPT_ISSET(router_lsa->options, OSPF6_OPT_R)
 		    || !OSPF6_OPT_ISSET(router_lsa->options, OSPF6_OPT_V6)) {
 			if (is_debug)
@@ -1039,22 +1067,22 @@ void ospf6_abr_examin_summary(struct ospf6_lsa *lsa, struct ospf6_area *oa)
 	    || CHECK_FLAG(abr_entry->flag, OSPF6_ROUTE_REMOVE)
 	    || !CHECK_FLAG(abr_entry->path.router_bits, OSPF6_ROUTER_BIT_B)) {
 		if (is_debug)
-			zlog_debug("%s: ABR router entry does not exist, ignore",
-				   __PRETTY_FUNCTION__);
+			zlog_debug(
+				"%s: ABR router entry %pFX does not exist, ignore",
+				__func__, &abr_prefix);
 		if (old) {
 			if (old->type == OSPF6_DEST_TYPE_ROUTER &&
 			    oa->intra_brouter_calc) {
 				if (is_debug)
 					zlog_debug(
-						   "%s: intra_brouter_calc is on, skip brouter remove: %s (%p)",
-						   __PRETTY_FUNCTION__, buf,
-						   (void *)old);
+						"%s: intra_brouter_calc is on, skip brouter remove: %s (%p)",
+						__func__, buf, (void *)old);
 			} else {
 				if (is_debug)
-					zlog_debug("%s: remove old entry: %s %p ",
-						   __PRETTY_FUNCTION__, buf,
-						   (void *)old);
-				ospf6_route_remove(old, table);
+					zlog_debug(
+						"%s: remove old entry: %s %p ",
+						__func__, buf, (void *)old);
+				ospf6_abr_old_route_remove(lsa, old, table);
 			}
 		}
 		return;
@@ -1118,7 +1146,11 @@ void ospf6_abr_examin_summary(struct ospf6_lsa *lsa, struct ospf6_area *oa)
 	   are identical.
 	*/
 	old = ospf6_route_lookup(&prefix, table);
-
+	if (old) {
+		if (is_debug)
+			zlog_debug("%s: found old route %pFX, paths %d",
+				   __func__, &prefix, listcount(old->paths));
+	}
 	for (old_route = old; old_route; old_route = old_route->next) {
 		if (!ospf6_route_is_same(old_route, route) ||
 			(old_route->type != route->type) ||
@@ -1128,11 +1160,10 @@ void ospf6_abr_examin_summary(struct ospf6_lsa *lsa, struct ospf6_area *oa)
 		if ((ospf6_route_cmp(route, old_route) != 0)) {
 			if (is_debug) {
 				prefix2str(&prefix, buf, sizeof(buf));
-				zlog_debug("%s: old %p %s cost %u new route cost %u are not same",
-					   __PRETTY_FUNCTION__,
-					   (void *)old_route, buf,
-					   old_route->path.cost,
-					   route->path.cost);
+				zlog_debug(
+					"%s: old %p %s cost %u new route cost %u are not same",
+					__func__, (void *)old_route, buf,
+					old_route->path.cost, route->path.cost);
 			}
 
 			/* Check new route's adv. router is same in one of
@@ -1167,30 +1198,43 @@ void ospf6_abr_examin_summary(struct ospf6_lsa *lsa, struct ospf6_area *oa)
 			listnode_add_sort(old_route->paths, ecmp_path);
 
 			if (is_debug) {
-				prefix2str(&route->prefix, buf, sizeof(buf));
-				inet_ntop(AF_INET,
-					  &ecmp_path->origin.adv_router,
-					  adv_router, sizeof(adv_router));
-				zlog_debug("%s: route %s cost %u another path %s added with nh %u, effective paths %u nh %u",
-						__PRETTY_FUNCTION__, buf,
-						old_route->path.cost,
-						adv_router,
-						listcount(ecmp_path->nh_list),
-						old_route->paths ?
-						listcount(old_route->paths) : 0,
-						listcount(old_route->nh_list));
+				zlog_debug(
+					"%s: route %pFX cost %u another path %pI4 added with nh %u, effective paths %u nh %u",
+					__func__, &route->prefix,
+					old_route->path.cost,
+					&ecmp_path->origin.adv_router,
+					listcount(ecmp_path->nh_list),
+					old_route->paths
+						? listcount(old_route->paths)
+						: 0,
+					listcount(old_route->nh_list));
 			}
 		} else {
-			/* adv. router exists in the list, update the nhs */
-			list_delete_all_node(o_path->nh_list);
-			ospf6_copy_nexthops(o_path->nh_list, route->nh_list);
+			struct ospf6_route *tmp_route = ospf6_route_create();
+
+			ospf6_copy_nexthops(tmp_route->nh_list,
+					    o_path->nh_list);
+
+			if (ospf6_route_cmp_nexthops(tmp_route, route) != 0) {
+				/* adv. router exists in the list, update nhs */
+				list_delete_all_node(o_path->nh_list);
+				ospf6_copy_nexthops(o_path->nh_list,
+						    route->nh_list);
+				ospf6_route_delete(tmp_route);
+			} else {
+				/* adv. router has no change in nhs */
+				old_entry_updated = false;
+				ospf6_route_delete(tmp_route);
+				continue;
+			}
 		}
 
 		if (is_debug)
-			zlog_debug("%s: Update route: %s %p old cost %u new cost %u nh %u",
-				   __PRETTY_FUNCTION__, buf, (void *)old_route,
-				   old_route->path.cost, route->path.cost,
-				   listcount(route->nh_list));
+			zlog_debug(
+				"%s: Update route: %s %p old cost %u new cost %u nh %u",
+				__func__, buf, (void *)old_route,
+				old_route->path.cost, route->path.cost,
+				listcount(old_route->nh_list));
 
 		/* For Inter-Prefix route: Update RIB/FIB,
 		 * For Inter-Router trigger summary update
@@ -1203,13 +1247,22 @@ void ospf6_abr_examin_summary(struct ospf6_lsa *lsa, struct ospf6_area *oa)
 		break;
 	}
 
+	/* If the old entry is not updated and old entry not found or old entry
+	 * does not match with the new entry then add the new route
+	 */
 	if (old_entry_updated == false) {
+		if ((old == NULL) || (old->type != route->type)
+		    || (old->path.type != route->path.type))
+			add_route = true;
+	}
+
+	if (add_route) {
 		if (is_debug) {
-			inet_ntop(AF_INET, &route->path.origin.adv_router,
-				  adv_router, sizeof(adv_router));
-			zlog_debug("%s: Install route: %s cost %u nh %u adv_router %s ",
-				   __PRETTY_FUNCTION__, buf, route->path.cost,
-				   listcount(route->nh_list), adv_router);
+			zlog_debug(
+				"%s: Install new route: %s cost %u nh %u adv_router %pI4",
+				__func__, buf, route->path.cost,
+				listcount(route->nh_list),
+				&route->path.origin.adv_router);
 		}
 
 		path = ospf6_path_dup(&route->path);
@@ -1406,7 +1459,7 @@ void install_element_ospf6_debug_abr(void)
 	install_element(CONFIG_NODE, &no_debug_ospf6_abr_cmd);
 }
 
-struct ospf6_lsa_handler inter_prefix_handler = {
+static struct ospf6_lsa_handler inter_prefix_handler = {
 	.lh_type = OSPF6_LSTYPE_INTER_PREFIX,
 	.lh_name = "Inter-Prefix",
 	.lh_short_name = "IAP",
@@ -1414,7 +1467,7 @@ struct ospf6_lsa_handler inter_prefix_handler = {
 	.lh_get_prefix_str = ospf6_inter_area_prefix_lsa_get_prefix_str,
 	.lh_debug = 0};
 
-struct ospf6_lsa_handler inter_router_handler = {
+static struct ospf6_lsa_handler inter_router_handler = {
 	.lh_type = OSPF6_LSTYPE_INTER_ROUTER,
 	.lh_name = "Inter-Router",
 	.lh_short_name = "IAR",
