@@ -76,7 +76,8 @@ static char *ospf6_router_lsa_get_nbr_id(struct ospf6_lsa *lsa, char *buf,
 				  *)(start
 				     + pos * (sizeof(struct
 						     ospf6_router_lsdesc)));
-		if ((char *)lsdesc < end) {
+		if ((char *)lsdesc + sizeof(struct ospf6_router_lsdesc)
+		    <= end) {
 			if (buf && (buflen > INET_ADDRSTRLEN * 2)) {
 				inet_ntop(AF_INET,
 					  &lsdesc->neighbor_interface_id, buf1,
@@ -84,12 +85,13 @@ static char *ospf6_router_lsa_get_nbr_id(struct ospf6_lsa *lsa, char *buf,
 				inet_ntop(AF_INET, &lsdesc->neighbor_router_id,
 					  buf2, sizeof(buf2));
 				sprintf(buf, "%s/%s", buf2, buf1);
+
+				return buf;
 			}
-		} else
-			return NULL;
+		}
 	}
 
-	return buf;
+	return NULL;
 }
 
 static int ospf6_router_lsa_show(struct vty *vty, struct ospf6_lsa *lsa)
@@ -172,13 +174,13 @@ int ospf6_router_is_stub_router(struct ospf6_lsa *lsa)
 				      + sizeof(struct ospf6_lsa_header));
 
 		if (!OSPF6_OPT_ISSET(rtr_lsa->options, OSPF6_OPT_R)) {
-			return (OSPF6_IS_STUB_ROUTER);
+			return OSPF6_IS_STUB_ROUTER;
 		} else if (!OSPF6_OPT_ISSET(rtr_lsa->options, OSPF6_OPT_V6)) {
-			return (OSPF6_IS_STUB_ROUTER_V6);
+			return OSPF6_IS_STUB_ROUTER_V6;
 		}
 	}
 
-	return (OSPF6_NOT_STUB_ROUTER);
+	return OSPF6_NOT_STUB_ROUTER;
 }
 
 int ospf6_router_lsa_originate(struct thread *thread)
@@ -411,14 +413,15 @@ static char *ospf6_network_lsa_get_ar_id(struct ospf6_lsa *lsa, char *buf,
 
 		if ((current + sizeof(struct ospf6_network_lsdesc)) <= end) {
 			lsdesc = (struct ospf6_network_lsdesc *)current;
-			if (buf)
+			if (buf) {
 				inet_ntop(AF_INET, &lsdesc->router_id, buf,
 					  buflen);
-		} else
-			return NULL;
+				return buf;
+			}
+		}
 	}
 
-	return (buf);
+	return NULL;
 }
 
 static int ospf6_network_lsa_show(struct vty *vty, struct ospf6_lsa *lsa)
@@ -596,22 +599,21 @@ static char *ospf6_link_lsa_get_prefix_str(struct ospf6_lsa *lsa, char *buf,
 
 		prefixnum = ntohl(link_lsa->prefix_num);
 		if (pos > prefixnum)
-			return (NULL);
+			return NULL;
 
 		start = (char *)link_lsa + sizeof(struct ospf6_link_lsa);
 		end = (char *)lsa->header + ntohs(lsa->header->length);
 		current = start;
 
-		do {
+		while (current + sizeof(struct ospf6_prefix) <= end) {
 			prefix = (struct ospf6_prefix *)current;
 			if (prefix->prefix_length == 0
 			    || current + OSPF6_PREFIX_SIZE(prefix) > end) {
-				return (NULL);
+				return NULL;
 			}
 
-			if (cnt < pos) {
-				current =
-					start + pos * OSPF6_PREFIX_SIZE(prefix);
+			if (cnt < (pos - 1)) {
+				current += OSPF6_PREFIX_SIZE(prefix);
 				cnt++;
 			} else {
 				memset(&in6, 0, sizeof(in6));
@@ -621,9 +623,9 @@ static char *ospf6_link_lsa_get_prefix_str(struct ospf6_lsa *lsa, char *buf,
 				inet_ntop(AF_INET6, &in6, buf, buflen);
 				return (buf);
 			}
-		} while (current <= end);
+		}
 	}
-	return (NULL);
+	return NULL;
 }
 
 static int ospf6_link_lsa_show(struct vty *vty, struct ospf6_lsa *lsa)
@@ -796,15 +798,15 @@ static char *ospf6_intra_prefix_lsa_get_prefix_str(struct ospf6_lsa *lsa,
 				    + sizeof(struct ospf6_lsa_header));
 
 		prefixnum = ntohs(intra_prefix_lsa->prefix_num);
-		if (pos > prefixnum)
-			return (NULL);
+		if ((pos + 1) > prefixnum)
+			return NULL;
 
 		start = (char *)intra_prefix_lsa
 			+ sizeof(struct ospf6_intra_prefix_lsa);
 		end = (char *)lsa->header + ntohs(lsa->header->length);
 		current = start;
 
-		do {
+		while (current + sizeof(struct ospf6_prefix) <= end) {
 			prefix = (struct ospf6_prefix *)current;
 			if (prefix->prefix_length == 0
 			    || current + OSPF6_PREFIX_SIZE(prefix) > end) {
@@ -812,8 +814,7 @@ static char *ospf6_intra_prefix_lsa_get_prefix_str(struct ospf6_lsa *lsa,
 			}
 
 			if (cnt < pos) {
-				current =
-					start + pos * OSPF6_PREFIX_SIZE(prefix);
+				current += OSPF6_PREFIX_SIZE(prefix);
 				cnt++;
 			} else {
 				memset(&in6, 0, sizeof(in6));
@@ -825,9 +826,9 @@ static char *ospf6_intra_prefix_lsa_get_prefix_str(struct ospf6_lsa *lsa,
 					prefix->prefix_length);
 				return (buf);
 			}
-		} while (current <= end);
+		}
 	}
-	return (buf);
+	return NULL;
 }
 
 static int ospf6_intra_prefix_lsa_show(struct vty *vty, struct ospf6_lsa *lsa)
@@ -1022,8 +1023,9 @@ int ospf6_intra_prefix_lsa_originate_stub(struct thread *thread)
 	 */
 	if (oa->intra_prefix_originate) {
 		if (IS_OSPF6_DEBUG_ORIGINATE(INTRA_PREFIX))
-			zlog_debug("%s: Re-originate intra prefix LSA, Current full nbrs %u",
-				   __PRETTY_FUNCTION__, oa->full_nbrs);
+			zlog_debug(
+				"%s: Re-originate intra prefix LSA, Current full nbrs %u",
+				__func__, oa->full_nbrs);
 		if (old)
 			ospf6_lsa_purge_multi_ls_id(oa, old);
 		oa->intra_prefix_originate = 0;
@@ -1345,6 +1347,8 @@ static void ospf6_intra_prefix_update_route_origin(struct ospf6_route *oa_route)
 			g_route->path.origin.id = h_path->origin.id;
 			g_route->path.origin.adv_router =
 				h_path->origin.adv_router;
+			if (nroute)
+				ospf6_route_unlock(nroute);
 			break;
 		}
 	}
@@ -1367,7 +1371,7 @@ void ospf6_intra_prefix_route_ecmp_path(struct ospf6_area *oa,
 	struct ospf6_nexthop *nh, *rnh;
 	char buf[PREFIX2STR_BUFFER];
 	bool route_found = false;
-	struct interface *ifp;
+	struct interface *ifp = NULL;
 	struct ospf6_lsa *lsa;
 	struct ospf6_intra_prefix_lsa *intra_prefix_lsa;
 
@@ -1399,11 +1403,10 @@ void ospf6_intra_prefix_route_ecmp_path(struct ospf6_area *oa,
 				continue;
 
 			if (IS_OSPF6_DEBUG_EXAMIN(INTRA_PREFIX)) {
-				prefix2str(&old_route->prefix, buf,
-					   sizeof(buf));
-				zlog_debug("%s: route %s cost old %u new %u is not same, replace route",
-					   __PRETTY_FUNCTION__, buf,
-					   o_path->cost, route->path.cost);
+				zlog_debug(
+					"%s: route %pFX cost old %u new %u is not same, replace route",
+					__func__, &old_route->prefix, o_path->cost,
+					route->path.cost);
 			}
 
 			/* Remove selected current path's nh from
@@ -1456,12 +1459,11 @@ void ospf6_intra_prefix_route_ecmp_path(struct ospf6_area *oa,
 				}
 			} else {
 				if (IS_OSPF6_DEBUG_EXAMIN(INTRA_PREFIX)) {
-					prefix2str(&old_route->prefix, buf,
-						   sizeof(buf));
-					zlog_debug("%s: route %s old cost %u new cost %u, delete old entry.",
-						   __PRETTY_FUNCTION__, buf,
-						   old_route->path.cost,
-						   route->path.cost);
+					zlog_debug(
+						"%s: route %pFX old cost %u new cost %u, delete old entry.",
+						__func__, &old_route->prefix,
+						old_route->path.cost,
+						route->path.cost);
 				}
 				if (oa->route_table->hook_remove)
 					ospf6_route_remove(old_route,
@@ -1512,17 +1514,15 @@ void ospf6_intra_prefix_route_ecmp_path(struct ospf6_area *oa,
 				listnode_add_sort(old_route->paths, ecmp_path);
 
 				if (IS_OSPF6_DEBUG_EXAMIN(INTRA_PREFIX)) {
-					prefix2str(&route->prefix, buf,
-						   sizeof(buf));
 					zlog_debug(
-						"%s: route %s %p another path added with nh %u, effective paths %u nh %u",
-						__PRETTY_FUNCTION__, buf,
+						"%s: route %pFX %p another path added with nh %u, effective paths %u nh %u",
+						__func__, &route->prefix,
 						(void *)old_route,
 						listcount(ecmp_path->nh_list),
-						old_route->paths ?
-						listcount(old_route->paths) : 0,
+						old_route->paths ? listcount(
+							old_route->paths)
+								 : 0,
 						listcount(old_route->nh_list));
-
 				}
 			} else {
 				list_delete_all_node(o_path->nh_list);
@@ -1540,9 +1540,9 @@ void ospf6_intra_prefix_route_ecmp_path(struct ospf6_area *oa,
 							oa->spf_table);
 				if (ls_entry == NULL) {
 					if (IS_OSPF6_DEBUG_EXAMIN(INTRA_PREFIX))
-						zlog_debug("%s: ls_prfix %s ls_entry not found.",
-							   __PRETTY_FUNCTION__,
-							   buf);
+						zlog_debug(
+							"%s: ls_prfix %s ls_entry not found.",
+							__func__, buf);
 					continue;
 				}
 				lsa = ospf6_lsdb_lookup(o_path->origin.type,
@@ -1559,9 +1559,9 @@ void ospf6_intra_prefix_route_ecmp_path(struct ospf6_area *oa,
 						o_path->origin.id, &adv_prefix);
 						prefix2str(&adv_prefix, buf,
 							   sizeof(buf));
-						zlog_debug("%s: adv_router %s lsa not found",
-							   __PRETTY_FUNCTION__,
-							   buf);
+						zlog_debug(
+							"%s: adv_router %s lsa not found",
+							__func__, buf);
 					}
 					continue;
 				}
@@ -1572,14 +1572,22 @@ void ospf6_intra_prefix_route_ecmp_path(struct ospf6_area *oa,
 				if (intra_prefix_lsa->ref_adv_router
 				     == oa->ospf6->router_id) {
 					ifp = if_lookup_prefix(
-							&old_route->prefix,
-							VRF_DEFAULT);
-					if (ifp)
-						ospf6_route_add_nexthop(
-								old_route,
+						&old_route->prefix,
+						oa->ospf6->vrf_id);
+				}
+
+				if (ifp) {
+					/* Nexthop interface found */
+					ospf6_route_add_nexthop(old_route,
 								ifp->ifindex,
 								NULL);
 				} else {
+					/* The connected interfaces between
+					 * routers can be in different networks.
+					 * In this case the matching interface
+					 * is not found. Copy nexthops from the
+					 * link state entry
+					 */
 					ospf6_route_merge_nexthops(old_route,
 								   ls_entry);
 				}
@@ -1587,12 +1595,13 @@ void ospf6_intra_prefix_route_ecmp_path(struct ospf6_area *oa,
 
 			if (IS_OSPF6_DEBUG_EXAMIN(INTRA_PREFIX)) {
 				prefix2str(&route->prefix, buf, sizeof(buf));
-				zlog_debug("%s: route %s %p with final effective paths %u nh%u",
-					   __PRETTY_FUNCTION__, buf,
-					   (void *)old_route,
-					   old_route->paths ?
-					   listcount(old_route->paths) : 0,
-					   listcount(old_route->nh_list));
+				zlog_debug(
+					"%s: route %s %p with final effective paths %u nh%u",
+					__func__, buf, (void *)old_route,
+					old_route->paths
+						? listcount(old_route->paths)
+						: 0,
+					listcount(old_route->nh_list));
 			}
 
 			/* used in intra_route_calculation() to add to
@@ -1628,7 +1637,7 @@ void ospf6_intra_prefix_lsa_add(struct ospf6_lsa *lsa)
 	struct ospf6_prefix *op;
 	char *start, *current, *end;
 	char buf[PREFIX2STR_BUFFER];
-	struct interface *ifp;
+	struct interface *ifp = NULL;
 	int direct_connect = 0;
 	struct ospf6_path *path;
 
@@ -1636,7 +1645,7 @@ void ospf6_intra_prefix_lsa_add(struct ospf6_lsa *lsa)
 		return;
 
 	if (IS_OSPF6_DEBUG_EXAMIN(INTRA_PREFIX))
-		zlog_debug("%s: LSA %s found", __PRETTY_FUNCTION__, lsa->name);
+		zlog_debug("%s: LSA %s found", __func__, lsa->name);
 
 	oa = OSPF6_AREA(lsa->lsdb->data);
 
@@ -1716,11 +1725,19 @@ void ospf6_intra_prefix_lsa_add(struct ospf6_lsa *lsa)
 		memcpy(&route->path.ls_prefix, &ls_prefix,
 		       sizeof(struct prefix));
 		if (direct_connect) {
-			ifp = if_lookup_prefix(&route->prefix, VRF_DEFAULT);
-			if (ifp)
-				ospf6_route_add_nexthop(route, ifp->ifindex,
-							NULL);
+			ifp = if_lookup_prefix(&route->prefix,
+					       oa->ospf6->vrf_id);
+		}
+
+		if (ifp) {
+			/* Nexthop interface found */
+			ospf6_route_add_nexthop(route, ifp->ifindex, NULL);
 		} else {
+			/* The connected interfaces between routers can be in
+			 * different networks. In this case the matching
+			 * interface is not found. Copy nexthops from the
+			 * link state entry
+			 */
 			ospf6_route_copy_nexthops(route, ls_entry);
 		}
 
@@ -1729,24 +1746,25 @@ void ospf6_intra_prefix_lsa_add(struct ospf6_lsa *lsa)
 		listnode_add_sort(route->paths, path);
 
 		old = ospf6_route_lookup(&route->prefix, oa->route_table);
-		if (old && (ospf6_route_cmp(route, old) == 0)) {
+		if (old) {
 			if (IS_OSPF6_DEBUG_EXAMIN(INTRA_PREFIX)) {
 				prefix2str(&route->prefix, buf, sizeof(buf));
-				zlog_debug("%s Update route: %s old cost %u new cost %u paths %u nh %u",
-					   __PRETTY_FUNCTION__, buf,
-					   old->path.cost, route->path.cost,
-					   listcount(route->paths),
-					   listcount(route->nh_list));
+				zlog_debug(
+					"%s Update route: %s old cost %u new cost %u paths %u nh %u",
+					__func__, buf, old->path.cost,
+					route->path.cost,
+					listcount(route->paths),
+					listcount(route->nh_list));
 			}
 			ospf6_intra_prefix_route_ecmp_path(oa, old, route);
 		} else {
 			if (IS_OSPF6_DEBUG_EXAMIN(INTRA_PREFIX)) {
 				prefix2str(&route->prefix, buf, sizeof(buf));
-				zlog_debug("%s route %s add with cost %u paths %u nh %u",
-					   __PRETTY_FUNCTION__, buf,
-					   route->path.cost,
-					   listcount(route->paths),
-					   listcount(route->nh_list));
+				zlog_debug(
+					"%s route %s add with cost %u paths %u nh %u",
+					__func__, buf, route->path.cost,
+					listcount(route->paths),
+					listcount(route->nh_list));
 			}
 			ospf6_route_add(route, oa->route_table);
 		}
@@ -1783,7 +1801,7 @@ static void ospf6_intra_prefix_lsa_remove_update_route(struct ospf6_lsa *lsa,
 			prefix2str(&route->prefix, buf, sizeof(buf));
 			zlog_debug(
 				"%s: route %s path found with cost %u nh %u to remove.",
-				__PRETTY_FUNCTION__, buf, o_path->cost,
+				__func__, buf, o_path->cost,
 				listcount(o_path->nh_list));
 		}
 
@@ -1819,11 +1837,10 @@ static void ospf6_intra_prefix_lsa_remove_update_route(struct ospf6_lsa *lsa,
 
 		if (IS_OSPF6_DEBUG_EXAMIN(INTRA_PREFIX)) {
 			prefix2str(&route->prefix, buf, sizeof(buf));
-			zlog_debug("%s: route %s update paths %u nh %u",
-				   __PRETTY_FUNCTION__, buf,
-				   route->paths ? listcount(route->paths) : 0,
-				   route->nh_list ? listcount(route->nh_list)
-				   : 0);
+			zlog_debug(
+				"%s: route %s update paths %u nh %u", __func__,
+				buf, route->paths ? listcount(route->paths) : 0,
+				route->nh_list ? listcount(route->nh_list) : 0);
 		}
 
 		/* Update Global Route table and
@@ -1859,8 +1876,7 @@ void ospf6_intra_prefix_lsa_remove(struct ospf6_lsa *lsa)
 	char buf[PREFIX2STR_BUFFER];
 
 	if (IS_OSPF6_DEBUG_EXAMIN(INTRA_PREFIX))
-		zlog_debug("%s: %s disappearing", __PRETTY_FUNCTION__,
-			   lsa->name);
+		zlog_debug("%s: %s disappearing", __func__, lsa->name);
 
 	oa = OSPF6_AREA(lsa->lsdb->data);
 
@@ -1917,12 +1933,12 @@ void ospf6_intra_prefix_lsa_remove(struct ospf6_lsa *lsa)
 				if (IS_OSPF6_DEBUG_EXAMIN(INTRA_PREFIX)) {
 					prefix2str(&route->prefix, buf,
 						   sizeof(buf));
-					zlog_debug("%s: route remove %s with path type %u cost %u paths %u nh %u",
-						   __PRETTY_FUNCTION__, buf,
-						   route->path.type,
-						   route->path.cost,
-						   listcount(route->paths),
-						   listcount(route->nh_list));
+					zlog_debug(
+						"%s: route remove %s with path type %u cost %u paths %u nh %u",
+						__func__, buf, route->path.type,
+						route->path.cost,
+						listcount(route->paths),
+						listcount(route->nh_list));
 				}
 				ospf6_route_remove(route, oa->route_table);
 			}
@@ -2054,8 +2070,8 @@ void ospf6_intra_brouter_calculation(struct ospf6_area *oa)
 
 	if (IS_OSPF6_DEBUG_BROUTER_SPECIFIC_AREA_ID(oa->area_id) ||
 	    IS_OSPF6_DEBUG_ROUTE(MEMORY))
-		zlog_info("%s: border-router calculation for area %s",
-			  __PRETTY_FUNCTION__, oa->name);
+		zlog_info("%s: border-router calculation for area %s", __func__,
+			  oa->name);
 
 	hook_add = oa->ospf6->brouter_table->hook_add;
 	hook_remove = oa->ospf6->brouter_table->hook_remove;
@@ -2161,10 +2177,11 @@ void ospf6_intra_brouter_calculation(struct ospf6_area *oa)
 
 			if (ospf6_route_lookup(&adv_prefix, oa->spf_table)) {
 				if (IS_OSPF6_DEBUG_BROUTER) {
-					zlog_debug("%s: keep inter brouter %s as adv router 0x%x found in spf",
-						   __PRETTY_FUNCTION__,
-						   brouter_name,
-					brouter->path.origin.adv_router);
+					zlog_debug(
+						"%s: keep inter brouter %s as adv router 0x%x found in spf",
+						__func__, brouter_name,
+						brouter->path.origin
+							.adv_router);
 					ospf6_brouter_debug_print(brouter);
 				}
 				UNSET_FLAG(brouter->flag, OSPF6_ROUTE_REMOVE);
@@ -2183,9 +2200,9 @@ void ospf6_intra_brouter_calculation(struct ospf6_area *oa)
 				       brouter_id)
 			    || IS_OSPF6_DEBUG_BROUTER_SPECIFIC_AREA_ID(
 				       oa->area_id))
-				zlog_info("%s: brouter %s disappears via area %s",
-					  __PRETTY_FUNCTION__, brouter_name,
-					  oa->name);
+				zlog_info(
+					"%s: brouter %s disappears via area %s",
+					__func__, brouter_name, oa->name);
 			/* This is used to protect nbrouter from removed from
 			 * the table. For an example, ospf6_abr_examin_summary,
 			 * removes brouters which are marked for remove.
@@ -2201,8 +2218,7 @@ void ospf6_intra_brouter_calculation(struct ospf6_area *oa)
 			    || IS_OSPF6_DEBUG_BROUTER_SPECIFIC_AREA_ID(
 				       oa->area_id))
 				zlog_info("%s: brouter %s appears via area %s",
-					  __PRETTY_FUNCTION__, brouter_name,
-					  oa->name);
+					  __func__, brouter_name, oa->name);
 
 			/* newly added */
 			if (hook_add)
@@ -2229,34 +2245,34 @@ void ospf6_intra_brouter_calculation(struct ospf6_area *oa)
 	if (IS_OSPF6_DEBUG_BROUTER_SPECIFIC_AREA_ID(oa->area_id) ||
 	    IS_OSPF6_DEBUG_ROUTE(MEMORY))
 		zlog_info("%s: border-router calculation for area %s: done",
-			  __PRETTY_FUNCTION__, oa->name);
+			  __func__, oa->name);
 }
 
-struct ospf6_lsa_handler router_handler = {.lh_type = OSPF6_LSTYPE_ROUTER,
-					   .lh_name = "Router",
-					   .lh_short_name = "Rtr",
-					   .lh_show = ospf6_router_lsa_show,
-					   .lh_get_prefix_str =
-						   ospf6_router_lsa_get_nbr_id,
-					   .lh_debug = 0};
+static struct ospf6_lsa_handler router_handler = {
+	.lh_type = OSPF6_LSTYPE_ROUTER,
+	.lh_name = "Router",
+	.lh_short_name = "Rtr",
+	.lh_show = ospf6_router_lsa_show,
+	.lh_get_prefix_str = ospf6_router_lsa_get_nbr_id,
+	.lh_debug = 0};
 
-struct ospf6_lsa_handler network_handler = {.lh_type = OSPF6_LSTYPE_NETWORK,
-					    .lh_name = "Network",
-					    .lh_short_name = "Net",
-					    .lh_show = ospf6_network_lsa_show,
-					    .lh_get_prefix_str =
-						    ospf6_network_lsa_get_ar_id,
-					    .lh_debug = 0};
+static struct ospf6_lsa_handler network_handler = {
+	.lh_type = OSPF6_LSTYPE_NETWORK,
+	.lh_name = "Network",
+	.lh_short_name = "Net",
+	.lh_show = ospf6_network_lsa_show,
+	.lh_get_prefix_str = ospf6_network_lsa_get_ar_id,
+	.lh_debug = 0};
 
-struct ospf6_lsa_handler link_handler = {.lh_type = OSPF6_LSTYPE_LINK,
-					 .lh_name = "Link",
-					 .lh_short_name = "Lnk",
-					 .lh_show = ospf6_link_lsa_show,
-					 .lh_get_prefix_str =
-						 ospf6_link_lsa_get_prefix_str,
-					 .lh_debug = 0};
+static struct ospf6_lsa_handler link_handler = {
+	.lh_type = OSPF6_LSTYPE_LINK,
+	.lh_name = "Link",
+	.lh_short_name = "Lnk",
+	.lh_show = ospf6_link_lsa_show,
+	.lh_get_prefix_str = ospf6_link_lsa_get_prefix_str,
+	.lh_debug = 0};
 
-struct ospf6_lsa_handler intra_prefix_handler = {
+static struct ospf6_lsa_handler intra_prefix_handler = {
 	.lh_type = OSPF6_LSTYPE_INTRA_PREFIX,
 	.lh_name = "Intra-Prefix",
 	.lh_short_name = "INP",
