@@ -147,19 +147,19 @@ void ospf6_linkstate_prefix2str(struct prefix *prefix, char *buf, int size)
 }
 
 /* Global strings for logging */
-const char *ospf6_dest_type_str[OSPF6_DEST_TYPE_MAX] = {
+const char *const ospf6_dest_type_str[OSPF6_DEST_TYPE_MAX] = {
 	"Unknown", "Router", "Network", "Discard", "Linkstate", "AddressRange",
 };
 
-const char *ospf6_dest_type_substr[OSPF6_DEST_TYPE_MAX] = {
+const char *const ospf6_dest_type_substr[OSPF6_DEST_TYPE_MAX] = {
 	"?", "R", "N", "D", "L", "A",
 };
 
-const char *ospf6_path_type_str[OSPF6_PATH_TYPE_MAX] = {
+const char *const ospf6_path_type_str[OSPF6_PATH_TYPE_MAX] = {
 	"Unknown", "Intra-Area", "Inter-Area", "External-1", "External-2",
 };
 
-const char *ospf6_path_type_substr[OSPF6_PATH_TYPE_MAX] = {
+const char *const ospf6_path_type_substr[OSPF6_PATH_TYPE_MAX] = {
 	"??", "IA", "IE", "E1", "E2",
 };
 
@@ -174,8 +174,7 @@ struct ospf6_nexthop *ospf6_nexthop_create(void)
 
 void ospf6_nexthop_delete(struct ospf6_nexthop *nh)
 {
-	if (nh)
-		XFREE(MTYPE_OSPF6_NEXTHOP, nh);
+	XFREE(MTYPE_OSPF6_NEXTHOP, nh);
 }
 
 void ospf6_clear_nexthops(struct list *nh_list)
@@ -308,14 +307,14 @@ void ospf6_route_zebra_copy_nexthops(struct ospf6_route *route,
 				inet_ntop(AF_INET6, &nh->address, buf,
 					  sizeof(buf));
 				ifname = ifindex2ifname(nh->ifindex,
-							VRF_DEFAULT);
+							ospf6->vrf_id);
 				zlog_debug("  nexthop: %s%%%.*s(%d)", buf,
 					   IFNAMSIZ, ifname, nh->ifindex);
 			}
 			if (i >= entries)
 				return;
 
-			nexthops[i].vrf_id = VRF_DEFAULT;
+			nexthops[i].vrf_id = ospf6->vrf_id;
 			nexthops[i].ifindex = nh->ifindex;
 			if (!IN6_IS_ADDR_UNSPECIFIED(&nh->address)) {
 				nexthops[i].gate.ipv6 = nh->address;
@@ -332,11 +331,12 @@ int ospf6_route_get_first_nh_index(struct ospf6_route *route)
 	struct ospf6_nexthop *nh;
 
 	if (route) {
-		if ((nh = (struct ospf6_nexthop *)listhead(route->nh_list)))
-			return (nh->ifindex);
+		nh = listnode_head(route->nh_list);
+		if (nh)
+			return nh->ifindex;
 	}
 
-	return (-1);
+	return -1;
 }
 
 int ospf6_nexthop_cmp(struct ospf6_nexthop *a, struct ospf6_nexthop *b)
@@ -365,7 +365,7 @@ static int ospf6_path_cmp(struct ospf6_path *a, struct ospf6_path *b)
 void ospf6_path_free(struct ospf6_path *op)
 {
 	if (op->nh_list)
-		list_delete_and_null(&op->nh_list);
+		list_delete(&op->nh_list);
 	XFREE(MTYPE_OSPF6_PATH, op);
 }
 
@@ -413,9 +413,9 @@ void ospf6_route_delete(struct ospf6_route *route)
 {
 	if (route) {
 		if (route->nh_list)
-			list_delete_and_null(&route->nh_list);
+			list_delete(&route->nh_list);
 		if (route->paths)
-			list_delete_and_null(&route->paths);
+			list_delete(&route->paths);
 		XFREE(MTYPE_OSPF6_ROUTE, route);
 	}
 }
@@ -574,8 +574,9 @@ static void route_table_assert(struct ospf6_route_table *table)
 	if (link_error == 0 && num == table->count)
 		return;
 
-	flog_err(LIB_ERR_DEVELOPMENT, "PANIC !!");
-	flog_err(LIB_ERR_DEVELOPMENT, "Something has gone wrong with ospf6_route_table[%p]", table);
+	flog_err(EC_LIB_DEVELOPMENT, "PANIC !!");
+	flog_err(EC_LIB_DEVELOPMENT,
+		 "Something has gone wrong with ospf6_route_table[%p]", table);
 	zlog_debug("table count = %d, real number = %d", table->count, num);
 	zlog_debug("DUMP START");
 	for (r = ospf6_route_head(table); r; r = ospf6_route_next(r)) {
@@ -732,7 +733,7 @@ struct ospf6_route *ospf6_route_add(struct ospf6_route *route,
 		route->next = next;
 
 		if (node->info == next) {
-			assert(next->rnode == node);
+			assert(next && next->rnode == node);
 			node->info = route;
 			UNSET_FLAG(next->flag, OSPF6_ROUTE_BEST);
 			SET_FLAG(route->flag, OSPF6_ROUTE_BEST);
@@ -1041,6 +1042,11 @@ void ospf6_route_show(struct vty *vty, struct ospf6_route *route)
 	struct listnode *node;
 	struct ospf6_nexthop *nh;
 
+	if (ospf6 == NULL) {
+		vty_out(vty, "OSPFv3 is not running\n");
+		return;
+	}
+
 	monotime(&now);
 	timersub(&now, &route->changed, &res);
 	timerstring(&res, duration, sizeof(duration));
@@ -1059,7 +1065,7 @@ void ospf6_route_show(struct vty *vty, struct ospf6_route *route)
 	for (ALL_LIST_ELEMENTS_RO(route->nh_list, node, nh)) {
 		/* nexthop */
 		inet_ntop(AF_INET6, &nh->address, nexthop, sizeof(nexthop));
-		ifname = ifindex2ifname(nh->ifindex, VRF_DEFAULT);
+		ifname = ifindex2ifname(nh->ifindex, ospf6->vrf_id);
 
 		if (!i) {
 			vty_out(vty, "%c%1s %2s %-30s %-25s %6.*s %s\n",
@@ -1085,6 +1091,11 @@ void ospf6_route_show_detail(struct vty *vty, struct ospf6_route *route)
 	struct listnode *node;
 	struct ospf6_nexthop *nh;
 
+	if (ospf6 == NULL) {
+		vty_out(vty, "OSPFv3 is not running\n");
+		return;
+	}
+
 	monotime(&now);
 
 	/* destination */
@@ -1159,7 +1170,7 @@ void ospf6_route_show_detail(struct vty *vty, struct ospf6_route *route)
 	for (ALL_LIST_ELEMENTS_RO(route->nh_list, node, nh)) {
 		/* nexthop */
 		inet_ntop(AF_INET6, &nh->address, nexthop, sizeof(nexthop));
-		ifname = ifindex2ifname(nh->ifindex, VRF_DEFAULT);
+		ifname = ifindex2ifname(nh->ifindex, ospf6->vrf_id);
 		vty_out(vty, "  %s %.*s\n", nexthop, IFNAMSIZ, ifname);
 	}
 	vty_out(vty, "\n");
