@@ -955,8 +955,6 @@ static uint8_t *ospfv3WwLsdbEntry(struct variable *v, oid *name, size_t *length,
 		else if (v->magic & OSPFv3WWLINKTABLE) {
 			/* We build a sorted list of interfaces */
 			ifslist = list_new();
-			if (!ifslist)
-				return NULL;
 			ifslist->cmp = (int (*)(void *, void *))if_icmp_func;
 			FOR_ALL_INTERFACES (vrf, iif)
 				listnode_add_sort(ifslist, iif);
@@ -985,6 +983,7 @@ static uint8_t *ospfv3WwLsdbEntry(struct variable *v, oid *name, size_t *length,
 			}
 
 			list_delete_all_node(ifslist);
+			list_delete(&ifslist);
 		}
 	}
 
@@ -1089,8 +1088,6 @@ static uint8_t *ospfv3IfEntry(struct variable *v, oid *name, size_t *length,
 	} else {
 		/* We build a sorted list of interfaces */
 		ifslist = list_new();
-		if (!ifslist)
-			return NULL;
 		ifslist->cmp = (int (*)(void *, void *))if_icmp_func;
 		FOR_ALL_INTERFACES (vrf, iif)
 			listnode_add_sort(ifslist, iif);
@@ -1110,6 +1107,7 @@ static uint8_t *ospfv3IfEntry(struct variable *v, oid *name, size_t *length,
 		}
 
 		list_delete_all_node(ifslist);
+		list_delete(&ifslist);
 	}
 
 	if (!oi)
@@ -1253,8 +1251,6 @@ static uint8_t *ospfv3NbrEntry(struct variable *v, oid *name, size_t *length,
 	} else {
 		/* We build a sorted list of interfaces */
 		ifslist = list_new();
-		if (!ifslist)
-			return NULL;
 		ifslist->cmp = (int (*)(void *, void *))if_icmp_func;
 		FOR_ALL_INTERFACES (vrf, iif)
 			listnode_add_sort(ifslist, iif);
@@ -1282,6 +1278,7 @@ static uint8_t *ospfv3NbrEntry(struct variable *v, oid *name, size_t *length,
 		}
 
 		list_delete_all_node(ifslist);
+		list_delete(&ifslist);
 	}
 
 	if (!oi || !on)
