@@ -48,8 +48,7 @@ unsigned char conf_debug_ospf6_zebra = 0;
 struct zclient *zclient = NULL;
 
 /* Router-id update message from zebra. */
-static int ospf6_router_id_update_zebra(int command, struct zclient *zclient,
-					zebra_size_t length, vrf_id_t vrf_id)
+static int ospf6_router_id_update_zebra(ZAPI_CALLBACK_ARGS)
 {
 	struct prefix router_id;
 	struct ospf6 *o = ospf6;
@@ -65,10 +64,9 @@ static int ospf6_router_id_update_zebra(int command, struct zclient *zclient,
 	if (IS_OSPF6_DEBUG_ZEBRA(RECV)) {
 		char buf[INET_ADDRSTRLEN];
 
-		zlog_debug("%s: zebra router-id %s update",
-			   __PRETTY_FUNCTION__,
-			   inet_ntop(AF_INET, &router_id.u.prefix4,
-				     buf, INET_ADDRSTRLEN));
+		zlog_debug("%s: zebra router-id %s update", __func__,
+			   inet_ntop(AF_INET, &router_id.u.prefix4, buf,
+				     INET_ADDRSTRLEN));
 	}
 
 	ospf6_router_id_update();
@@ -77,85 +75,28 @@ static int ospf6_router_id_update_zebra(int command, struct zclient *zclient,
 }
 
 /* redistribute function */
-void ospf6_zebra_redistribute(int type)
+void ospf6_zebra_redistribute(int type, vrf_id_t vrf_id)
 {
-	if (vrf_bitmap_check(zclient->redist[AFI_IP6][type], VRF_DEFAULT))
+	if (vrf_bitmap_check(zclient->redist[AFI_IP6][type], vrf_id))
 		return;
-	vrf_bitmap_set(zclient->redist[AFI_IP6][type], VRF_DEFAULT);
+	vrf_bitmap_set(zclient->redist[AFI_IP6][type], vrf_id);
 
 	if (zclient->sock > 0)
 		zebra_redistribute_send(ZEBRA_REDISTRIBUTE_ADD, zclient,
-					AFI_IP6, type, 0, VRF_DEFAULT);
+					AFI_IP6, type, 0, vrf_id);
 }
 
-void ospf6_zebra_no_redistribute(int type)
+void ospf6_zebra_no_redistribute(int type, vrf_id_t vrf_id)
 {
-	if (!vrf_bitmap_check(zclient->redist[AFI_IP6][type], VRF_DEFAULT))
+	if (!vrf_bitmap_check(zclient->redist[AFI_IP6][type], vrf_id))
 		return;
-	vrf_bitmap_unset(zclient->redist[AFI_IP6][type], VRF_DEFAULT);
+	vrf_bitmap_unset(zclient->redist[AFI_IP6][type], vrf_id);
 	if (zclient->sock > 0)
 		zebra_redistribute_send(ZEBRA_REDISTRIBUTE_DELETE, zclient,
-					AFI_IP6, type, 0, VRF_DEFAULT);
+					AFI_IP6, type, 0, vrf_id);
 }
 
-/* Inteface addition message from zebra. */
-static int ospf6_zebra_if_add(int command, struct zclient *zclient,
-			      zebra_size_t length, vrf_id_t vrf_id)
-{
-	struct interface *ifp;
-
-	ifp = zebra_interface_add_read(zclient->ibuf, vrf_id);
-	if (IS_OSPF6_DEBUG_ZEBRA(RECV))
-		zlog_debug("Zebra Interface add: %s index %d mtu %d", ifp->name,
-			   ifp->ifindex, ifp->mtu6);
-	ospf6_interface_if_add(ifp);
-	return 0;
-}
-
-static int ospf6_zebra_if_del(int command, struct zclient *zclient,
-			      zebra_size_t length, vrf_id_t vrf_id)
-{
-	struct interface *ifp;
-
-	if (!(ifp = zebra_interface_state_read(zclient->ibuf, vrf_id)))
-		return 0;
-
-	if (if_is_up(ifp))
-		zlog_warn("Zebra: got delete of %s, but interface is still up",
-			  ifp->name);
-
-	if (IS_OSPF6_DEBUG_ZEBRA(RECV))
-		zlog_debug("Zebra Interface delete: %s index %d mtu %d",
-			   ifp->name, ifp->ifindex, ifp->mtu6);
-
-	if_set_index(ifp, IFINDEX_INTERNAL);
-	return 0;
-}
-
-static int ospf6_zebra_if_state_update(int command, struct zclient *zclient,
-				       zebra_size_t length, vrf_id_t vrf_id)
-{
-	struct interface *ifp;
-
-	ifp = zebra_interface_state_read(zclient->ibuf, vrf_id);
-	if (ifp == NULL)
-		return 0;
-
-	if (IS_OSPF6_DEBUG_ZEBRA(RECV))
-		zlog_debug(
-			"Zebra Interface state change: "
-			"%s index %d flags %llx metric %d mtu %d bandwidth %d",
-			ifp->name, ifp->ifindex, (unsigned long long)ifp->flags,
-			ifp->metric, ifp->mtu6, ifp->bandwidth);
-
-	ospf6_interface_state_update(ifp);
-	return 0;
-}
-
-static int ospf6_zebra_if_address_update_add(int command,
-					     struct zclient *zclient,
-					     zebra_size_t length,
-					     vrf_id_t vrf_id)
+static int ospf6_zebra_if_address_update_add(ZAPI_CALLBACK_ARGS)
 {
 	struct connected *c;
 	char buf[128];
@@ -179,10 +120,7 @@ static int ospf6_zebra_if_address_update_add(int command,
 	return 0;
 }
 
-static int ospf6_zebra_if_address_update_delete(int command,
-						struct zclient *zclient,
-						zebra_size_t length,
-						vrf_id_t vrf_id)
+static int ospf6_zebra_if_address_update_delete(ZAPI_CALLBACK_ARGS)
 {
 	struct connected *c;
 	char buf[128];
@@ -204,13 +142,12 @@ static int ospf6_zebra_if_address_update_delete(int command,
 		ospf6_interface_state_update(c->ifp);
 	}
 
-	connected_free(c);
+	connected_free(&c);
 
 	return 0;
 }
 
-static int ospf6_zebra_read_route(int command, struct zclient *zclient,
-				  zebra_size_t length, vrf_id_t vrf_id)
+static int ospf6_zebra_read_route(ZAPI_CALLBACK_ARGS)
 {
 	struct zapi_route api;
 	unsigned long ifindex;
@@ -226,24 +163,27 @@ static int ospf6_zebra_read_route(int command, struct zclient *zclient,
 	if (CHECK_FLAG(api.message, ZAPI_MESSAGE_SRCPFX))
 		return 0;
 
+	if (IN6_IS_ADDR_LINKLOCAL(&api.prefix.u.prefix6))
+		return 0;
+
 	ifindex = api.nexthops[0].ifindex;
 	nexthop = &api.nexthops[0].gate.ipv6;
 
 	if (IS_OSPF6_DEBUG_ZEBRA(RECV)) {
 		char prefixstr[PREFIX2STR_BUFFER], nexthopstr[128];
-		prefix2str((struct prefix *)&api.prefix, prefixstr,
-			   sizeof(prefixstr));
+
+		prefix2str(&api.prefix, prefixstr, sizeof(prefixstr));
 		inet_ntop(AF_INET6, nexthop, nexthopstr, sizeof(nexthopstr));
 
 		zlog_debug(
 			"Zebra Receive route %s: %s %s nexthop %s ifindex %ld tag %" ROUTE_TAG_PRI,
-			(command == ZEBRA_REDISTRIBUTE_ROUTE_ADD ? "add"
-								 : "delete"),
+			(cmd == ZEBRA_REDISTRIBUTE_ROUTE_ADD ? "add"
+							     : "delete"),
 			zebra_route_string(api.type), prefixstr, nexthopstr,
 			ifindex, api.tag);
 	}
 
-	if (command == ZEBRA_REDISTRIBUTE_ROUTE_ADD)
+	if (cmd == ZEBRA_REDISTRIBUTE_ROUTE_ADD)
 		ospf6_asbr_redistribute_add(api.type, ifindex, &api.prefix,
 					    api.nexthop_num, nexthop, api.tag);
 	else
@@ -269,7 +209,8 @@ DEFUN (show_zebra,
 	vty_out(vty, "Zebra Information\n");
 	vty_out(vty, "  fail: %d\n", zclient->fail);
 	vty_out(vty, "  redistribute default: %d\n",
-		vrf_bitmap_check(zclient->default_information, VRF_DEFAULT));
+		vrf_bitmap_check(zclient->default_information[AFI_IP6],
+				 VRF_DEFAULT));
 	vty_out(vty, "  redistribute:");
 	for (i = 0; i < ZEBRA_ROUTE_MAX; i++) {
 		if (vrf_bitmap_check(zclient->redist[AFI_IP6][i], VRF_DEFAULT))
@@ -315,7 +256,7 @@ static void ospf6_zebra_route_update(int type, struct ospf6_route *request)
 	    && ospf6_route_is_same(request, request->next)) {
 		if (IS_OSPF6_DEBUG_ZEBRA(SEND))
 			zlog_debug(
-				"  Best-path removal resulted Sencondary addition");
+				"  Best-path removal resulted Secondary addition");
 		type = ADD;
 		request = request->next;
 	}
@@ -338,7 +279,7 @@ static void ospf6_zebra_route_update(int type, struct ospf6_route *request)
 	dest = &request->prefix;
 
 	memset(&api, 0, sizeof(api));
-	api.vrf_id = VRF_DEFAULT;
+	api.vrf_id = ospf6->vrf_id;
 	api.type = ZEBRA_ROUTE_OSPF6;
 	api.safi = SAFI_UNICAST;
 	api.prefix = *dest;
@@ -363,10 +304,10 @@ static void ospf6_zebra_route_update(int type, struct ospf6_route *request)
 		ret = zclient_route_send(ZEBRA_ROUTE_ADD, zclient, &api);
 
 	if (ret < 0)
-		flog_err(LIB_ERR_ZAPI_SOCKET,
-			  "zclient_route_send() %s failed: %s",
-			  (type == REM ? "delete" : "add"),
-			  safe_strerror(errno));
+		flog_err(EC_LIB_ZAPI_SOCKET,
+			 "zclient_route_send() %s failed: %s",
+			 (type == REM ? "delete" : "add"),
+			 safe_strerror(errno));
 
 	return;
 }
@@ -389,7 +330,7 @@ void ospf6_zebra_add_discard(struct ospf6_route *request)
 
 	if (!CHECK_FLAG(request->flag, OSPF6_ROUTE_BLACKHOLE_ADDED)) {
 		memset(&api, 0, sizeof(api));
-		api.vrf_id = VRF_DEFAULT;
+		api.vrf_id = ospf6->vrf_id;
 		api.type = ZEBRA_ROUTE_OSPF6;
 		api.safi = SAFI_UNICAST;
 		api.prefix = *dest;
@@ -422,7 +363,7 @@ void ospf6_zebra_delete_discard(struct ospf6_route *request)
 
 	if (CHECK_FLAG(request->flag, OSPF6_ROUTE_BLACKHOLE_ADDED)) {
 		memset(&api, 0, sizeof(api));
-		api.vrf_id = VRF_DEFAULT;
+		api.vrf_id = ospf6->vrf_id;
 		api.type = ZEBRA_ROUTE_OSPF6;
 		api.safi = SAFI_UNICAST;
 		api.prefix = *dest;
@@ -578,7 +519,7 @@ uint8_t ospf6_distance_apply(struct prefix_ipv6 *p, struct ospf6_route * or)
 static void ospf6_zebra_connected(struct zclient *zclient)
 {
 	/* Send the client registration */
-	bfd_client_sendmsg(zclient, ZEBRA_BFD_CLIENT_REGISTER);
+	bfd_client_sendmsg(zclient, ZEBRA_BFD_CLIENT_REGISTER, VRF_DEFAULT);
 
 	zclient_send_reg_requests(zclient, VRF_DEFAULT);
 }
@@ -586,14 +527,10 @@ static void ospf6_zebra_connected(struct zclient *zclient)
 void ospf6_zebra_init(struct thread_master *master)
 {
 	/* Allocate zebra structure. */
-	zclient = zclient_new_notify(master, &zclient_options_default);
+	zclient = zclient_new(master, &zclient_options_default);
 	zclient_init(zclient, ZEBRA_ROUTE_OSPF6, 0, &ospf6d_privs);
 	zclient->zebra_connected = ospf6_zebra_connected;
 	zclient->router_id_update = ospf6_router_id_update_zebra;
-	zclient->interface_add = ospf6_zebra_if_add;
-	zclient->interface_delete = ospf6_zebra_if_del;
-	zclient->interface_up = ospf6_zebra_if_state_update;
-	zclient->interface_down = ospf6_zebra_if_state_update;
 	zclient->interface_address_add = ospf6_zebra_if_address_update_add;
 	zclient->interface_address_delete =
 		ospf6_zebra_if_address_update_delete;
