@@ -39,7 +39,7 @@
 #include "ospfd/ospf_zebra.h"
 #include "ospfd/ospf_dump.h"
 
-struct ospf_route *ospf_route_new()
+struct ospf_route *ospf_route_new(void)
 {
 	struct ospf_route *new;
 
@@ -51,15 +51,15 @@ struct ospf_route *ospf_route_new()
 	return new;
 }
 
-void ospf_route_free(struct ospf_route * or)
+void ospf_route_free(struct ospf_route *or)
 {
 	if (or->paths)
-		list_delete_and_null(& or->paths);
+		list_delete(& or->paths);
 
 	XFREE(MTYPE_OSPF_ROUTE, or);
 }
 
-struct ospf_path *ospf_path_new()
+struct ospf_path *ospf_path_new(void)
 {
 	struct ospf_path *new;
 
@@ -322,8 +322,7 @@ void ospf_intra_add_router(struct route_table *rt, struct vertex *v,
 	if (!IS_ROUTER_LSA_BORDER(lsa) && !IS_ROUTER_LSA_EXTERNAL(lsa)) {
 		if (IS_DEBUG_OSPF_EVENT)
 			zlog_debug(
-				"ospf_intra_add_router: "
-				"this router is neither ASBR nor ABR, skipping it");
+				"ospf_intra_add_router: this router is neither ASBR nor ABR, skipping it");
 		return;
 	}
 
@@ -376,7 +375,7 @@ void ospf_intra_add_router(struct route_table *rt, struct vertex *v,
 	else
 		route_unlock_node(rn);
 
-	ospf_route_copy_nexthops_from_vertex(or, v);
+	ospf_route_copy_nexthops_from_vertex(area, or, v);
 
 	listnode_add(rn->info, or);
 
@@ -439,7 +438,7 @@ void ospf_intra_add_transit(struct route_table *rt, struct vertex *v,
 	or->type = OSPF_DESTINATION_NETWORK;
 	or->u.std.origin = (struct lsa_header *)lsa;
 
-	ospf_route_copy_nexthops_from_vertex(or, v);
+	ospf_route_copy_nexthops_from_vertex(area, or, v);
 
 	rn->info = or ;
 }
@@ -454,7 +453,7 @@ void ospf_intra_add_stub(struct route_table *rt, struct router_lsa_link *link,
 	struct ospf_route * or ;
 	struct prefix_ipv4 p;
 	struct router_lsa *lsa;
-	struct ospf_interface *oi;
+	struct ospf_interface *oi = NULL;
 	struct ospf_path *path;
 
 	if (IS_DEBUG_OSPF_EVENT)
@@ -506,8 +505,7 @@ void ospf_intra_add_stub(struct route_table *rt, struct router_lsa_link *link,
 
 		if (IS_DEBUG_OSPF_EVENT)
 			zlog_debug(
-				"ospf_intra_add_stub(): "
-				"another route to the same prefix found with cost %u",
+				"ospf_intra_add_stub(): another route to the same prefix found with cost %u",
 				cur_or->cost);
 
 		/* Compare this distance to the current best cost to the stub
@@ -540,7 +538,7 @@ void ospf_intra_add_stub(struct route_table *rt, struct router_lsa_link *link,
 				zlog_debug(
 					"ospf_intra_add_stub(): routes are equal, merge");
 
-			ospf_route_copy_nexthops_from_vertex(cur_or, v);
+			ospf_route_copy_nexthops_from_vertex(area, cur_or, v);
 
 			if (IPV4_ADDR_CMP(&cur_or->u.std.origin->id,
 					  &lsa->header.id)
@@ -565,7 +563,7 @@ void ospf_intra_add_stub(struct route_table *rt, struct router_lsa_link *link,
 
 			list_delete_all_node(cur_or->paths);
 
-			ospf_route_copy_nexthops_from_vertex(cur_or, v);
+			ospf_route_copy_nexthops_from_vertex(area, cur_or, v);
 
 			cur_or->u.std.origin = (struct lsa_header *)lsa;
 			return;
@@ -590,24 +588,35 @@ void ospf_intra_add_stub(struct route_table *rt, struct router_lsa_link *link,
 		if (IS_DEBUG_OSPF_EVENT)
 			zlog_debug(
 				"ospf_intra_add_stub(): this network is on remote router");
-		ospf_route_copy_nexthops_from_vertex(or, v);
+		ospf_route_copy_nexthops_from_vertex(area, or, v);
 	} else {
 		if (IS_DEBUG_OSPF_EVENT)
 			zlog_debug(
 				"ospf_intra_add_stub(): this network is on this router");
 
-		if ((oi = ospf_if_lookup_by_lsa_pos(area, lsa_pos))) {
+		/*
+		 * Only deal with interface data when we
+		 * don't do a dry run
+		 */
+		if (!area->spf_dry_run)
+			oi = ospf_if_lookup_by_lsa_pos(area, lsa_pos);
+
+		if (oi || area->spf_dry_run) {
 			if (IS_DEBUG_OSPF_EVENT)
 				zlog_debug(
-					"ospf_intra_add_stub(): the interface is %s",
-					IF_NAME(oi));
+					"ospf_intra_add_stub(): the lsa pos is %d",
+					lsa_pos);
 
 			path = ospf_path_new();
-			path->nexthop.s_addr = 0;
-			path->ifindex = oi->ifp->ifindex;
-			if (CHECK_FLAG(oi->connected->flags,
-				       ZEBRA_IFA_UNNUMBERED))
-				path->unnumbered = 1;
+			path->nexthop.s_addr = INADDR_ANY;
+
+			if (oi) {
+				path->ifindex = oi->ifp->ifindex;
+				if (CHECK_FLAG(oi->connected->flags,
+					       ZEBRA_IFA_UNNUMBERED))
+					path->unnumbered = 1;
+			}
+
 			listnode_add(or->paths, path);
 		} else {
 			if (IS_DEBUG_OSPF_EVENT)
@@ -622,34 +631,24 @@ void ospf_intra_add_stub(struct route_table *rt, struct router_lsa_link *link,
 		zlog_debug("ospf_intra_add_stub(): Stop");
 }
 
-const char *ospf_path_type_str[] = {"unknown-type", "intra-area", "inter-area",
-				    "type1-external", "type2-external"};
+static const char *const ospf_path_type_str[] = {
+	"unknown-type", "intra-area", "inter-area", "type1-external",
+	"type2-external"
+};
 
 void ospf_route_table_dump(struct route_table *rt)
 {
 	struct route_node *rn;
 	struct ospf_route * or ;
-	char buf1[BUFSIZ];
-	char buf2[BUFSIZ];
 	struct listnode *pnode;
 	struct ospf_path *path;
 
-#if 0
-  zlog_debug ("Type   Dest   Area   Path	 Type	 Cost	Next	 Adv.");
-  zlog_debug ("					Hop(s)	 Router(s)");
-#endif /* 0 */
-
 	zlog_debug("========== OSPF routing table ==========");
 	for (rn = route_top(rt); rn; rn = route_next(rn))
 		if ((or = rn->info) != NULL) {
 			if (or->type == OSPF_DESTINATION_NETWORK) {
-				zlog_debug("N %s/%d\t%s\t%s\t%d",
-					   inet_ntop(AF_INET, &rn->p.u.prefix4,
-						     buf1, BUFSIZ),
-					   rn->p.prefixlen,
-					   inet_ntop(AF_INET,
-						     & or->u.std.area_id, buf2,
-						     BUFSIZ),
+				zlog_debug("N %-18pFX %-15pI4 %s %d", &rn->p,
+					   &or->u.std.area_id,
 					   ospf_path_type_str[or->path_type],
 					   or->cost);
 				for (ALL_LIST_ELEMENTS_RO(or->paths, pnode,
@@ -657,18 +656,46 @@ void ospf_route_table_dump(struct route_table *rt)
 					zlog_debug("  -> %s",
 						   inet_ntoa(path->nexthop));
 			} else
-				zlog_debug("R %s\t%s\t%s\t%d",
-					   inet_ntop(AF_INET, &rn->p.u.prefix4,
-						     buf1, BUFSIZ),
-					   inet_ntop(AF_INET,
-						     & or->u.std.area_id, buf2,
-						     BUFSIZ),
+				zlog_debug("R %-18pI4 %-15pI4 %s %d",
+					   &rn->p.u.prefix4,
+					   &or->u.std.area_id,
 					   ospf_path_type_str[or->path_type],
 					   or->cost);
 		}
 	zlog_debug("========================================");
 }
 
+void ospf_route_table_print(struct vty *vty, struct route_table *rt)
+{
+	struct route_node *rn;
+	struct ospf_route * or ;
+	struct listnode *pnode;
+	struct ospf_path *path;
+
+	vty_out(vty, "========== OSPF routing table ==========\n");
+	for (rn = route_top(rt); rn; rn = route_next(rn))
+		if ((or = rn->info) != NULL) {
+			if (or->type == OSPF_DESTINATION_NETWORK) {
+				vty_out(vty, "N %-18pFX %-15pI4 %s %d\n",
+					&rn->p, & or->u.std.area_id,
+					ospf_path_type_str[or->path_type],
+					or->cost);
+				for (ALL_LIST_ELEMENTS_RO(or->paths, pnode,
+							  path))
+					vty_out(vty, "  -> %s\n",
+						path->nexthop.s_addr != 0
+							? inet_ntoa(
+								path->nexthop)
+							: "directly connected");
+			} else
+				vty_out(vty, "R %-18pI4 %-15pI4 %s %d\n",
+					&rn->p.u.prefix4, & or->u.std.area_id,
+					ospf_path_type_str[or->path_type],
+					or->cost);
+		}
+	vty_out(vty, "========================================\n");
+}
+
 /* This is 16.4.1 implementation.
    o Intra-area paths using non-backbone areas are always the most preferred.
    o The other paths, intra-area backbone paths and inter-area paths,
@@ -749,30 +776,41 @@ static int ospf_path_exist(struct list *plist, struct in_addr nexthop,
 	return 0;
 }
 
-void ospf_route_copy_nexthops_from_vertex(struct ospf_route *to,
+void ospf_route_copy_nexthops_from_vertex(struct ospf_area *area,
+					  struct ospf_route *to,
 					  struct vertex *v)
 {
 	struct listnode *node;
 	struct ospf_path *path;
 	struct vertex_nexthop *nexthop;
 	struct vertex_parent *vp;
+	struct ospf_interface *oi = NULL;
 
 	assert(to->paths);
 
 	for (ALL_LIST_ELEMENTS_RO(v->parents, node, vp)) {
 		nexthop = vp->nexthop;
 
-		if (nexthop->oi != NULL) {
-			if (!ospf_path_exist(to->paths, nexthop->router,
-					     nexthop->oi)) {
-				path = ospf_path_new();
-				path->nexthop = nexthop->router;
-				path->ifindex = nexthop->oi->ifp->ifindex;
-				if (CHECK_FLAG(nexthop->oi->connected->flags,
+		/*
+		 * Only deal with interface data when we
+		 * don't do a dry run
+		 */
+		if (!area->spf_dry_run)
+			oi = ospf_if_lookup_by_lsa_pos(area, nexthop->lsa_pos);
+
+		if ((oi && !ospf_path_exist(to->paths, nexthop->router, oi))
+		    || area->spf_dry_run) {
+			path = ospf_path_new();
+			path->nexthop = nexthop->router;
+
+			if (oi) {
+				path->ifindex = oi->ifp->ifindex;
+				if (CHECK_FLAG(oi->connected->flags,
 					       ZEBRA_IFA_UNNUMBERED))
 					path->unnumbered = 1;
-				listnode_add(to->paths, path);
 			}
+
+			listnode_add(to->paths, path);
 		}
 	}
 }
@@ -905,7 +943,7 @@ void ospf_prune_unreachable_routers(struct route_table *rtrs)
 				zlog_debug("Pruning router node %s",
 					   inet_ntoa(rn->p.u.prefix4));
 
-			list_delete_and_null(&paths);
+			list_delete(&paths);
 			rn->info = NULL;
 			route_unlock_node(rn);
 		}
@@ -936,16 +974,14 @@ int ospf_add_discard_route(struct ospf *ospf, struct route_table *rt,
 		if (or->path_type == OSPF_PATH_INTRA_AREA) {
 			if (IS_DEBUG_OSPF_EVENT)
 				zlog_debug(
-					"ospf_add_discard_route(): "
-					"an intra-area route exists");
+					"ospf_add_discard_route(): an intra-area route exists");
 			return 0;
 		}
 
 		if (or->type == OSPF_DESTINATION_DISCARD) {
 			if (IS_DEBUG_OSPF_EVENT)
 				zlog_debug(
-					"ospf_add_discard_route(): "
-					"discard entry already installed");
+					"ospf_add_discard_route(): discard entry already installed");
 			return 0;
 		}
 
@@ -954,13 +990,12 @@ int ospf_add_discard_route(struct ospf *ospf, struct route_table *rt,
 
 	if (IS_DEBUG_OSPF_EVENT)
 		zlog_debug(
-			"ospf_add_discard_route(): "
-			"adding %s/%d",
+			"ospf_add_discard_route(): adding %s/%d",
 			inet_ntoa(p->prefix), p->prefixlen);
 
 	new_or = ospf_route_new();
 	new_or->type = OSPF_DESTINATION_DISCARD;
-	new_or->id.s_addr = 0;
+	new_or->id.s_addr = INADDR_ANY;
 	new_or->cost = 0;
 	new_or->u.std.area_id = area->area_id;
 	new_or->u.std.external_routing = area->external_routing;
@@ -980,8 +1015,7 @@ void ospf_delete_discard_route(struct ospf *ospf, struct route_table *rt,
 
 	if (IS_DEBUG_OSPF_EVENT)
 		zlog_debug(
-			"ospf_delete_discard_route(): "
-			"deleting %s/%d",
+			"ospf_delete_discard_route(): deleting %s/%d",
 			inet_ntoa(p->prefix), p->prefixlen);
 
 	rn = route_node_lookup(rt, (struct prefix *)p);
@@ -998,16 +1032,14 @@ void ospf_delete_discard_route(struct ospf *ospf, struct route_table *rt,
 	if (or->path_type == OSPF_PATH_INTRA_AREA) {
 		if (IS_DEBUG_OSPF_EVENT)
 			zlog_debug(
-				"ospf_delete_discard_route(): "
-				"an intra-area route exists");
+				"ospf_delete_discard_route(): an intra-area route exists");
 		return;
 	}
 
 	if (or->type != OSPF_DESTINATION_DISCARD) {
 		if (IS_DEBUG_OSPF_EVENT)
 			zlog_debug(
-				"ospf_delete_discard_route(): "
-				"not a discard entry");
+				"ospf_delete_discard_route(): not a discard entry");
 		return;
 	}
 
