@@ -548,54 +548,45 @@ static uint8_t *ospfGeneralGroup(struct variable *v, oid *name, size_t *length,
 			return SNMP_IPADDRESS(ospf->router_id);
 		else
 			return SNMP_IPADDRESS(ospf_empty_addr);
-		break;
 	case OSPFADMINSTAT: /* 2 */
 		/* The administrative status of OSPF in the router. */
 		if (ospf_admin_stat(ospf))
 			return SNMP_INTEGER(OSPF_STATUS_ENABLED);
 		else
 			return SNMP_INTEGER(OSPF_STATUS_DISABLED);
-		break;
 	case OSPFVERSIONNUMBER: /* 3 */
 		/* OSPF version 2. */
 		return SNMP_INTEGER(OSPF_VERSION);
-		break;
 	case OSPFAREABDRRTRSTATUS: /* 4 */
 		/* Area Border router status. */
 		if (ospf && CHECK_FLAG(ospf->flags, OSPF_FLAG_ABR))
 			return SNMP_INTEGER(SNMP_TRUE);
 		else
 			return SNMP_INTEGER(SNMP_FALSE);
-		break;
 	case OSPFASBDRRTRSTATUS: /* 5 */
 		/* AS Border router status. */
 		if (ospf && CHECK_FLAG(ospf->flags, OSPF_FLAG_ASBR))
 			return SNMP_INTEGER(SNMP_TRUE);
 		else
 			return SNMP_INTEGER(SNMP_FALSE);
-		break;
 	case OSPFEXTERNLSACOUNT: /* 6 */
 		/* External LSA counts. */
 		if (ospf)
 			return SNMP_INTEGER(ospf_lsdb_count_all(ospf->lsdb));
 		else
 			return SNMP_INTEGER(0);
-		break;
 	case OSPFEXTERNLSACKSUMSUM: /* 7 */
 		/* External LSA checksum. */
 		return SNMP_INTEGER(0);
-		break;
 	case OSPFTOSSUPPORT: /* 8 */
 		/* TOS is not supported. */
 		return SNMP_INTEGER(SNMP_FALSE);
-		break;
 	case OSPFORIGINATENEWLSAS: /* 9 */
 		/* The number of new link-state advertisements. */
 		if (ospf)
 			return SNMP_INTEGER(ospf->lsa_originate_count);
 		else
 			return SNMP_INTEGER(0);
-		break;
 	case OSPFRXNEWLSAS: /* 10 */
 		/* The number of link-state advertisements received determined
 		   to be new instantiations. */
@@ -603,24 +594,19 @@ static uint8_t *ospfGeneralGroup(struct variable *v, oid *name, size_t *length,
 			return SNMP_INTEGER(ospf->rx_lsa_count);
 		else
 			return SNMP_INTEGER(0);
-		break;
 	case OSPFEXTLSDBLIMIT: /* 11 */
 		/* There is no limit for the number of non-default
 		   AS-external-LSAs. */
 		return SNMP_INTEGER(-1);
-		break;
 	case OSPFMULTICASTEXTENSIONS: /* 12 */
 		/* Multicast Extensions to OSPF is not supported. */
 		return SNMP_INTEGER(0);
-		break;
 	case OSPFEXITOVERFLOWINTERVAL: /* 13 */
 		/* Overflow is not supported. */
 		return SNMP_INTEGER(0);
-		break;
 	case OSPFDEMANDEXTENSIONS: /* 14 */
 		/* Demand routing is not supported. */
 		return SNMP_INTEGER(SNMP_FALSE);
-		break;
 	default:
 		return NULL;
 	}
@@ -717,28 +703,20 @@ static uint8_t *ospfAreaEntry(struct variable *v, oid *name, size_t *length,
 	switch (v->magic) {
 	case OSPFAREAID: /* 1 */
 		return SNMP_IPADDRESS(area->area_id);
-		break;
 	case OSPFAUTHTYPE: /* 2 */
 		return SNMP_INTEGER(area->auth_type);
-		break;
 	case OSPFIMPORTASEXTERN: /* 3 */
 		return SNMP_INTEGER(area->external_routing + 1);
-		break;
 	case OSPFSPFRUNS: /* 4 */
 		return SNMP_INTEGER(area->spf_calculation);
-		break;
 	case OSPFAREABDRRTRCOUNT: /* 5 */
 		return SNMP_INTEGER(area->abr_count);
-		break;
 	case OSPFASBDRRTRCOUNT: /* 6 */
 		return SNMP_INTEGER(area->asbr_count);
-		break;
 	case OSPFAREALSACOUNT: /* 7 */
 		return SNMP_INTEGER(area->lsdb->total);
-		break;
 	case OSPFAREALSACKSUMSUM: /* 8 */
 		return SNMP_INTEGER(0);
-		break;
 	case OSPFAREASUMMARY: /* 9 */
 #define OSPF_noAreaSummary   1
 #define OSPF_sendAreaSummary 2
@@ -746,13 +724,10 @@ static uint8_t *ospfAreaEntry(struct variable *v, oid *name, size_t *length,
 			return SNMP_INTEGER(OSPF_noAreaSummary);
 		else
 			return SNMP_INTEGER(OSPF_sendAreaSummary);
-		break;
 	case OSPFAREASTATUS: /* 10 */
 		return SNMP_INTEGER(SNMP_VALID);
-		break;
 	default:
 		return NULL;
-		break;
 	}
 	return NULL;
 }
@@ -809,7 +784,7 @@ static struct ospf_area *ospfStubAreaLookup(struct variable *v, oid name[],
 
 		area = ospf_area_lookup_by_area_id(ospf, *addr);
 
-		if (area->external_routing == OSPF_AREA_STUB)
+		if (area && area->external_routing == OSPF_AREA_STUB)
 			return area;
 		else
 			return NULL;
@@ -857,29 +832,23 @@ static uint8_t *ospfStubAreaEntry(struct variable *v, oid *name, size_t *length,
 	case OSPFSTUBAREAID: /* 1 */
 		/* OSPF stub area id. */
 		return SNMP_IPADDRESS(area->area_id);
-		break;
 	case OSPFSTUBTOS: /* 2 */
 		/* TOS value is not supported. */
 		return SNMP_INTEGER(0);
-		break;
 	case OSPFSTUBMETRIC: /* 3 */
 		/* Default cost to stub area. */
 		return SNMP_INTEGER(area->default_cost);
-		break;
 	case OSPFSTUBSTATUS: /* 4 */
 		/* Status of the stub area. */
 		return SNMP_INTEGER(SNMP_VALID);
-		break;
 	case OSPFSTUBMETRICTYPE: /* 5 */
 				 /* OSPF Metric type. */
 #define OSPF_ospfMetric     1
 #define OSPF_comparableCost 2
 #define OSPF_nonComparable  3
 		return SNMP_INTEGER(OSPF_ospfMetric);
-		break;
 	default:
 		return NULL;
-		break;
 	}
 	return NULL;
 }
@@ -901,7 +870,7 @@ static struct ospf_lsa *lsdb_lookup_next(struct ospf_area *area, uint8_t *type,
 	/* Sanity check, if LSA type unknwon
 	   merley skip any LSA */
 	if ((i < OSPF_MIN_LSA) || (i >= OSPF_MAX_LSA)) {
-		zlog_debug("Strange request with LSA type %d\n", i);
+		zlog_debug("Strange request with LSA type %d", i);
 		return NULL;
 	}
 
@@ -995,7 +964,6 @@ static struct ospf_lsa *ospfLsdbLookup(struct variable *v, oid *name,
 			if (len <= 0)
 				type_next = 1;
 			else {
-				len = 1;
 				type_next = 0;
 				*type = *offset;
 			}
@@ -1098,32 +1066,23 @@ static uint8_t *ospfLsdbEntry(struct variable *v, oid *name, size_t *length,
 	switch (v->magic) {
 	case OSPFLSDBAREAID: /* 1 */
 		return SNMP_IPADDRESS(lsa->area->area_id);
-		break;
 	case OSPFLSDBTYPE: /* 2 */
 		return SNMP_INTEGER(lsah->type);
-		break;
 	case OSPFLSDBLSID: /* 3 */
 		return SNMP_IPADDRESS(lsah->id);
-		break;
 	case OSPFLSDBROUTERID: /* 4 */
 		return SNMP_IPADDRESS(lsah->adv_router);
-		break;
 	case OSPFLSDBSEQUENCE: /* 5 */
 		return SNMP_INTEGER(lsah->ls_seqnum);
-		break;
 	case OSPFLSDBAGE: /* 6 */
 		return SNMP_INTEGER(lsah->ls_age);
-		break;
 	case OSPFLSDBCHECKSUM: /* 7 */
 		return SNMP_INTEGER(lsah->checksum);
-		break;
 	case OSPFLSDBADVERTISEMENT: /* 8 */
 		*var_len = ntohs(lsah->length);
 		return (uint8_t *)lsah;
-		break;
 	default:
 		return NULL;
-		break;
 	}
 	return NULL;
 }
@@ -1257,24 +1216,18 @@ static uint8_t *ospfAreaRangeEntry(struct variable *v, oid *name,
 	switch (v->magic) {
 	case OSPFAREARANGEAREAID: /* 1 */
 		return SNMP_IPADDRESS(area_id);
-		break;
 	case OSPFAREARANGENET: /* 2 */
 		return SNMP_IPADDRESS(range_net);
-		break;
 	case OSPFAREARANGEMASK: /* 3 */
 		return SNMP_IPADDRESS(mask);
-		break;
 	case OSPFAREARANGESTATUS: /* 4 */
 		return SNMP_INTEGER(SNMP_VALID);
-		break;
 	case OSPFAREARANGEEFFECT: /* 5 */
 #define OSPF_advertiseMatching      1
 #define OSPF_doNotAdvertiseMatching 2
 		return SNMP_INTEGER(OSPF_advertiseMatching);
-		break;
 	default:
 		return NULL;
-		break;
 	}
 	return NULL;
 }
@@ -1360,28 +1313,22 @@ static uint8_t *ospfHostEntry(struct variable *v, oid *name, size_t *length,
 	switch (v->magic) {
 	case OSPFHOSTIPADDRESS: /* 1 */
 		return SNMP_IPADDRESS(nbr_nbma->addr);
-		break;
 	case OSPFHOSTTOS: /* 2 */
 		return SNMP_INTEGER(0);
-		break;
 	case OSPFHOSTMETRIC: /* 3 */
 		if (oi)
 			return SNMP_INTEGER(oi->output_cost);
 		else
 			return SNMP_INTEGER(1);
-		break;
 	case OSPFHOSTSTATUS: /* 4 */
 		return SNMP_INTEGER(SNMP_VALID);
-		break;
 	case OSPFHOSTAREAID: /* 5 */
 		if (oi && oi->area)
 			return SNMP_IPADDRESS(oi->area->area_id);
 		else
 			return SNMP_IPADDRESS(ospf_empty_addr);
-		break;
 	default:
 		return NULL;
-		break;
 	}
 	return NULL;
 }
@@ -1684,80 +1631,59 @@ static uint8_t *ospfIfEntry(struct variable *v, oid *name, size_t *length,
 	switch (v->magic) {
 	case OSPFIFIPADDRESS: /* 1 */
 		return SNMP_IPADDRESS(ifaddr);
-		break;
 	case OSPFADDRESSLESSIF: /* 2 */
 		return SNMP_INTEGER(ifindex);
-		break;
 	case OSPFIFAREAID: /* 3 */
 		if (oi->area)
 			return SNMP_IPADDRESS(oi->area->area_id);
 		else
 			return SNMP_IPADDRESS(ospf_empty_addr);
-		break;
 	case OSPFIFTYPE: /* 4 */
 		return SNMP_INTEGER(ospf_snmp_iftype(oi->ifp));
-		break;
 	case OSPFIFADMINSTAT: /* 5 */
 		if (oi)
 			return SNMP_INTEGER(OSPF_STATUS_ENABLED);
 		else
 			return SNMP_INTEGER(OSPF_STATUS_DISABLED);
-		break;
 	case OSPFIFRTRPRIORITY: /* 6 */
 		return SNMP_INTEGER(PRIORITY(oi));
-		break;
 	case OSPFIFTRANSITDELAY: /* 7 */
 		return SNMP_INTEGER(OSPF_IF_PARAM(oi, transmit_delay));
-		break;
 	case OSPFIFRETRANSINTERVAL: /* 8 */
 		return SNMP_INTEGER(OSPF_IF_PARAM(oi, retransmit_interval));
-		break;
 	case OSPFIFHELLOINTERVAL: /* 9 */
 		return SNMP_INTEGER(OSPF_IF_PARAM(oi, v_hello));
-		break;
 	case OSPFIFRTRDEADINTERVAL: /* 10 */
 		return SNMP_INTEGER(OSPF_IF_PARAM(oi, v_wait));
-		break;
 	case OSPFIFPOLLINTERVAL: /* 11 */
 		return SNMP_INTEGER(OSPF_POLL_INTERVAL_DEFAULT);
-		break;
 	case OSPFIFSTATE: /* 12 */
 		return SNMP_INTEGER(ISM_SNMP(oi->state));
-		break;
 	case OSPFIFDESIGNATEDROUTER: /* 13 */
 		return SNMP_IPADDRESS(DR(oi));
-		break;
 	case OSPFIFBACKUPDESIGNATEDROUTER: /* 14 */
 		return SNMP_IPADDRESS(BDR(oi));
-		break;
 	case OSPFIFEVENTS: /* 15 */
 		return SNMP_INTEGER(oi->state_change);
-		break;
 	case OSPFIFAUTHKEY: /* 16 */
 		*var_len = 0;
 		return (uint8_t *)OSPF_IF_PARAM(oi, auth_simple);
-		break;
 	case OSPFIFSTATUS: /* 17 */
 		return SNMP_INTEGER(SNMP_VALID);
-		break;
 	case OSPFIFMULTICASTFORWARDING: /* 18 */
 #define ospf_snmp_multiforward_blocked    1
 #define ospf_snmp_multiforward_multicast  2
 #define ospf_snmp_multiforward_unicast    3
 		return SNMP_INTEGER(ospf_snmp_multiforward_blocked);
-		break;
 	case OSPFIFDEMAND: /* 19 */
 		return SNMP_INTEGER(SNMP_FALSE);
-		break;
 	case OSPFIFAUTHTYPE: /* 20 */
 		if (oi->area)
 			return SNMP_INTEGER(oi->area->auth_type);
 		else
 			return SNMP_INTEGER(0);
-		break;
 	default:
 		return NULL;
-		break;
 	}
 	return NULL;
 }
@@ -1852,22 +1778,16 @@ static uint8_t *ospfIfMetricEntry(struct variable *v, oid *name, size_t *length,
 	switch (v->magic) {
 	case OSPFIFMETRICIPADDRESS:
 		return SNMP_IPADDRESS(ifaddr);
-		break;
 	case OSPFIFMETRICADDRESSLESSIF:
 		return SNMP_INTEGER(ifindex);
-		break;
 	case OSPFIFMETRICTOS:
 		return SNMP_INTEGER(0);
-		break;
 	case OSPFIFMETRICVALUE:
 		return SNMP_INTEGER(OSPF_SNMP_METRIC_VALUE);
-		break;
 	case OSPFIFMETRICSTATUS:
 		return SNMP_INTEGER(1);
-		break;
 	default:
 		return NULL;
-		break;
 	}
 	return NULL;
 }
@@ -2042,44 +1962,32 @@ static uint8_t *ospfVirtIfEntry(struct variable *v, oid *name, size_t *length,
 	switch (v->magic) {
 	case OSPFVIRTIFAREAID:
 		return SNMP_IPADDRESS(area_id);
-		break;
 	case OSPFVIRTIFNEIGHBOR:
 		return SNMP_IPADDRESS(neighbor);
-		break;
 	case OSPFVIRTIFTRANSITDELAY:
 		return SNMP_INTEGER(OSPF_IF_PARAM(oi, transmit_delay));
-		break;
 	case OSPFVIRTIFRETRANSINTERVAL:
 		return SNMP_INTEGER(OSPF_IF_PARAM(oi, retransmit_interval));
-		break;
 	case OSPFVIRTIFHELLOINTERVAL:
 		return SNMP_INTEGER(OSPF_IF_PARAM(oi, v_hello));
-		break;
 	case OSPFVIRTIFRTRDEADINTERVAL:
 		return SNMP_INTEGER(OSPF_IF_PARAM(oi, v_wait));
-		break;
 	case OSPFVIRTIFSTATE:
 		return SNMP_INTEGER(oi->state);
-		break;
 	case OSPFVIRTIFEVENTS:
 		return SNMP_INTEGER(oi->state_change);
-		break;
 	case OSPFVIRTIFAUTHKEY:
 		*var_len = 0;
 		return (uint8_t *)OSPF_IF_PARAM(oi, auth_simple);
-		break;
 	case OSPFVIRTIFSTATUS:
 		return SNMP_INTEGER(SNMP_VALID);
-		break;
 	case OSPFVIRTIFAUTHTYPE:
 		if (oi->area)
 			return SNMP_INTEGER(oi->area->auth_type);
 		else
 			return SNMP_INTEGER(0);
-		break;
 	default:
 		return NULL;
-		break;
 	}
 	return NULL;
 }
@@ -2258,47 +2166,33 @@ static uint8_t *ospfNbrEntry(struct variable *v, oid *name, size_t *length,
 	if (!nbr)
 		return NULL;
 	oi = nbr->oi;
-	if (!oi)
-		return NULL;
 
 	/* Return the current value of the variable */
 	switch (v->magic) {
 	case OSPFNBRIPADDR:
 		return SNMP_IPADDRESS(nbr_addr);
-		break;
 	case OSPFNBRADDRESSLESSINDEX:
 		return SNMP_INTEGER(ifindex);
-		break;
 	case OSPFNBRRTRID:
 		return SNMP_IPADDRESS(nbr->router_id);
-		break;
 	case OSPFNBROPTIONS:
 		return SNMP_INTEGER(oi->nbr_self->options);
-		break;
 	case OSPFNBRPRIORITY:
 		return SNMP_INTEGER(nbr->priority);
-		break;
 	case OSPFNBRSTATE:
 		return SNMP_INTEGER(ospf_snmp_neighbor_state(nbr->state));
-		break;
 	case OSPFNBREVENTS:
 		return SNMP_INTEGER(nbr->state_change);
-		break;
 	case OSPFNBRLSRETRANSQLEN:
 		return SNMP_INTEGER(ospf_ls_retransmit_count(nbr));
-		break;
 	case OSPFNBMANBRSTATUS:
 		return SNMP_INTEGER(SNMP_VALID);
-		break;
 	case OSPFNBMANBRPERMANENCE:
 		return SNMP_INTEGER(2);
-		break;
 	case OSPFNBRHELLOSUPPRESSED:
 		return SNMP_INTEGER(SNMP_FALSE);
-		break;
 	default:
 		return NULL;
-		break;
 	}
 	return NULL;
 }
@@ -2332,31 +2226,22 @@ static uint8_t *ospfVirtNbrEntry(struct variable *v, oid *name, size_t *length,
 	switch (v->magic) {
 	case OSPFVIRTNBRAREA:
 		return (uint8_t *)NULL;
-		break;
 	case OSPFVIRTNBRRTRID:
 		return (uint8_t *)NULL;
-		break;
 	case OSPFVIRTNBRIPADDR:
 		return (uint8_t *)NULL;
-		break;
 	case OSPFVIRTNBROPTIONS:
 		return (uint8_t *)NULL;
-		break;
 	case OSPFVIRTNBRSTATE:
 		return (uint8_t *)NULL;
-		break;
 	case OSPFVIRTNBREVENTS:
 		return (uint8_t *)NULL;
-		break;
 	case OSPFVIRTNBRLSRETRANSQLEN:
 		return (uint8_t *)NULL;
-		break;
 	case OSPFVIRTNBRHELLOSUPPRESSED:
 		return (uint8_t *)NULL;
-		break;
 	default:
 		return NULL;
-		break;
 	}
 	return NULL;
 }
@@ -2486,29 +2371,21 @@ static uint8_t *ospfExtLsdbEntry(struct variable *v, oid *name, size_t *length,
 	switch (v->magic) {
 	case OSPFEXTLSDBTYPE:
 		return SNMP_INTEGER(OSPF_AS_EXTERNAL_LSA);
-		break;
 	case OSPFEXTLSDBLSID:
 		return SNMP_IPADDRESS(lsah->id);
-		break;
 	case OSPFEXTLSDBROUTERID:
 		return SNMP_IPADDRESS(lsah->adv_router);
-		break;
 	case OSPFEXTLSDBSEQUENCE:
 		return SNMP_INTEGER(lsah->ls_seqnum);
-		break;
 	case OSPFEXTLSDBAGE:
 		return SNMP_INTEGER(lsah->ls_age);
-		break;
 	case OSPFEXTLSDBCHECKSUM:
 		return SNMP_INTEGER(lsah->checksum);
-		break;
 	case OSPFEXTLSDBADVERTISEMENT:
 		*var_len = ntohs(lsah->length);
 		return (uint8_t *)lsah;
-		break;
 	default:
 		return NULL;
-		break;
 	}
 	return NULL;
 }
@@ -2526,25 +2403,18 @@ static uint8_t *ospfAreaAggregateEntry(struct variable *v, oid *name,
 	switch (v->magic) {
 	case OSPFAREAAGGREGATEAREAID:
 		return (uint8_t *)NULL;
-		break;
 	case OSPFAREAAGGREGATELSDBTYPE:
 		return (uint8_t *)NULL;
-		break;
 	case OSPFAREAAGGREGATENET:
 		return (uint8_t *)NULL;
-		break;
 	case OSPFAREAAGGREGATEMASK:
 		return (uint8_t *)NULL;
-		break;
 	case OSPFAREAAGGREGATESTATUS:
 		return (uint8_t *)NULL;
-		break;
 	case OSPFAREAAGGREGATEEFFECT:
 		return (uint8_t *)NULL;
-		break;
 	default:
 		return NULL;
-		break;
 	}
 	return NULL;
 }
@@ -2585,19 +2455,16 @@ static void ospfTrapNbrStateChange(struct ospf_neighbor *on)
 
 	ospf_nbr_state_message(on, msgbuf, sizeof(msgbuf));
 	if (IS_DEBUG_OSPF_EVENT)
-		zlog_info("%s: trap sent: %s now %s", __PRETTY_FUNCTION__,
+		zlog_info("%s: trap sent: %s now %s", __func__,
 			  inet_ntoa(on->address.u.prefix4), msgbuf);
 
 	oid_copy_addr(index, &(on->address.u.prefix4), IN_ADDR_SIZE);
 	index[IN_ADDR_SIZE] = 0;
 
-	smux_trap(ospf_variables,
-		  sizeof ospf_variables / sizeof(struct variable),
-		  ospf_trap_oid, sizeof ospf_trap_oid / sizeof(oid), ospf_oid,
-		  sizeof ospf_oid / sizeof(oid), index, IN_ADDR_SIZE + 1,
-		  ospfNbrTrapList,
-		  sizeof ospfNbrTrapList / sizeof(struct trap_object),
-		  NBRSTATECHANGE);
+	smux_trap(ospf_variables, array_size(ospf_variables), ospf_trap_oid,
+		  array_size(ospf_trap_oid), ospf_oid,
+		  sizeof(ospf_oid) / sizeof(oid), index, IN_ADDR_SIZE + 1,
+		  ospfNbrTrapList, array_size(ospfNbrTrapList), NBRSTATECHANGE);
 }
 
 static void ospfTrapVirtNbrStateChange(struct ospf_neighbor *on)
@@ -2609,12 +2476,10 @@ static void ospfTrapVirtNbrStateChange(struct ospf_neighbor *on)
 	oid_copy_addr(index, &(on->address.u.prefix4), IN_ADDR_SIZE);
 	index[IN_ADDR_SIZE] = 0;
 
-	smux_trap(ospf_variables,
-		  sizeof ospf_variables / sizeof(struct variable),
-		  ospf_trap_oid, sizeof ospf_trap_oid / sizeof(oid), ospf_oid,
-		  sizeof ospf_oid / sizeof(oid), index, IN_ADDR_SIZE + 1,
-		  ospfVirtNbrTrapList,
-		  sizeof ospfVirtNbrTrapList / sizeof(struct trap_object),
+	smux_trap(ospf_variables, array_size(ospf_variables), ospf_trap_oid,
+		  array_size(ospf_trap_oid), ospf_oid,
+		  sizeof(ospf_oid) / sizeof(oid), index, IN_ADDR_SIZE + 1,
+		  ospfVirtNbrTrapList, array_size(ospfVirtNbrTrapList),
 		  VIRTNBRSTATECHANGE);
 }
 
@@ -2643,20 +2508,17 @@ static void ospfTrapIfStateChange(struct ospf_interface *oi)
 	oid index[sizeof(oid) * (IN_ADDR_SIZE + 1)];
 
 	if (IS_DEBUG_OSPF_EVENT)
-		zlog_info("%s: trap sent: %s now %s", __PRETTY_FUNCTION__,
+		zlog_info("%s: trap sent: %s now %s", __func__,
 			  inet_ntoa(oi->address->u.prefix4),
 			  lookup_msg(ospf_ism_state_msg, oi->state, NULL));
 
 	oid_copy_addr(index, &(oi->address->u.prefix4), IN_ADDR_SIZE);
 	index[IN_ADDR_SIZE] = 0;
 
-	smux_trap(ospf_variables,
-		  sizeof ospf_variables / sizeof(struct variable),
-		  ospf_trap_oid, sizeof ospf_trap_oid / sizeof(oid), ospf_oid,
-		  sizeof ospf_oid / sizeof(oid), index, IN_ADDR_SIZE + 1,
-		  ospfIfTrapList,
-		  sizeof ospfIfTrapList / sizeof(struct trap_object),
-		  IFSTATECHANGE);
+	smux_trap(ospf_variables, array_size(ospf_variables), ospf_trap_oid,
+		  array_size(ospf_trap_oid), ospf_oid,
+		  sizeof(ospf_oid) / sizeof(oid), index, IN_ADDR_SIZE + 1,
+		  ospfIfTrapList, array_size(ospfIfTrapList), IFSTATECHANGE);
 }
 
 static void ospfTrapVirtIfStateChange(struct ospf_interface *oi)
@@ -2668,12 +2530,10 @@ static void ospfTrapVirtIfStateChange(struct ospf_interface *oi)
 	oid_copy_addr(index, &(oi->address->u.prefix4), IN_ADDR_SIZE);
 	index[IN_ADDR_SIZE] = 0;
 
-	smux_trap(ospf_variables,
-		  sizeof ospf_variables / sizeof(struct variable),
-		  ospf_trap_oid, sizeof ospf_trap_oid / sizeof(oid), ospf_oid,
-		  sizeof ospf_oid / sizeof(oid), index, IN_ADDR_SIZE + 1,
-		  ospfVirtIfTrapList,
-		  sizeof ospfVirtIfTrapList / sizeof(struct trap_object),
+	smux_trap(ospf_variables, array_size(ospf_variables), ospf_trap_oid,
+		  array_size(ospf_trap_oid), ospf_oid,
+		  sizeof(ospf_oid) / sizeof(oid), index, IN_ADDR_SIZE + 1,
+		  ospfVirtIfTrapList, array_size(ospfVirtIfTrapList),
 		  VIRTIFSTATECHANGE);
 }
 
