@@ -61,6 +61,7 @@
 #include "ospfd/ospf_zebra.h"
 #include "ospfd/ospf_te.h"
 #include "ospfd/ospf_vty.h"
+#include "ospfd/ospf_errors.h"
 
 /*
  * Global variable to manage Opaque-LSA/MPLS-TE on this node.
@@ -68,7 +69,7 @@
  */
 struct ospf_mpls_te OspfMplsTE;
 
-const char *mode2text[] = {"Off", "AS", "Area"};
+static const char *const mode2text[] = {"Off", "AS", "Area"};
 
 /*------------------------------------------------------------------------*
  * Followings are initialize/terminate functions for MPLS-TE handling.
@@ -102,7 +103,8 @@ int ospf_mpls_te_init(void)
 		ospf_mpls_te_lsa_refresh, NULL, /* ospf_mpls_te_new_lsa_hook */
 		NULL /* ospf_mpls_te_del_lsa_hook */);
 	if (rc != 0) {
-		zlog_warn(
+		flog_warn(
+			EC_OSPF_OPAQUE_REGISTRATION,
 			"ospf_mpls_te_init: Failed to register Traffic Engineering functions");
 		return rc;
 	}
@@ -139,7 +141,8 @@ static int ospf_mpls_te_register(enum inter_as_mode mode)
 					  ospf_mpls_te_lsa_refresh, NULL, NULL);
 
 	if (rc != 0) {
-		zlog_warn(
+		flog_warn(
+			EC_OSPF_OPAQUE_REGISTRATION,
 			"ospf_router_info_init: Failed to register Inter-AS functions");
 		return rc;
 	}
@@ -147,7 +150,7 @@ static int ospf_mpls_te_register(enum inter_as_mode mode)
 	return rc;
 }
 
-static int ospf_mpls_te_unregister()
+static int ospf_mpls_te_unregister(void)
 {
 	uint8_t scope;
 
@@ -166,7 +169,7 @@ static int ospf_mpls_te_unregister()
 
 void ospf_mpls_te_term(void)
 {
-	list_delete_and_null(&OspfMplsTE.iflist);
+	list_delete(&OspfMplsTE.iflist);
 
 	ospf_delete_opaque_functab(OSPF_OPAQUE_AREA_LSA,
 				   OPAQUE_TYPE_TRAFFIC_ENGINEERING_LSA);
@@ -184,6 +187,7 @@ void ospf_mpls_te_finish(void)
 	// list_delete_all_node(OspfMplsTE.iflist);
 
 	OspfMplsTE.enabled = false;
+	ospf_mpls_te_unregister();
 	OspfMplsTE.inter_as = Off;
 }
 
@@ -231,7 +235,7 @@ static struct mpls_te_link *lookup_linkparams_by_instance(struct ospf_lsa *lsa)
 		if (lp->instance == key)
 			return lp;
 
-	zlog_warn("lookup_linkparams_by_instance: Entry not found: key(%x)",
+	zlog_info("lookup_linkparams_by_instance: Entry not found: key(%x)",
 		  key);
 	return NULL;
 }
@@ -394,53 +398,13 @@ static void set_linkparams_link_type(struct ospf_interface *oi,
 	return;
 }
 
-static void set_linkparams_link_id(struct ospf_interface *oi,
-				   struct mpls_te_link *lp)
+static void set_linkparams_link_id(struct mpls_te_link *lp,
+				   struct in_addr link_id)
 {
-	struct ospf_neighbor *nbr;
-	int done = 0;
 
 	lp->link_id.header.type = htons(TE_LINK_SUBTLV_LINK_ID);
 	lp->link_id.header.length = htons(TE_LINK_SUBTLV_DEF_SIZE);
-
-	/*
-	 * The Link ID is identical to the contents of the Link ID field
-	 * in the Router LSA for these link types.
-	 */
-	switch (oi->type) {
-	case OSPF_IFTYPE_POINTOPOINT:
-		/* Take the router ID of the neighbor. */
-		if ((nbr = ospf_nbr_lookup_ptop(oi))
-		    && nbr->state == NSM_Full) {
-			lp->link_id.value = nbr->router_id;
-			done = 1;
-		}
-		break;
-	case OSPF_IFTYPE_BROADCAST:
-	case OSPF_IFTYPE_NBMA:
-		/* Take the interface address of the designated router. */
-		if ((nbr = ospf_nbr_lookup_by_addr(oi->nbrs, &DR(oi))) == NULL)
-			break;
-
-		if (nbr->state == NSM_Full
-		    || (IPV4_ADDR_SAME(&oi->address->u.prefix4, &DR(oi))
-			&& ospf_nbr_count(oi, NSM_Full) > 0)) {
-			lp->link_id.value = DR(oi);
-			done = 1;
-		}
-		break;
-	default:
-		/* Not supported yet. */ /* XXX */
-		lp->link_id.header.type = htons(0);
-		break;
-	}
-
-	if (!done) {
-		struct in_addr mask;
-		masklen2ip(oi->address->prefixlen, &mask);
-		lp->link_id.value.s_addr =
-			oi->address->u.prefix4.s_addr & mask.s_addr;
-	}
+	lp->link_id.value = link_id;
 	return;
 }
 
@@ -783,7 +747,7 @@ static void initialize_linkparams(struct mpls_te_link *lp)
 
 	if ((oi == NULL) || (oi->ifp != ifp)) {
 		if (IS_DEBUG_OSPF_TE)
-			zlog_warn(
+			zlog_debug(
 				"MPLS-TE(initialize_linkparams) Could not find corresponding OSPF Interface for %s",
 				ifp->name);
 		return;
@@ -816,18 +780,21 @@ static int is_mandated_params_set(struct mpls_te_link *lp)
 	int rc = 0;
 
 	if (ntohs(OspfMplsTE.router_addr.header.type) == 0) {
-		zlog_warn(
+		flog_warn(
+			EC_OSPF_TE_UNEXPECTED,
 			"MPLS-TE(is_mandated_params_set) Missing Router Address");
 		return rc;
 	}
 
 	if (ntohs(lp->link_type.header.type) == 0) {
-		zlog_warn("MPLS-TE(is_mandated_params_set) Missing Link Type");
+		flog_warn(EC_OSPF_TE_UNEXPECTED,
+			  "MPLS-TE(is_mandated_params_set) Missing Link Type");
 		return rc;
 	}
 
 	if (!IS_INTER_AS(lp->type) && (ntohs(lp->link_id.header.type) == 0)) {
-		zlog_warn("MPLS-TE(is_mandated_params_set) Missing Link ID");
+		flog_warn(EC_OSPF_TE_UNEXPECTED,
+			  "MPLS-TE(is_mandated_params_set) Missing Link ID");
 		return rc;
 	}
 
@@ -842,19 +809,14 @@ static int is_mandated_params_set(struct mpls_te_link *lp)
 static int ospf_mpls_te_new_if(struct interface *ifp)
 {
 	struct mpls_te_link *new;
-	int rc = -1;
 
 	if (IS_DEBUG_OSPF_TE)
 		zlog_debug(
 			"MPLS-TE(ospf_mpls_te_new_if) Add new %s interface %s to MPLS-TE list",
 			ifp->link_params ? "Active" : "Inactive", ifp->name);
 
-	if (lookup_linkparams_by_ifp(ifp) != NULL) {
-		zlog_warn("ospf_mpls_te_new_if: ifp(%p) already in use?",
-			  (void *)ifp);
-		rc = 0; /* Do nothing here. */
-		return rc;
-	}
+	if (lookup_linkparams_by_ifp(ifp) != NULL)
+		return 0;
 
 	new = XCALLOC(MTYPE_OSPF_MPLS_TE, sizeof(struct mpls_te_link));
 
@@ -882,9 +844,7 @@ static int ospf_mpls_te_new_if(struct interface *ifp)
 			ifp->name, new->flags, new->type);
 
 	/* Schedule Opaque-LSA refresh. */ /* XXX */
-
-	rc = 0;
-	return rc;
+	return 0;
 }
 
 static int ospf_mpls_te_del_if(struct interface *ifp)
@@ -898,10 +858,6 @@ static int ospf_mpls_te_del_if(struct interface *ifp)
 		/* Dequeue listnode entry from the list. */
 		listnode_delete(iflist, lp);
 
-		/* Avoid misjudgement in the next lookup. */
-		if (listcount(iflist) == 0)
-			iflist->head = iflist->tail = NULL;
-
 		XFREE(MTYPE_OSPF_MPLS_TE, lp);
 	}
 
@@ -925,7 +881,8 @@ void ospf_mpls_te_update_if(struct interface *ifp)
 
 	/* Get Link context from interface */
 	if ((lp = lookup_linkparams_by_ifp(ifp)) == NULL) {
-		zlog_warn(
+		flog_warn(
+			EC_OSPF_TE_UNEXPECTED,
 			"OSPF MPLS-TE Update: Did not find Link Parameters context for interface %s",
 			ifp->name);
 		return;
@@ -962,37 +919,33 @@ void ospf_mpls_te_update_if(struct interface *ifp)
 	return;
 }
 
+/*
+ * Just add interface and set available information. Other information
+ * and flooding of LSA will be done later when adjacency will be up
+ * See ospf_mpls_te_nsm_change() after
+ */
 static void ospf_mpls_te_ism_change(struct ospf_interface *oi, int old_state)
 {
-	struct te_link_subtlv_link_type old_type;
-	struct te_link_subtlv_link_id old_id;
+
 	struct mpls_te_link *lp;
 
-	if ((lp = lookup_linkparams_by_ifp(oi->ifp)) == NULL) {
-		zlog_warn(
-			"ospf_mpls_te_ism_change: Cannot get linkparams from OI(%s)?",
-			IF_NAME(oi));
+	lp = lookup_linkparams_by_ifp(oi->ifp);
+	if (lp == NULL) {
+		flog_warn(
+			EC_OSPF_TE_UNEXPECTED,
+			"MPLS-TE (%s): Cannot get linkparams from OI(%s)?",
+			__func__, IF_NAME(oi));
 		return;
 	}
 
 	if (oi->area == NULL || oi->area->ospf == NULL) {
-		zlog_warn(
-			"ospf_mpls_te_ism_change: Cannot refer to OSPF from OI(%s)?",
-			IF_NAME(oi));
+		flog_warn(
+			EC_OSPF_TE_UNEXPECTED,
+			"MPLS-TE (%s): Cannot refer to OSPF from OI(%s)?",
+			__func__, IF_NAME(oi));
 		return;
 	}
-#ifdef notyet
-	if ((lp->area != NULL
-	     && !IPV4_ADDR_SAME(&lp->area->area_id, &oi->area->area_id))
-	    || (lp->area != NULL && oi->area == NULL)) {
-		/* How should we consider this case? */
-		zlog_warn(
-			"MPLS-TE: Area for OI(%s) has changed to [%s], flush previous LSAs",
-			IF_NAME(oi),
-			oi->area ? inet_ntoa(oi->area->area_id) : "N/A");
-		ospf_mpls_te_lsa_schedule(lp, FLUSH_THIS_LSA);
-	}
-#endif
+
 	/* Keep Area information in combination with linkparams. */
 	lp->area = oi->area;
 
@@ -1004,55 +957,103 @@ static void ospf_mpls_te_ism_change(struct ospf_interface *oi, int old_state)
 	case ISM_DROther:
 	case ISM_Backup:
 	case ISM_DR:
-		old_type = lp->link_type;
-		old_id = lp->link_id;
-
-		/* Set Link type, Link ID, Local and Remote IP addr */
+		/* Set Link type and Local IP addr */
 		set_linkparams_link_type(oi, lp);
-		set_linkparams_link_id(oi, lp);
 		set_linkparams_lclif_ipaddr(lp, oi->address->u.prefix4);
 
-		if (oi->type == LINK_TYPE_SUBTLV_VALUE_PTP) {
-			struct prefix *pref = CONNECTED_PREFIX(oi->connected);
-			if (pref != NULL)
-				set_linkparams_rmtif_ipaddr(lp,
-							    pref->u.prefix4);
-		}
-
-		/* Update TE parameters */
-		update_linkparams(lp);
-
-		/* Try to Schedule LSA */
-		if ((ntohs(old_type.header.type)
-			     != ntohs(lp->link_type.header.type)
-		     || old_type.link_type.value
-				!= lp->link_type.link_type.value)
-		    || (ntohs(old_id.header.type)
-				!= ntohs(lp->link_id.header.type)
-			|| ntohl(old_id.value.s_addr)
-				   != ntohl(lp->link_id.value.s_addr))) {
-			if (CHECK_FLAG(lp->flags, LPFLG_LSA_ENGAGED))
-				ospf_mpls_te_lsa_schedule(lp, REFRESH_THIS_LSA);
-			else
-				ospf_mpls_te_lsa_schedule(lp,
-							  REORIGINATE_THIS_LSA);
-		}
 		break;
 	default:
-		lp->link_type.header.type = htons(0);
-		lp->link_id.header.type = htons(0);
-
+		/* State is undefined: Flush LSA if engaged */
 		if (CHECK_FLAG(lp->flags, LPFLG_LSA_ENGAGED))
 			ospf_mpls_te_lsa_schedule(lp, FLUSH_THIS_LSA);
 		break;
 	}
 
+	if (IS_DEBUG_OSPF_TE)
+		zlog_debug(
+			"MPLS-TE(%s): Update Link parameters for interface %s",
+			__func__, IF_NAME(oi));
+
 	return;
 }
 
+/*
+ * Complete TE info and schedule LSA flooding
+ * Link-ID and Remote IP address must be set with neighbor info
+ * which are only valid once NSM state is FULL
+ */
 static void ospf_mpls_te_nsm_change(struct ospf_neighbor *nbr, int old_state)
 {
-	/* Nothing to do here */
+	struct ospf_interface *oi = nbr->oi;
+	struct mpls_te_link *lp;
+
+	/* Process Neighbor only when its state is NSM Full */
+	if (nbr->state != NSM_Full)
+		return;
+
+	/* Get interface information for Traffic Engineering */
+	lp = lookup_linkparams_by_ifp(oi->ifp);
+	if (lp == NULL) {
+		flog_warn(
+			EC_OSPF_TE_UNEXPECTED,
+			"MPLS-TE (%s): Cannot get linkparams from OI(%s)?",
+			__func__, IF_NAME(oi));
+		return;
+	}
+
+	if (oi->area == NULL || oi->area->ospf == NULL) {
+		flog_warn(
+			EC_OSPF_TE_UNEXPECTED,
+			"MPLS-TE (%s): Cannot refer to OSPF from OI(%s)?",
+			__func__, IF_NAME(oi));
+		return;
+	}
+
+	/* Keep Area information in combination with SR info. */
+	lp->area = oi->area;
+
+	/* Keep interface MPLS-TE status */
+	lp->flags = HAS_LINK_PARAMS(oi->ifp);
+
+	/*
+	 * The Link ID is identical to the contents of the Link ID field
+	 * in the Router LSA for these link types.
+	 */
+	switch (oi->state) {
+	case ISM_PointToPoint:
+		/* Set Link ID with neighbor Router ID */
+		set_linkparams_link_id(lp, nbr->router_id);
+		/* Set Remote IP address */
+		set_linkparams_rmtif_ipaddr(lp, nbr->address.u.prefix4);
+		break;
+
+	case ISM_DR:
+	case ISM_DROther:
+	case ISM_Backup:
+		/* Set Link ID with the Designated Router ID */
+		set_linkparams_link_id(lp, DR(oi));
+		break;
+
+	default:
+		/* State is undefined: Flush LSA if engaged */
+		if (OspfMplsTE.enabled &&
+			CHECK_FLAG(lp->flags, LPFLG_LSA_ENGAGED))
+			ospf_mpls_te_lsa_schedule(lp, FLUSH_THIS_LSA);
+		return;
+	}
+
+	if (IS_DEBUG_OSPF_TE)
+		zlog_debug(
+			"MPLS-TE (%s): Add Link-ID %s for interface %s ",
+			__func__, inet_ntoa(lp->link_id.value), oi->ifp->name);
+
+	/* Try to Schedule LSA */
+	if (OspfMplsTE.enabled) {
+		if (CHECK_FLAG(lp->flags, LPFLG_LSA_ENGAGED))
+			ospf_mpls_te_lsa_schedule(lp, REFRESH_THIS_LSA);
+		else
+			ospf_mpls_te_lsa_schedule(lp, REORIGINATE_THIS_LSA);
+	}
 	return;
 }
 
@@ -1146,10 +1147,7 @@ static struct ospf_lsa *ospf_mpls_te_lsa_new(struct ospf *ospf,
 	uint16_t length;
 
 	/* Create a stream for LSA. */
-	if ((s = stream_new(OSPF_MAX_LSA_SIZE)) == NULL) {
-		zlog_warn("ospf_mpls_te_lsa_new: stream_new() ?");
-		return NULL;
-	}
+	s = stream_new(OSPF_MAX_LSA_SIZE);
 	lsah = (struct lsa_header *)STREAM_DATA(s);
 
 	options = OSPF_OPTION_O; /* Don't forget this :-) */
@@ -1223,14 +1221,16 @@ static int ospf_mpls_te_lsa_originate1(struct ospf_area *area,
 	/* Create new Opaque-LSA/MPLS-TE instance. */
 	new = ospf_mpls_te_lsa_new(area->ospf, area, lp);
 	if (new == NULL) {
-		zlog_warn(
+		flog_warn(
+			EC_OSPF_TE_UNEXPECTED,
 			"ospf_mpls_te_lsa_originate1: ospf_mpls_te_lsa_new() ?");
 		return rc;
 	}
 
 	/* Install this LSA into LSDB. */
 	if (ospf_lsa_install(area->ospf, NULL /*oi*/, new) == NULL) {
-		zlog_warn("ospf_mpls_te_lsa_originate1: ospf_lsa_install() ?");
+		flog_warn(EC_OSPF_LSA_INSTALL_FAILURE,
+			  "ospf_mpls_te_lsa_originate1: ospf_lsa_install() ?");
 		ospf_lsa_unlock(&new);
 		return rc;
 	}
@@ -1286,7 +1286,7 @@ static int ospf_mpls_te_lsa_originate_area(void *arg)
 		if (CHECK_FLAG(lp->flags, LPFLG_LSA_ENGAGED)) {
 			if (CHECK_FLAG(lp->flags, LPFLG_LSA_FORCED_REFRESH)) {
 				UNSET_FLAG(lp->flags, LPFLG_LSA_FORCED_REFRESH);
-				zlog_warn(
+				zlog_info(
 					"OSPF MPLS-TE (ospf_mpls_te_lsa_originate_area): Refresh instead of Originate");
 				ospf_mpls_te_lsa_schedule(lp, REFRESH_THIS_LSA);
 			}
@@ -1294,7 +1294,7 @@ static int ospf_mpls_te_lsa_originate_area(void *arg)
 		}
 
 		if (!is_mandated_params_set(lp)) {
-			zlog_warn(
+			zlog_info(
 				"ospf_mpls_te_lsa_originate_area: Link(%s) lacks some mandated MPLS-TE parameters.",
 				lp->ifp ? lp->ifp->name : "?");
 			continue;
@@ -1323,7 +1323,8 @@ static int ospf_mpls_te_lsa_originate2(struct ospf *top,
 	/* Create new Opaque-LSA/Inter-AS instance. */
 	new = ospf_mpls_te_lsa_new(top, NULL, lp);
 	if (new == NULL) {
-		zlog_warn(
+		flog_warn(
+			EC_OSPF_LSA_UNEXPECTED,
 			"ospf_mpls_te_lsa_originate2: ospf_router_info_lsa_new() ?");
 		return rc;
 	}
@@ -1331,7 +1332,8 @@ static int ospf_mpls_te_lsa_originate2(struct ospf *top,
 
 	/* Install this LSA into LSDB. */
 	if (ospf_lsa_install(top, NULL /*oi */, new) == NULL) {
-		zlog_warn("ospf_mpls_te_lsa_originate2: ospf_lsa_install() ?");
+		flog_warn(EC_OSPF_LSA_INSTALL_FAILURE,
+			  "ospf_mpls_te_lsa_originate2: ospf_lsa_install() ?");
 		ospf_lsa_unlock(&new);
 		return rc;
 	}
@@ -1385,7 +1387,8 @@ static int ospf_mpls_te_lsa_originate_as(void *arg)
 		}
 
 		if (!is_mandated_params_set(lp)) {
-			zlog_warn(
+			flog_warn(
+				EC_OSPF_TE_UNEXPECTED,
 				"ospf_mpls_te_lsa_originate_as: Link(%s) lacks some mandated MPLS-TE parameters.",
 				lp->ifp ? lp->ifp->name : "?");
 			continue;
@@ -1432,7 +1435,8 @@ static struct ospf_lsa *ospf_mpls_te_lsa_refresh(struct ospf_lsa *lsa)
 
 	/* At first, resolve lsa/lp relationship. */
 	if ((lp = lookup_linkparams_by_instance(lsa)) == NULL) {
-		zlog_warn("ospf_mpls_te_lsa_refresh: Invalid parameter?");
+		flog_warn(EC_OSPF_TE_UNEXPECTED,
+			  "ospf_mpls_te_lsa_refresh: Invalid parameter?");
 		lsa->data->ls_age =
 			htons(OSPF_LSA_MAXAGE); /* Flush it anyway. */
 		ospf_opaque_lsa_flush_schedule(lsa);
@@ -1441,7 +1445,8 @@ static struct ospf_lsa *ospf_mpls_te_lsa_refresh(struct ospf_lsa *lsa)
 
 	/* Check if lp was not disable in the interval */
 	if (!CHECK_FLAG(lp->flags, LPFLG_LSA_ACTIVE)) {
-		zlog_warn(
+		flog_warn(
+			EC_OSPF_TE_UNEXPECTED,
 			"ospf_mpls_te_lsa_refresh: lp was disabled: Flush it!");
 		lsa->data->ls_age =
 			htons(OSPF_LSA_MAXAGE); /* Flush it anyway. */
@@ -1457,7 +1462,8 @@ static struct ospf_lsa *ospf_mpls_te_lsa_refresh(struct ospf_lsa *lsa)
 	/* Create new Opaque-LSA/MPLS-TE instance. */
 	new = ospf_mpls_te_lsa_new(top, area, lp);
 	if (new == NULL) {
-		zlog_warn("ospf_mpls_te_lsa_refresh: ospf_mpls_te_lsa_new() ?");
+		flog_warn(EC_OSPF_TE_UNEXPECTED,
+			  "ospf_mpls_te_lsa_refresh: ospf_mpls_te_lsa_new() ?");
 		return NULL;
 	}
 	new->data->ls_seqnum = lsa_seqnum_increment(lsa);
@@ -1470,7 +1476,8 @@ static struct ospf_lsa *ospf_mpls_te_lsa_refresh(struct ospf_lsa *lsa)
 		top = area->ospf;
 
 	if (ospf_lsa_install(top, NULL /*oi */, new) == NULL) {
-		zlog_warn("ospf_mpls_te_lsa_refresh: ospf_lsa_install() ?");
+		flog_warn(EC_OSPF_LSA_INSTALL_FAILURE,
+			  "ospf_mpls_te_lsa_refresh: ospf_lsa_install() ?");
 		ospf_lsa_unlock(&new);
 		return NULL;
 	}
@@ -1524,7 +1531,8 @@ void ospf_mpls_te_lsa_schedule(struct mpls_te_link *lp, enum lsa_opcode opcode)
 					top, OspfMplsTE.interas_areaid);
 			/* Unable to set the area context. Abort! */
 			if (lp->area == NULL) {
-				zlog_warn(
+				flog_warn(
+					EC_OSPF_TE_UNEXPECTED,
 					"MPLS-TE(ospf_mpls_te_lsa_schedule) Area context is null. Abort !");
 				return;
 			}
@@ -1567,7 +1575,8 @@ void ospf_mpls_te_lsa_schedule(struct mpls_te_link *lp, enum lsa_opcode opcode)
 		ospf_opaque_lsa_flush_schedule(&lsa);
 		break;
 	default:
-		zlog_warn("ospf_mpls_te_lsa_schedule: Unknown opcode (%u)",
+		flog_warn(EC_OSPF_TE_UNEXPECTED,
+			  "ospf_mpls_te_lsa_schedule: Unknown opcode (%u)",
 			  opcode);
 		break;
 	}
@@ -1772,7 +1781,7 @@ static uint16_t show_vty_link_subtlv_unrsv_bw(struct vty *vty,
 				i, fval1, i + 1, fval2);
 		else
 			zlog_debug(
-				"      [%d]: %g (Bytes/sec),\t[%d]: %g (Bytes/sec)",
+				"      [%d]: %g (Bytes/sec),  [%d]: %g (Bytes/sec)",
 				i, fval1, i + 1, fval2);
 	}
 
@@ -2111,7 +2120,7 @@ static uint16_t ospf_mpls_te_show_link_subtlv(struct vty *vty,
 
 static void ospf_mpls_te_show_info(struct vty *vty, struct ospf_lsa *lsa)
 {
-	struct lsa_header *lsah = (struct lsa_header *)lsa->data;
+	struct lsa_header *lsah = lsa->data;
 	struct tlv_header *tlvh, *next;
 	uint16_t sum, total;
 	uint16_t (*subfunc)(struct vty * vty, struct tlv_header * tlvh,
@@ -2227,6 +2236,17 @@ DEFUN (no_ospf_mpls_te,
 		if (CHECK_FLAG(lp->flags, LPFLG_LSA_ENGAGED))
 			ospf_mpls_te_lsa_schedule(lp, FLUSH_THIS_LSA);
 
+	/*
+	 * This resets the OspfMplsTE.inter_as to its initial state.
+	 * This is to avoid having an inter-as value different from
+	 * Off when mpls-te gets restarted (after being removed)
+	 */
+	if (OspfMplsTE.inter_as != Off) {
+		/* Deregister the Callbacks for Inter-AS support */
+		ospf_mpls_te_unregister();
+		OspfMplsTE.inter_as = Off;
+	}
+
 	return CMD_SUCCESS;
 }
 
@@ -2391,16 +2411,16 @@ DEFUN (no_ospf_mpls_te_inter_as,
 		zlog_debug("MPLS-TE: Inter-AS support OFF");
 
 	if ((OspfMplsTE.enabled) && (OspfMplsTE.inter_as != Off)) {
-		OspfMplsTE.inter_as = Off;
 		/* Flush all Inter-AS LSA */
 		for (ALL_LIST_ELEMENTS(OspfMplsTE.iflist, node, nnode, lp))
 			if (IS_INTER_AS(lp->type)
 			    && CHECK_FLAG(lp->flags, LPFLG_LSA_ENGAGED))
 				ospf_mpls_te_lsa_schedule(lp, FLUSH_THIS_LSA);
-	}
 
-	/* Deregister the Callbacks for Inter-AS support */
-	ospf_mpls_te_unregister();
+		/* Deregister the Callbacks for Inter-AS support */
+		ospf_mpls_te_unregister();
+		OspfMplsTE.inter_as = Off;
+	}
 
 	return CMD_SUCCESS;
 }
@@ -2523,11 +2543,11 @@ DEFUN (show_ip_ospf_mpls_te_link,
        "Interface name\n")
 {
 	struct vrf *vrf;
-	int idx_interface = 5;
-	struct interface *ifp;
+	int idx_interface = 0;
+	struct interface *ifp = NULL;
 	struct listnode *node;
 	char *vrf_name = NULL;
-	bool all_vrf;
+	bool all_vrf = false;
 	int inst = 0;
 	int idx_vrf = 0;
 	struct ospf *ospf = NULL;
@@ -2536,7 +2556,7 @@ DEFUN (show_ip_ospf_mpls_te_link,
 		vrf_name = argv[idx_vrf + 1]->arg;
 		all_vrf = strmatch(vrf_name, "all");
 	}
-
+	argv_find(argv, argc, "INTERFACE", &idx_interface);
 	/* vrf input is provided could be all or specific vrf*/
 	if (vrf_name) {
 		if (all_vrf) {
@@ -2550,32 +2570,31 @@ DEFUN (show_ip_ospf_mpls_te_link,
 			return CMD_SUCCESS;
 		}
 		ospf = ospf_lookup_by_inst_name(inst, vrf_name);
-		if (ospf == NULL || !ospf->oi_running)
+	} else
+		ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
+	if (ospf == NULL || !ospf->oi_running)
+		return CMD_SUCCESS;
+
+	vrf = vrf_lookup_by_id(ospf->vrf_id);
+	if (!vrf)
+		return CMD_SUCCESS;
+	if (idx_interface) {
+		ifp = if_lookup_by_name(
+					argv[idx_interface]->arg,
+					ospf->vrf_id);
+		if (ifp == NULL) {
+			vty_out(vty, "No such interface name in vrf %s\n",
+				vrf->name);
 			return CMD_SUCCESS;
-		vrf = vrf_lookup_by_id(ospf->vrf_id);
+		}
+	}
+	if (!ifp) {
 		FOR_ALL_INTERFACES (vrf, ifp)
 			show_mpls_te_link_sub(vty, ifp);
 		return CMD_SUCCESS;
 	}
-	/* Show All Interfaces. */
-	if (argc == 5) {
-		for (ALL_LIST_ELEMENTS_RO(om->ospf, node, ospf)) {
-			if (!ospf->oi_running)
-				continue;
-			vrf = vrf_lookup_by_id(ospf->vrf_id);
-			FOR_ALL_INTERFACES (vrf, ifp)
-				show_mpls_te_link_sub(vty, ifp);
-		}
-	}
-	/* Interface name is specified. */
-	else {
-		ifp = if_lookup_by_name_all_vrf(argv[idx_interface]->arg);
-		if (ifp == NULL)
-			vty_out(vty, "No such interface name\n");
-		else
-			show_mpls_te_link_sub(vty, ifp);
-	}
 
+	show_mpls_te_link_sub(vty, ifp);
 	return CMD_SUCCESS;
 }
 
