@@ -66,16 +66,13 @@ static bool mtrace_fwd_info_weak(struct pim_instance *pim,
 	struct pim_nexthop nexthop;
 	struct interface *ifp_in;
 	struct in_addr nh_addr;
-	int ret;
 	char nexthop_str[INET_ADDRSTRLEN];
 
-	nh_addr.s_addr = 0;
+	nh_addr.s_addr = INADDR_ANY;
 
 	memset(&nexthop, 0, sizeof(nexthop));
 
-	ret = pim_nexthop_lookup(pim, &nexthop, mtracep->src_addr, 1);
-
-	if (ret != 0) {
+	if (!pim_nexthop_lookup(pim, &nexthop, mtracep->src_addr, 1)) {
 		if (PIM_DEBUG_MTRACE)
 			zlog_debug("mtrace not found neighbor");
 		return false;
@@ -126,13 +123,20 @@ static bool mtrace_fwd_info(struct pim_instance *pim,
 	up = pim_upstream_find(pim, &sg);
 
 	if (!up) {
-		sg.src.s_addr = 0;
+		sg.src.s_addr = INADDR_ANY;
 		up = pim_upstream_find(pim, &sg);
 	}
 
 	if (!up)
 		return false;
 
+	if (!up->rpf.source_nexthop.interface) {
+		if (PIM_DEBUG_TRACE)
+			zlog_debug("%s: up %s RPF is not present", __func__,
+				   up->sg_str);
+		return false;
+	}
+
 	ifp_in = up->rpf.source_nexthop.interface;
 	nh_addr = up->rpf.source_nexthop.mrib_nexthop_addr.u.prefix4;
 	total = htonl(MTRACE_UNKNOWN_COUNT);
@@ -156,7 +160,7 @@ static bool mtrace_fwd_info(struct pim_instance *pim,
 	rspp->rtg_proto = MTRACE_RTG_PROTO_PIM;
 
 	/* 6.2.2. 4. Fill in ... S, and Src Mask */
-	if (sg.src.s_addr) {
+	if (sg.src.s_addr != INADDR_ANY) {
 		rspp->s = 1;
 		rspp->src_mask = MTRACE_SRC_MASK_SOURCE;
 	} else {
@@ -177,9 +181,9 @@ static void mtrace_rsp_set_fwd_code(struct igmp_mtrace_rsp *mtrace_rspp,
 static void mtrace_rsp_init(struct igmp_mtrace_rsp *mtrace_rspp)
 {
 	mtrace_rspp->arrival = 0;
-	mtrace_rspp->incoming.s_addr = 0;
-	mtrace_rspp->outgoing.s_addr = 0;
-	mtrace_rspp->prev_hop.s_addr = 0;
+	mtrace_rspp->incoming.s_addr = INADDR_ANY;
+	mtrace_rspp->outgoing.s_addr = INADDR_ANY;
+	mtrace_rspp->prev_hop.s_addr = INADDR_ANY;
 	mtrace_rspp->in_count = htonl(MTRACE_UNKNOWN_COUNT);
 	mtrace_rspp->out_count = htonl(MTRACE_UNKNOWN_COUNT);
 	mtrace_rspp->total = htonl(MTRACE_UNKNOWN_COUNT);
@@ -227,9 +231,7 @@ static void mtrace_debug(struct pim_interface *pim_ifp,
 	ra = mtracep->rsp_addr;
 
 	zlog_debug(
-		"Rx mtrace packet incoming on %s: "
-		"hops=%d type=%d size=%d, grp=%s, src=%s,"
-		" dst=%s rsp=%s ttl=%d qid=%ud",
+		"Rx mtrace packet incoming on %s: hops=%d type=%d size=%d, grp=%s, src=%s, dst=%s rsp=%s ttl=%d qid=%ud",
 		inet_ntop(AF_INET, &(pim_ifp->primary_address), inc_str,
 			  sizeof(inc_str)),
 		mtracep->hops, mtracep->type, mtrace_len,
@@ -251,8 +253,7 @@ static void mtrace_debug(struct pim_interface *pim_ifp,
 		if ((responses % sizeof(struct igmp_mtrace_rsp)) != 0)
 			if (PIM_DEBUG_MTRACE)
 				zlog_debug(
-					"Mtrace response block of wrong"
-					" length");
+					"Mtrace response block of wrong length");
 
 		responses = responses / sizeof(struct igmp_mtrace_rsp);
 
@@ -267,11 +268,10 @@ static uint32_t query_arrival_time(void)
 	struct timeval tv;
 	uint32_t qat;
 
-	char m_qat[] = "Query arrival time lookup failed: errno=%d: %s";
-
 	if (gettimeofday(&tv, NULL) < 0) {
 		if (PIM_DEBUG_MTRACE)
-			zlog_warn(m_qat, errno, safe_strerror(errno));
+			zlog_warn("Query arrival time lookup failed: errno=%d: %s",
+				  errno, safe_strerror(errno));
 		return 0;
 	}
 	/* not sure second offset correct, as I get different value */
@@ -355,17 +355,14 @@ static int mtrace_send_packet(struct interface *ifp,
 		if (sent < 0) {
 			if (PIM_DEBUG_MTRACE)
 				zlog_warn(
-					"Send mtrace request failed for %s on"
-					"%s: group=%s msg_size=%zd: errno=%d: "
-					" %s",
+					"Send mtrace request failed for %s on%s: group=%s msg_size=%zd: errno=%d:  %s",
 					dst_str, ifp->name, group_str,
 					mtrace_buf_len, errno,
 					safe_strerror(errno));
 		} else {
 			if (PIM_DEBUG_MTRACE)
 				zlog_warn(
-					"Send mtrace request failed for %s on"
-					" %s: group=%s msg_size=%zd: sent=%zd",
+					"Send mtrace request failed for %s on %s: group=%s msg_size=%zd: sent=%zd",
 					dst_str, ifp->name, group_str,
 					mtrace_buf_len, sent);
 		}
@@ -411,14 +408,11 @@ static int mtrace_un_forward_packet(struct pim_instance *pim, struct ip *ip_hdr,
 
 	if (interface == NULL) {
 		memset(&nexthop, 0, sizeof(nexthop));
-		ret = pim_nexthop_lookup(pim, &nexthop, ip_hdr->ip_dst, 0);
-
-		if (ret != 0) {
+		if (!pim_nexthop_lookup(pim, &nexthop, ip_hdr->ip_dst, 0)) {
 			close(fd);
 			if (PIM_DEBUG_MTRACE)
 				zlog_warn(
-					"Dropping mtrace packet, "
-					"no route to destination");
+					"Dropping mtrace packet, no route to destination");
 			return -1;
 		}
 
@@ -447,8 +441,7 @@ static int mtrace_un_forward_packet(struct pim_instance *pim, struct ip *ip_hdr,
 	if (sent < 0) {
 		if (PIM_DEBUG_MTRACE)
 			zlog_warn(
-				"Failed to forward mtrace packet:"
-				" sendto errno=%d, %s",
+				"Failed to forward mtrace packet: sendto errno=%d, %s",
 				errno, safe_strerror(errno));
 		return -1;
 	}
@@ -479,8 +472,7 @@ static int mtrace_mc_forward_packet(struct pim_instance *pim, struct ip *ip_hdr)
 	if (c_oil == NULL) {
 		if (PIM_DEBUG_MTRACE) {
 			zlog_debug(
-				"Dropping mtrace multicast packet "
-				"len=%u to %s ttl=%u",
+				"Dropping mtrace multicast packet len=%u to %s ttl=%u",
 				ntohs(ip_hdr->ip_len),
 				inet_ntoa(ip_hdr->ip_dst), ip_hdr->ip_ttl);
 		}
@@ -531,8 +523,7 @@ static int mtrace_send_mc_response(struct pim_instance *pim,
 	if (c_oil == NULL) {
 		if (PIM_DEBUG_MTRACE) {
 			zlog_debug(
-				"Dropping mtrace multicast response packet "
-				"len=%u to %s",
+				"Dropping mtrace multicast response packet len=%u to %s",
 				(unsigned int)mtrace_len,
 				inet_ntoa(mtracep->rsp_addr));
 		}
@@ -561,7 +552,6 @@ static int mtrace_send_response(struct pim_instance *pim,
 				struct igmp_mtrace *mtracep, size_t mtrace_len)
 {
 	struct pim_nexthop nexthop;
-	int ret;
 
 	mtracep->type = PIM_IGMP_MTRACE_RESPONSE;
 
@@ -592,13 +582,10 @@ static int mtrace_send_response(struct pim_instance *pim,
 	} else {
 		memset(&nexthop, 0, sizeof(nexthop));
 		/* TODO: should use unicast rib lookup */
-		ret = pim_nexthop_lookup(pim, &nexthop, mtracep->rsp_addr, 1);
-
-		if (ret != 0) {
+		if (!pim_nexthop_lookup(pim, &nexthop, mtracep->rsp_addr, 1)) {
 			if (PIM_DEBUG_MTRACE)
 				zlog_warn(
-					"Dropped response qid=%ud, no route to "
-					"response address",
+					"Dropped response qid=%ud, no route to response address",
 					mtracep->qry_id);
 			return -1;
 		}
@@ -615,7 +602,7 @@ int igmp_mtrace_recv_qry_req(struct igmp_sock *igmp, struct ip *ip_hdr,
 	static uint32_t qry_id, qry_src;
 	char mtrace_buf[MTRACE_HDR_SIZE + MTRACE_MAX_HOPS * MTRACE_RSP_SIZE];
 	struct interface *ifp;
-	struct interface *out_ifp;
+	struct interface *out_ifp = NULL;
 	struct pim_interface *pim_ifp;
 	struct pim_instance *pim;
 	struct igmp_mtrace *mtracep;
@@ -647,8 +634,7 @@ int igmp_mtrace_recv_qry_req(struct igmp_sock *igmp, struct ip *ip_hdr,
 	if (igmp_msg_len < (int)sizeof(struct igmp_mtrace)) {
 		if (PIM_DEBUG_MTRACE)
 			zlog_warn(
-				"Recv mtrace packet from %s on %s: too short,"
-				" len=%d, min=%zu",
+				"Recv mtrace packet from %s on %s: too short, len=%d, min=%zu",
 				from_str, ifp->name, igmp_msg_len,
 				sizeof(struct igmp_mtrace));
 		return -1;
@@ -665,8 +651,7 @@ int igmp_mtrace_recv_qry_req(struct igmp_sock *igmp, struct ip *ip_hdr,
 	if (recv_checksum != checksum) {
 		if (PIM_DEBUG_MTRACE)
 			zlog_warn(
-				"Recv mtrace packet from %s on %s: checksum"
-				" mismatch: received=%x computed=%x",
+				"Recv mtrace packet from %s on %s: checksum mismatch: received=%x computed=%x",
 				from_str, ifp->name, recv_checksum, checksum);
 		return -1;
 	}
@@ -690,8 +675,7 @@ int igmp_mtrace_recv_qry_req(struct igmp_sock *igmp, struct ip *ip_hdr,
 			if (IPV4_CLASS_DE(ntohl(ip_hdr->ip_dst.s_addr))) {
 				if (PIM_DEBUG_MTRACE)
 					zlog_debug(
-						"Dropping multicast query "
-						"on wrong interface");
+						"Dropping multicast query on wrong interface");
 				return -1;
 			}
 			/* Unicast query on wrong interface */
@@ -702,8 +686,7 @@ int igmp_mtrace_recv_qry_req(struct igmp_sock *igmp, struct ip *ip_hdr,
 		if (qry_id == mtracep->qry_id && qry_src == from.s_addr) {
 			if (PIM_DEBUG_MTRACE)
 				zlog_debug(
-					"Dropping multicast query with "
-					"duplicate source and id");
+					"Dropping multicast query with duplicate source and id");
 			return -1;
 		}
 		qry_id = mtracep->qry_id;
@@ -723,8 +706,7 @@ int igmp_mtrace_recv_qry_req(struct igmp_sock *igmp, struct ip *ip_hdr,
 	} else {
 		if (PIM_DEBUG_MTRACE)
 			zlog_warn(
-				"Recv mtrace packet from %s on %s: "
-				"invalid length %d",
+				"Recv mtrace packet from %s on %s: invalid length %d",
 				from_str, ifp->name, igmp_msg_len);
 		return -1;
 	}
@@ -734,8 +716,7 @@ int igmp_mtrace_recv_qry_req(struct igmp_sock *igmp, struct ip *ip_hdr,
 	    && !IPV4_MC_LINKLOCAL(ntohl(ip_hdr->ip_dst.s_addr))) {
 		if (PIM_DEBUG_MTRACE)
 			zlog_warn(
-				"Recv mtrace packet from %s on %s:"
-				" not link-local multicast %s",
+				"Recv mtrace packet from %s on %s: not link-local multicast %s",
 				from_str, ifp->name, inet_ntoa(ip_hdr->ip_dst));
 		return -1;
 	}
@@ -780,7 +761,7 @@ int igmp_mtrace_recv_qry_req(struct igmp_sock *igmp, struct ip *ip_hdr,
 
 	/* 6.2.2. 2. Attempt to determine the forwarding information... */
 
-	if (mtracep->grp_addr.s_addr)
+	if (mtracep->grp_addr.s_addr != INADDR_ANY)
 		fwd_info = mtrace_fwd_info(pim, mtracep, rspp, &out_ifp);
 	else
 		fwd_info = mtrace_fwd_info_weak(pim, mtracep, rspp, &out_ifp);
@@ -798,7 +779,7 @@ int igmp_mtrace_recv_qry_req(struct igmp_sock *igmp, struct ip *ip_hdr,
 
 	reached_source = false;
 
-	if (nh_addr.s_addr == 0) {
+	if (nh_addr.s_addr == INADDR_ANY) {
 		/* no pim? i.e. 7.5.3. No Previous Hop */
 		if (!out_ifp->info) {
 			if (PIM_DEBUG_MTRACE)
@@ -865,6 +846,15 @@ int igmp_mtrace_recv_response(struct igmp_sock *igmp, struct ip *ip_hdr,
 	pim_ifp = ifp->info;
 	pim = pim_ifp->pim;
 
+	if (igmp_msg_len < (int)sizeof(struct igmp_mtrace)) {
+		if (PIM_DEBUG_MTRACE)
+			zlog_warn(
+				"Recv mtrace packet from %s on %s: too short, len=%d, min=%zu",
+				from_str, ifp->name, igmp_msg_len,
+				sizeof(struct igmp_mtrace));
+		return -1;
+	}
+
 	mtracep = (struct igmp_mtrace *)igmp_msg;
 
 	recv_checksum = mtracep->checksum;
@@ -876,8 +866,7 @@ int igmp_mtrace_recv_response(struct igmp_sock *igmp, struct ip *ip_hdr,
 	if (recv_checksum != checksum) {
 		if (PIM_DEBUG_MTRACE)
 			zlog_warn(
-				"Recv mtrace response from %s on %s: checksum"
-				" mismatch: received=%x computed=%x",
+				"Recv mtrace response from %s on %s: checksum mismatch: received=%x computed=%x",
 				from_str, ifp->name, recv_checksum, checksum);
 		return -1;
 	}
